/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  App.h
 * Description: Header file for App.c
 *
 ************************************************************************
 */

#ifndef __APP_H__
#define __APP_H__

#include "isyncore.h"
#include "osdefs.h"
#include "messages.h"

#define noPreferenceFound -1

#if defined(UNDER_PO)
#  include "PalmExt.h"
#  define appFileCreator	'ISYN'
#  define appHSynCreator	'ISYL'
#  define appLogDbType	    'alog'

#elif defined(UNDER_EP)
#  include "EpocOsDefs.h"
#  define appFileCreator isy_T("isync")

#else
#  define appFileCreator isy_T("isync")
#endif

#define appPrefID			0x00	
#define appPrefVersionNum	0x01

#define ISYNC_VersionNum810 0x08010000 

// for log file 
#define appLogDbName        isy_T("logdb-isyn")

#define APPMB_OK            1
#define APPMB_OKCANCEL      2
#define APPMB_YESNO         3

#define APPMB_LOGIN         8

#ifdef UNDER_EP
#define MessageOKButton     1
#define MessageCancelButton 2
#endif

// default advanced setting
#define appDefaultDetailedLog       true // Detailed log
#define appDefaultHotSyncButton     false // Not to use the HotSync button to sync
#define appDefaultSavePassword      false // Don't save the password
#define appDefaultSSLButton         false // standard tranport
#define appDefaultTimeout           1 // minute
#define appDefaultMessageSize       0 // none
#define appDefaultTargetDir	        0 // RAM
#define appDropNetLib				0x02
#define appDefaultURL               isy_T("http://127.0.0.1:8080")

#define ISYNCPREF_User                   0x0008
#define ISYNCPREF_Password               0x0010
#define ISYNCPREF_UnsavedPassword        0x0020
#define ISYNCPREF_TimeoutSend            0x0040
#define ISYNCPREF_TimeoutRecv            0x0080
#define ISYNCPREF_Trace                  0x0100	//sync engine pref
#define ISYNCPREF_Message_Size         	 0x0200
#define ISYNCPREF_URL                    0x0400
#define ISYNCPREF_TargetDir              0x8000
#define ISYNCPREF_TargetSlot             0x8002
#define ISYNCPREF_Proxy                  0x8003

// Application synchronization mode
#define ISYNCMODE_Synchronize            0x0001
#define ISYNCMODE_Refresh                0x0002

// Sync engine information/status code for the Listener
#define ISYNCLEN_User                    256
#define ISYNCLEN_Password                56
#define ISYNCLEN_DB2eUser                19
#define ISYNCLEN_DB2ePassword            19
#define ISYNCLEN_Target                  100
#define ISYNCLEN_TimeoutSend             ISCLEN_Timeout
#define ISYNCLEN_TimeoutRecv             ISCLEN_Timeout
#define	ISYNCLEN_MessageSize			 20

#define ISYNCLEN_Server                  64
#define ISYNCLEN_Port		             8
#define ISYNCLEN_Dest		             50
#define ISYNCLEN_URL		             513
// config object/targetdir support
#define ISYNCLEN_TargetDir               100
#define ISYNCLEN_MiscInfo                256

typedef isy_VOID* HISYNCLOG;

typedef struct
{
	isy_TCHAR URL[ISYNCLEN_URL]; // URL
	isy_TCHAR user[ISYNCLEN_User];
	isy_TCHAR password[ISYNCLEN_Password];
} ISYNCPREFS;

typedef struct 
{
	isy_UINT32 num;
	isy_UINT16 prefsSize;
} AppPrefsVersion;

typedef struct 
{
	AppPrefsVersion version;
	isy_BYTE   reserved1[24];
	isy_BYTE   options; // savePassword, trace

	isy_TCHAR  reserved2[20];
	long       reserved3;
	long       timeoutSend;
	long       timeoutReceive;
	isy_TCHAR  reserved4[56];
	isy_TCHAR  user[ISYNCLEN_User];
	isy_TCHAR  password[ISYNCLEN_Password];
	
	isy_TCHAR  dest[ISYNCLEN_Dest];
	isy_TCHAR  reserved5[64];
	isy_TCHAR  reserved6[8];
	
	isy_TCHAR  targetDir[ISYNCLEN_TargetDir];
	isy_DWORD  flag;	// 0x01 update application list
						// 0x02 drop Net.lib after sync				
} V8B2AppPrefs;
//end v8 beta2 for rushmore ga

typedef struct 
{
	isy_UINT32 version;
	isy_UINT16 prefsSize;
	isy_BYTE   reserved1[24];
	isy_BYTE   options; // savePassword, trace, proxy

	isy_TCHAR  reserved2[20];
	long       reserved3;
	long       reserved4;
	long       timeoutSend;
	long       timeoutReceive;
	isy_TCHAR  reserved5[56];
	isy_TCHAR  user[ISYNCLEN_User];
	isy_TCHAR  password[ISYNCLEN_Password];
	
	isy_TCHAR  reserved6[50];
	isy_TCHAR  reserved7[30];
	isy_TCHAR  dest[ISYNCLEN_Dest];
	
	isy_TCHAR  reserved8[100];
	isy_TCHAR  reserved9[100];
	isy_TCHAR  reserved10[20];
	isy_TCHAR  targetDir[ISYNCLEN_TargetDir];
	isy_DWORD  flag;	// 0x01 update application list
						// 0x02 drop Net.lib after sync		
	isy_TCHAR  targetSlot[2];				
	isy_UINT32 messageSize;

	isy_TCHAR  proxy[ISYNCLEN_Server+ISYNCLEN_Port];
	isy_TCHAR  URL[ISYNCLEN_URL]; // URL
} AppPrefs;

typedef struct
{
	char	sessionPassword[ISYNCLEN_Password];
	isy_BOOL	canceled;

#ifdef UNDER_PO
	isy_BOOL	mute; 
	isy_BOOL	hotSync; 
#endif // UNDER_PO
	
	isy_BOOL	errorCount; // Keep track on # of errors  d4317 JSY (9/26/2001)
	
	HISCSERV	hSyncServ;
	HISCCONF	hSyncConf;
	HISCENG		hSyncer;
	HISYNCLOG 	hLog;

	AppPrefs    prefs;

} APPDATA;


#define appPrefVersion(pApp)            ((pApp)->prefs.version)
#define appPrefSize(pApp)	            ((pApp)->prefs.prefsSize)
#define appPrefSendTimeout(pApp)        ((pApp)->prefs.timeoutSend)
#define appPrefRecvTimeout(pApp)        ((pApp)->prefs.timeoutReceive)
#define appPrefUser(pApp)               ((pApp)->prefs.user)
#define appPrefPassword(pApp)           ((pApp)->prefs.password)
#define appPrefDest(pApp)               ((pApp)->prefs.dest)
#define appPrefURL(pApp)                ((pApp)->prefs.URL)
#define appPrefTargetDir(pApp)          ((pApp)->prefs.targetDir)
#define appPrefFlag(pApp)               ((pApp)->prefs.flag)
#define appPrefTargetSlot(pApp)         ((pApp)->prefs.targetSlot)
#define appPrefMessageSize(pApp)        ((pApp)->prefs.messageSize)
#define appPrefProxy(pApp)              ((pApp)->prefs.proxy)

#define neverTimeout(sec)				((sec) == -1)
//max of 8 options
#define ISYNCOPTION_SavePassword		0x01
#define ISYNCOPTION_Trace				0x02
#define ISYNCOPTION_SSL 				0x04
#define ISYNCOPTION_PROXY 				0x08

#define prefOptions(prefs)              (prefs.options)
#define prefResetOptions(prefs)         (prefs.options = 0)

#define prefOptionTrace(prefs)          (prefs.options & ISYNCOPTION_Trace)
#define prefSetOptionTrace(prefs)       (prefs.options |= ISYNCOPTION_Trace)
#define prefResetOptionTrace(prefs)     (prefs.options &= (~ISYNCOPTION_Trace))

#define prefOptionSavePwd(prefs)        (prefs.options & ISYNCOPTION_SavePassword)
#define prefSetOptionSavePwd(prefs)     (prefs.options |= ISYNCOPTION_SavePassword)
#define prefResetOptionSavePwd(prefs)   (prefs.options &= (~ISYNCOPTION_SavePassword))

#define prefOptionSSL(prefs)            (prefs.options & ISYNCOPTION_SSL)
#define prefSetOptionSSL(prefs)         (prefs.options |= ISYNCOPTION_SSL)
#define prefResetOptionSSL(prefs)       (prefs.options &= (~ISYNCOPTION_SSL))

#define prefOptionProxy(prefs)          (prefs.options & ISYNCOPTION_PROXY)
#define prefSetOptionProxy(prefs)       (prefs.options |= ISYNCOPTION_PROXY)
#define prefResetOptionProxy(prefs)     (prefs.options &= (~ISYNCOPTION_PROXY))

#define appPrefs(pApp)                  ((pApp)->prefs)
#define appResetPrefOptions(pApp)       prefResetOptions(appPrefs(pApp))

#define appPrefOptionTrace(pApp)        prefOptionTrace(appPrefs(pApp))
#define appSetPrefOptionTrace(pApp)     prefSetOptionTrace(appPrefs(pApp))
#define appResetPrefOptionTrace(pApp)   prefResetOptionTrace(appPrefs(pApp))

#define appPrefOptionSavePwd(pApp)      prefOptionSavePwd(appPrefs(pApp))
#define appSetPrefOptionSavePwd(pApp)   prefSetOptionSavePwd(appPrefs(pApp))
#define appResetPrefOptionSavePwd(pApp) prefResetOptionSavePwd(appPrefs(pApp))

#define appPrefOptionSSL(pApp)          prefOptionSSL(appPrefs(pApp))
#define appSetPrefOptionSSL(pApp)       prefSetOptionSSL(appPrefs(pApp))
#define appResetPrefOptionSSL(pApp)     prefResetOptionSSL(appPrefs(pApp))

#define appPrefOptionProxy(pApp)        prefOptionProxy(appPrefs(pApp))
#define appSetPrefOptionProxy(pApp)     prefSetOptionProxy(appPrefs(pApp))
#define appResetPrefOptionProxy(pApp)   prefResetOptionProxy(appPrefs(pApp))

#ifdef __cplusplus
extern "C" {
#endif

int      appReadPreferences();
int      appSavePreferences();
isy_INT  appGetPref(isy_INT prefID, isy_TCHAR *prefVal, const isy_INT prefLen);
isy_INT  appSetPref(isy_INT prefID, const isy_TCHAR *prefVal);
int      syncStartup(iscEngineListenerPF pSyncListener);
void     syncShutdown();
void     appInitialize();
void     appTerminate();

void appVersionToString(APPDATA *pApp, isy_TCHAR *verString);
void appLogBuildVersion(APPDATA *pApp);
void appLogWrite(APPDATA *pApp, isy_INT32 eventCode, isy_INT32 eventType, const isy_TCHAR* msg);
const isy_TCHAR* appEventCodeToMessage(isy_INT32 eventCode);
const isy_TCHAR* appCurrentDateTime(isy_TCHAR* buf, isy_TCHAR* formatString);
isy_BOOL appMessageBox(const isy_TCHAR* pMsg, int iType);
int      appMessageBox2(const isy_TCHAR* pMsg, int iType, long timeout, int retTimeout);

extern APPDATA app;

#ifdef __cplusplus
}
#endif

#if defined(UNDER_PO) || defined(UNDER_NT) || defined(UNDER_CE)
#  define GET_APP_ APPDATA* _pApp = &app;
#elif defined(UNDER_EP)
#  define GET_APP_ APPDATA* _pApp = (APPDATA*) (epocGetGlobals())->hApp;	
#endif


#endif
