/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  logfile.c
 * Description: GUI API to the log file
 *
 ************************************************************************
 */

#include "app.h"
#include "logfile.h"

#ifdef UNDER_EP
#include "EpocOsDefs.h"
#endif

//________________________________________________________________________
//
// Basic log API: logOpen / logClose / logPurge / logWrite / logFileHandle
//________________________________________________________________________

HISYNCLOG logOpen(APPDATA* pApp)
{
	isy_TCHAR fileName[128];
	HOBJECT hl = MEM_HANDLE_ALLOC(sizeof(LOGDATA));
	LOGDATA* pl = (LOGDATA *) MEM_HANDLE_LOCK(hl);
	GET_APP_

	pApp->hLog = pl;
	pl->hThis = hl;
	pl->pApp = pApp;

	TSTR_CPY(fileName, DATADIR);
	//TSTR_CPY(fileName, appPrefTargetDir(_pApp));
	TSTR_CAT(fileName, appLogDbName);
#ifdef UNDER_PO
	{
	LocalID localID = DmFindDatabase(0, appLogDbName);
	pl->logFile = (localID ? DmOpenDatabase(0, localID, dmModeReadWrite) : NULL);
	}
#else
	pl->logFile = FOPEN(fileName, isy_T("ab+"));
#endif // UNDER_PO

	if (pl->logFile == NULL) {
#ifdef UNDER_PO
		Err err = DmGetLastErr();
		if (err == dmErrCantFind) {
			err = DmCreateDatabase(0, appLogDbName, appFileCreator, appLogDbType, false);
			if (err == 0) {
				LocalID dbID = 0;
				pl->logFile = DmOpenDatabaseByTypeCreator(appLogDbType, appFileCreator, dmModeReadWrite);
				dbID = DmFindDatabase(0, appLogDbName);
				if (dbID) {
					isy_UINT16 attr;
					DmDatabaseInfo(0, dbID, NULL, &attr, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
					attr |= dmHdrAttrBackup;
					DmSetDatabaseInfo(0, dbID, NULL, &attr, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
				}
			}
		}
#else
		pl->logFile = FOPEN(fileName, isy_T("wb+"));
#endif // UNDRR_PO

		if (pl->logFile != NULL) {
			logCodePage(pApp->hLog);
		}
	}

	if (pl->logFile == NULL) {
		pApp->hLog = NULL;
		MEM_HANDLE_UNLOCK(hl);
		MEM_HANDLE_FREE(hl);
		pl = NULL;
	}

	return (HISYNCLOG) pl;
}

void logClose(HISYNCLOG hl)
{
	HOBJECT hMem = ((LOGDATA*) hl)->hThis;

#ifdef UNDER_PO
	if (((LOGDATA*) hl)->logFile) DmCloseDatabase(((LOGDATA*) hl)->logFile);
#else
	FCLOSE( ((LOGDATA*)hl)->logFile );
#endif
	MEM_HANDLE_UNLOCK(hMem);
	MEM_HANDLE_FREE(hMem);
}

void logWrite(HISYNCLOG hl, const isy_TCHAR* msg)
{
	logWriteN(hl, msg, TSTR_LEN(msg));
}

HISYNCFILE logFileHandle(HISYNCLOG hl)
{
	return ((LOGDATA*) hl)->logFile;
}


//________________________________________________________________________
//
// Platform-dependent functions: logPurge / logWriteN
//________________________________________________________________________

#ifdef UNDER_PO
void logPurge(HISYNCLOG hl) 
{
	Err err;
	UInt numRecords = DmNumRecordsInCategory(((LOGDATA*) hl)->logFile, dmAllCategories);
	while (numRecords--) 
	{
		err = DmRemoveRecord(((LOGDATA*) hl)->logFile, numRecords);
		if (err) break;
	}
} // logPurge UNDER_PO

void logWriteN(HISYNCLOG hl, const isy_TCHAR* msg, long len)
{
	UInt recordNum = dmMaxRecordIndex;
	VoidHand hRecord;
	char* p;
	ULong cb;
	
	if (len == 0)
	{
		len = 1;
		msg = isy_T(" ");
	}

	cb = (ULong) len;
	hRecord = DmNewRecord(((LOGDATA*) hl)->logFile, &recordNum, cb);
	p = MemHandleLock(hRecord);
	DmWrite(p, 0, msg, cb);
	MemHandleUnlock(hRecord);
	DmReleaseRecord(((LOGDATA*) hl)->logFile, recordNum, true);
} // logWriteN UNDER_PO

#else // non PalmOS
void logPurge(HISYNCLOG hl) 
{
	isy_TCHAR fileName[128];
	GET_APP_

	// if (!hl) return;

	TSTR_CPY(fileName, DATADIR);
	//TSTR_CPY(fileName, appPrefTargetDir(_pApp));
	TSTR_CAT(fileName, appLogDbName);

	FCLOSE( ((LOGDATA*)hl)->logFile );
	((LOGDATA*)hl)->logFile = FOPEN(fileName, isy_T("wb+"));

	if (((LOGDATA*)hl)->logFile == NULL)
		return;

	logCodePage(hl);
} // logPurge not UNDER_PO

void logWriteN(HISYNCLOG hl, const isy_TCHAR* msg, long len)
{
	FWRITE(msg,	sizeof(isy_TCHAR), len, ((LOGDATA*)hl)->logFile);
	
	FWRITE(FILE_NL, sizeof(isy_TCHAR), TSTR_LEN(FILE_NL), ((LOGDATA*)hl)->logFile); // CRLF
	
	FFLUSH(((LOGDATA*)hl)->logFile);
} // logWriteN not UNDER_PO

#endif // UNDER_PO


//________________________________________________________________________
//
// Extended log functions
//________________________________________________________________________

/*
void logStatistics(HISYNCLOG hl, const isy_TCHAR* msg)
{
	isy_TCHAR buf[40];

	if (msg) logWrite(hl, msg);

	TSTR_CPY(buf, isy_T("Total Time: "));	
	TSTR_ITOA(buf + TSTR_LEN(buf), ((LOGDATA*) hl)->pApp->totalTime);
	logWrite(hl, buf);
	
	TSTR_CPY(buf, isy_T("Send Time: "));	
	TSTR_ITOA(buf + TSTR_LEN(buf), ((LOGDATA*) hl)->pApp->sendTime);
	logWrite(hl, buf);

	TSTR_CPY(buf, isy_T("Rcv Time: "));	
	TSTR_ITOA(buf + TSTR_LEN(buf), ((LOGDATA*) hl)->pApp->receiveTime);
	logWrite(hl, buf);
}
*/

void logWriteSystemStatus(HISYNCLOG hl, const isy_TCHAR* msg)
{
#ifdef UNDER_PO
	ULong free1 = 0, max1 = 0;
	char buf[30];
	MemHeapFreeBytes(0, &free1, &max1);

	if (msg) logWrite(hl, msg);

	STR_CPY(buf, "Free: ");	
	STR_ITOA(buf + STR_LEN(buf), free1);
	logWrite(hl, buf);

	STR_CPY(buf, "Max: ");	
	STR_ITOA(buf + STR_LEN(buf), max1);
	logWrite(hl, buf);
#endif // UNDER_PO
}

void logCodePage(HISYNCLOG hl)
{
/*
Byte-order mark		Description 
EF BB BF			UTF-8 
FE FF				UTF-16/UCS-2, little endian 
FF FE				UTF-16/UCS-2, big endian 
FF FE 00 00			UTF-32/UCS-4, little endian. 
00 00 FE FF			UTF-32/UCS-4, big-endian. 
*/
#ifdef _UNICODE
	FWRITE("\xFF", 1, 1, ((LOGDATA*)hl)->logFile);
	FWRITE("\xFE", 1, 1, ((LOGDATA*)hl)->logFile);
	FFLUSH(((LOGDATA*)hl)->logFile);
#endif
}
