/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  PalmExt.c
 * Description: Palm-specific GUI routines
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#endif
#include "StarterRsc.h"
#include "PalmExt.h"

#include "app.h"


Word extFrmDoDialog(FormPtr frm, long waitBeforeDismiss, Word retOnDismiss)
{
	Word error;
	Word buttonHit;
	EventType event;
	FormActiveStateType	curFrmState;
	long ticks;
	
	if (waitBeforeDismiss == -1)
		waitBeforeDismiss = evtWaitForever;
	else
		waitBeforeDismiss *= SysTicksPerSecond();
	
	FrmSaveActiveState(&curFrmState);		// save active form/window state

	FrmSetActiveForm (frm);
	if (! FrmVisible(frm))
		FrmDrawForm (frm);
	FrmSetFocus (frm, FrmGetFocus(frm));

	ticks = (long) TimGetTicks();
	while (true)
		{
		EvtGetEvent (&event, waitBeforeDismiss);
		if ((event.eType == nilEvent || event.eType == keyDownEvent) && waitBeforeDismiss != evtWaitForever
			&& ((long) TimGetTicks()) - ticks > waitBeforeDismiss)
		{
			buttonHit = retOnDismiss;
			break;
		}
		
		if (frm->handler)
			{
			if (frm->handler (&event))
				continue;
			}
			
//		if (! SysHandleEvent (&event))
			if (! MenuHandleEvent (0, &event, &error))
				FrmHandleEvent (frm, &event);

		if (event.eType == ctlSelectEvent)
			{
			
			if (((ControlPtr)(event.data.ctlSelect.pControl))->style == buttonCtl)
				{
				buttonHit = event.data.ctlSelect.controlID;
				break;
				}
			}


		else if (event.eType == appStopEvent)
			{
			buttonHit = frm->defaultButton;
			EvtAddEventToQueue (&event);
			break;
			}
		}

	FrmEraseForm (frm);
	FrmRestoreActiveState(&curFrmState);		// restore active form/window state
	
	return (buttonHit);
}

Word extMessageBoxPtr(char* pMsg, UInt uFlags, long waitBeforeDismiss, Word retOnDismiss)
{
	return extMessageBox(getStringCopy(pMsg), uFlags, waitBeforeDismiss, retOnDismiss);
}

Word extMessageBox(VoidHand hText, UInt uFlags, long waitBeforeDismiss, Word retOnDismiss)
{
	FormPtr prevForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(MessageForm);
	FieldPtr fld;
	VoidHand hOldText;
	Word w;

//	WinEraseWindow();
	FrmSetActiveForm(frm);
	FrmDrawForm(frm);
	
	setCtrlLabel(frm, MessageOKButton, ISYNCSTR_OK);
	setCtrlLabel(frm, MessageCancelButton, ISYNCSTR_Cancel);

	if (!(uFlags & (EXTMB_OK | EXTMB_YESNO))) 
		FrmHideObject(frm, FrmGetObjectIndex(frm, MessageOKButton));
	if (!(uFlags & (EXTMB_CANCEL | EXTMB_YESNO))) 
		FrmHideObject(frm, FrmGetObjectIndex(frm, MessageCancelButton));

	fld = FrmGetObjectPtr(frm, FrmGetObjectIndex(frm, MessageMessageField));
	hOldText = FldGetTextHandle(fld);
	FldSetTextHandle(fld, hText);
	FldDrawField(fld);

	if (hOldText) MemHandleFree(hOldText);
	
	w = extFrmDoDialog(frm, waitBeforeDismiss, retOnDismiss);
	FrmDeleteForm(frm);
	
//	WinEraseWindow();

	if (prevForm) 
	{	
		FrmSetActiveForm(prevForm);
		FrmDrawForm(prevForm);
	}

	return w;
}

VoidHand getStringCopy(const char* sz)
{
	VoidHand hCopy = MemHandleNew(StrLen(sz) + 1);
	char* pCopy = (char*) MemHandleLock(hCopy);
	StrCopy(pCopy, sz);
	MemHandleUnlock(hCopy);
	return hCopy;
}

VoidPtr GetObjectPtr(FormPtr formP, Word objectID)
{
	FormPtr frmP;

	if ( formP != NULL )
		frmP = formP;
	else
		frmP = FrmGetActiveForm();
		
	return FrmGetObjectPtr(frmP, FrmGetObjectIndex(frmP, objectID));
}
