/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  SyncForm.c
 * Description: Synchronization monitoring
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#endif
#include <SysEvtMgr.h>
#include <SoundMgr.h>  
#include <NetMgr.h>
#include "StarterRsc.h"

#include "app.h"
#include "MainForm.h"
#include "SyncForm.h"
#include "logfile.h"

#ifdef UNDER_PO
#define GET_APP_ APPDATA* _pApp = &app;
#endif

#ifdef UNDER_EP
#define GET_APP_	APPDATA* _pApp = (APPDATA*) (epocGetGlobals())->hApp;	
#endif

static void syncLoop();
static isy_INT32 syncListener(isy_UINT32 listenerData, ISCEVT* event, isy_VOID* pExtraInfo);
#define ISYNC_TUNE_MASK     0x000000FF
#define ISYNC_TUNE_START    0x005E6259 // { 89, 98, 94 }
#define ISYNC_TUNE_END      0x0059625E // { 94, 98, 89 }
#define ISYNC_TUNE_CANCEL   0x00625962 // { 98, 89, 98 }

#define LINE_SIZE 28

static void PlayTune(int note);
static void GenTune(isy_UINT32 se);

static void formInit(FormPtr frmP)
{
    FrmCopyTitle(frmP, ISYNCSTR_DB2Sync);
    setCtrlLabel(frmP, SyncCancelButton, ISYNCSTR_Cancel);
}

static Boolean formDoCommand(Word command)
{
	Boolean handled = false;

	switch (command) {
	default:
		break;
	}
	
	return handled;
}

Boolean cancelPressed(long wait)
{
	EventType event;
	GET_APP_
	
// commented out - didn't seem to work on emulator
//	if (EvtSysEventAvail(false)) 
	{
		while (1)
		{
			EvtGetEvent(&event, wait);
			if (event.eType == nilEvent) break;
			if (event.eType == keyDownEvent) break;
				
			if (!SyncFormHandleEvent(&event))
				FrmHandleEvent(FrmGetActiveForm(), &event);

			if (_pApp->canceled) return true;
		}
	}
	
	return _pApp->canceled;

}

static void syncLoop()
{
	GET_APP_
	int rc = 0;
	UInt prevTimeOut;
	
	rc = syncStartup(syncListener);
	if (rc == ISCRTN_Succeeded) 
	{		
		_pApp->canceled = false;
		_pApp->errorCount = 0;
		
		logPurge(_pApp->hLog);
		appLogBuildVersion(_pApp);

		// Turn off the AutoOff feature temporarily		
		prevTimeOut = SysSetAutoOffTime(0);

		iscEngineSync(_pApp->hSyncer);	

		// Restore the AutoOff feature		
		EvtResetAutoOffTimer();
		SysSetAutoOffTime(prevTimeOut);
     	
		if (_pApp->sessionPassword[0])
		{
			ISYNCPREFS prefs;
			prefs.password[0] = '\0';
			appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
		}
	}
	else
	{
		isy_TCHAR statusMsg[128];
		statusMsg[0] = '\0';
		TSTR_CPY(statusMsg, ISYNCSTR_StartupError);
		TSTR_ITOA(statusMsg + TSTR_LEN(statusMsg), rc);
		appMessageBox(statusMsg, APPMB_OK);
	}
		

	syncShutdown();
	FrmGotoForm(MainForm);
}


Boolean SyncFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
    FormPtr frmP;
    GET_APP_

	switch (eventP->eType) {
	case menuEvent:
		return formDoCommand(eventP->data.menu.itemID);

	case frmOpenEvent:
		frmP = FrmGetActiveForm();
		formInit(frmP);
		FrmDrawForm(frmP);
		if (!(_pApp->mute))
		   GenTune(ISYNC_TUNE_START); 
		handled = true;
		syncLoop();
		break;
	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID) {
		case SyncCancelButton:
			_pApp->canceled = true;
			handled = true;
			if (!(_pApp->mute))
			   GenTune(ISYNC_TUNE_CANCEL); 
			break;
		}
		break;
	default:
		break;
	
	}
	
	return handled;
}

void showStatus(const char* pMsg, int box)
{
	char *tempCopy;
	char temp;
	int chopSize = LINE_SIZE;
	int i;
	FormPtr frmP = FrmGetActiveForm();
	Word w;

	if(box == 1)	
		w = FrmGetObjectIndex(frmP, SyncStatus1Label);
	else 
		w = FrmGetObjectIndex(frmP, SyncStatus2Label);		
	FrmHideObject(frmP, w);
	
	if(box == 1)
		w = FrmGetObjectIndex(frmP, SyncStatus1aLabel);
	else
		w = FrmGetObjectIndex(frmP, SyncStatus2aLabel);
	FrmHideObject(frmP, w);
		
	if(StrLen(pMsg) > LINE_SIZE)
	{
		tempCopy = MemPtrNew(StrLen(pMsg) + 1);
		if(tempCopy)
		{

			StrCopy(tempCopy, pMsg);		
			
			//Need to figure out where to chop first
			for(i=LINE_SIZE; i >= 0; i--)
			{
				if(tempCopy[i] == ' ')
				{
					chopSize = i;
					break;
				}
			}
			temp = tempCopy[chopSize];
			tempCopy[chopSize] = '\0';		
			if(box == 1)
				FrmCopyLabel(frmP, SyncStatus1Label, (char*) tempCopy);
			else
				FrmCopyLabel(frmP, SyncStatus2Label, (char*) tempCopy);
			tempCopy[chopSize] = temp;
			if(box == 1)
				FrmCopyLabel(frmP, SyncStatus1aLabel, (char*)(tempCopy + chopSize));		
			else
				FrmCopyLabel(frmP, SyncStatus2aLabel, (char*)(tempCopy + chopSize));		
			MemPtrFree(tempCopy);
		}
		
		FrmShowObject(frmP, w);
		if(box == 1)
			w = FrmGetObjectIndex(frmP, SyncStatus1Label);			
		else
			w = FrmGetObjectIndex(frmP, SyncStatus2Label);
		FrmShowObject(frmP, w);
		
	}
	else	
	{
		if(box == 1)
		{
			FrmCopyLabel(frmP, SyncStatus1Label, (char*) pMsg);		
			w = FrmGetObjectIndex(frmP, SyncStatus1Label);
		}
		else
		{
			FrmCopyLabel(frmP, SyncStatus2Label, (char*) pMsg);		
			w = FrmGetObjectIndex(frmP, SyncStatus2Label);
		}
			
		FrmShowObject(frmP, w);
	}
}

void showStatus1(const char* pMsg)
{
	showStatus(pMsg, 1);			
}

void showStatus2(const char* pMsg)
{
	showStatus(pMsg, 2);			
}

void updateProgress(int percent)
{
	char buf[128] = {0};
	
	FormPtr frmP = FrmGetActiveForm();
	Word w = FrmGetObjectIndex(frmP, SyncProgressLabel);
	
	StrCopy(buf, ISYNCSTR_Synchronize);
	StrCat(buf, " ");
	StrIToA(buf + StrLen(buf), percent);
	StrCat(buf, "%");
	FrmHideObject(frmP, w);
	FrmCopyLabel(frmP, SyncProgressLabel, buf);
	FrmShowObject(frmP, w);
}



/*
   
   Frequency is the speed of vibration
   Note is a given name to describe a musical frequency
   Octaves of a note are multiples of the original frequency 

               Calculation for Equal-Tempered tuning (A3 = 440Hz) 
               --------------------------------------------------
               
  Note         Octave=0   Octave=1   Octave=2   Octave=3   Octave=4 
  ------------------------------------------------------------------
  A              55.000    110.000    220.000    440.000    880.000
  A#/Bb          58.270    116.541    233.082    466.164    932.328
  B              61.735    123.471    246.942    493.883    987.767
  C              65.406    130.813    261.626    523.251   1046.502 
  C#/Db          69.296    138.591    277.183    554.365   1108.731
  D              73.416    146.832    293.665    587.330   1174.659
  D#/Eb          77.782    155.563    311.127    622.254   1244.508
  E              82.407    164.814    329.628    659.255   1318.510
  F              87.307    174.614    349.228    698.456   1396.913
  F#/Gb          92.499    184.997    369.994    739.989   1479.978
  G              97.999    195.998    391.995    783.991   1567.982
  G#/Ab         103.826    207.652    415.305    830.609   1661.219
  A             110.000    220.000    440.000    880.000   1760.000
 
 */
 
static void PlayTune(int note)
{
   SndCommandType cmd;
   int   octv;
   float freq;
   float logTbl[] = { 1.00000, 1.05945, 1.12245, 1.18920, 1.25993, 1.33484, 1.41422, 1.49831, 1.58740, 1.68180, 1.78180, 1.88775 };
   
   octv   = note / 12;
   note   = note - (octv * 12);
   freq   = logTbl[note] * 55;
   
   while (octv-- > 3) freq *= 2;
      
   cmd.cmd    = sndCmdFreqDurationAmp;
   cmd.param1 = freq;
   cmd.param2 = 250;
   cmd.param3 = sndMaxAmp;
   
   // play synchronous
   SndDoCmd(0, &cmd, 0);
}

static void GenTune(isy_UINT32 se) {
	int i = 0;
	int note;
	
	do {
		note = (int) (se & ISYNC_TUNE_MASK);
		se = se >> 8; 
		if (note == 0) break;
		PlayTune(note);
	} while (++i < 4);
}


/******************************************************************/
/**
 * This function defines the sync listener. See isyncore.h for more
 * information.
 * param: listenerData, your personal data.
 * param: event, event object
 * param: pExtraInfo (reserved)
 * return: integer, when event type is ISYNCEVTTYPE_Retry:
 *                  . ISYNCRTNCB_ReplyYes    : retry less than 3 times
 *                  . ISYNCRTNCB_ReplyNo     : retry more than or equal to 3 times
 *                  when event type is ISYNCEVTTYPE_Info or ISYNCEVTTYPE_Conflict:
 *                  . ISYNCRTNCB_Done        : don't care
 *                  other event types
 *                  . ISYNCRTNCB_Default
 **/

static isy_INT32 syncListener(isy_UINT32 listenerData, ISCEVT* event, isy_VOID* pExtraInfo)
{
	GET_APP_

	isy_TCHAR buf[256];
	isy_TCHAR *statusMsg = (isy_TCHAR *) appEventCodeToMessage(event->code);

	updateProgress(event->state.syncProg);			
	
	switch (event->type) {
		case ISCEVTTYPE_Retry: // RetryOkCancel
		{
			long timeout = 60;
			int timeoutReturnValue = ISCRTNCB_ReplyNo;
			int rc = appMessageBox2(statusMsg, APPMB_OKCANCEL, timeout, timeoutReturnValue);
			
			if(rc == 1) //yes pressed
				return ISCRTNCB_ReplyYes;
			else 
				return ISCRTNCB_ReplyNo;				
		} 
		break;
		
		case ISCEVTTYPE_Query: // RetryOkCancel
		{
			switch (event->code) 
			{
				case ISCEVT_QueCancel:					
					if (cancelPressed(1))
						return ISCRTNCB_ReplyYes;
					else
						return ISCRTNCB_ReplyNo;
				break;
				case ISCEVT_QueCancelUponError:
				{	long timeout = 60;
					int timeoutReturnValue = ISCRTNCB_ReplyNo;
					// Ask if continue with the next subscription set upon the occurrence of an error
					int rc = appMessageBox2(statusMsg, APPMB_OKCANCEL, timeout, timeoutReturnValue);
					
					if(rc == 1) // OK pressed, continue
						return ISCRTNCB_ReplyNo; // Don't cancel
					else 
						return ISCRTNCB_ReplyYes; // Cancel
				}
				break;
				
				case ISCEVT_QueLogin:
				{				
					ISCLISTENARG *args = event->info;
					Word hitButton;
					Handle h = NULL;
					Handle newh = NULL;
					Handle oldh = NULL;
					CharPtr p;
					FieldPtr fld;
					FormPtr prevForm = FrmGetActiveForm();
					FormPtr localEncFrm = FrmInitForm(LocalEncryptionForm);	
					
					FrmCopyTitle(localEncFrm, ISYNCSTR_DB2e_Authen);
					setCtrlLabel(localEncFrm, LocalEncryptionOKButton, ISYNCSTR_OK);
					setCtrlLabel(localEncFrm, LocalEncryptionCancelButton, ISYNCSTR_Cancel);
					setLabel(localEncFrm, LocalEncryptionUnnamed2303Label, ISYNCSTR_Target);
					setLabel(localEncFrm, LocalEncryptionUnnamed2306Label, ISYNCSTR_DB2e_User);
					setLabel(localEncFrm, LocalEncryptionUnnamed2307Label, ISYNCSTR_DB2e_Password);

					
					FrmSetActiveForm(localEncFrm);
					FrmDrawForm(localEncFrm);				
					fld = GetObjectPtr(localEncFrm, LocalEncryptionTargetField);
					
					newh = MEM_HANDLE_ALLOC((TSTR_LEN(args->argv[0]) + 1) * sizeof(isy_TCHAR));
					oldh = FldGetTextHandle(fld);
					p = (isy_TCHAR *) MEM_HANDLE_LOCK(newh);				
					TSTR_CPY(p, args->argv[0]);
					MEM_HANDLE_UNLOCK(newh);
					
					FldSetTextHandle(fld, newh);
					FldDrawField(fld);
					
					if (oldh) MEM_HANDLE_FREE(oldh);
					
					hitButton = FrmDoDialog(localEncFrm);
					
					if(hitButton == LocalEncryptionOKButton)
					{
						h = FldGetTextHandle(GetObjectPtr(localEncFrm, LocalEncryptionUsernameField));
						p = h ? ((char*) MEM_HANDLE_LOCK(h)) : "";
						if(h != NULL)
						{
							STR_N_CPY(args->argv[1], p, STR_LEN(p));
							(args->argv[1])[STR_LEN(p)] = '\0';
						}
							
						if (h) 
							MEM_HANDLE_UNLOCK(h);
									
						h = FldGetTextHandle(GetObjectPtr(localEncFrm, LocalEncryptionPasswordField));
						p = h ? ((char*) MEM_HANDLE_LOCK(h)) : "";
						if(h != NULL)
						{
							STR_N_CPY(args->argv[2], p, STR_LEN(p));
							(args->argv[2])[STR_LEN(p)] = '\0';							
						}
									
						if (h) 
							MEM_HANDLE_UNLOCK(h);								
					}					
					if (prevForm) 
					{
						FrmSetActiveForm(prevForm);
						FrmDrawForm(prevForm);
					}
					FrmDeleteForm(localEncFrm);
					
					if(hitButton == LocalEncryptionOKButton)
						return ISCRTNCB_Done;										
				}
			}
			
		} break;				
		
        case ISCEVTTYPE_Info:			
            switch (event->code) {				
               	case ISCEVT_InfSubsSetStarted:
               		StrCopy(buf, ISYNCSTR_SubsSet);
               		StrCat(buf, event->state.currSubsSet);
               		showStatus1(buf);
               		appLogWrite(_pApp, event->code, event->type, buf);
					break;
				case ISCEVT_InfSyncingSubs:
					if (*event->state.currSubs)
					{
						StrCopy(buf, ISYNCSTR_Subscription);
	               		StrCat(buf, event->state.currSubs);
    	           		showStatus1(buf);		
    	           		appLogWrite(_pApp, event->code, event->type, buf);
					}
					break;
				
				case ISCEVT_InfSyncSucceeded:
					if (!_pApp->mute)
					   GenTune(ISYNC_TUNE_END);
				case ISCEVT_InfSyncFailed:
				case ISCEVT_InfSyncCanceled:
					MainFormDisplayMessages(statusMsg, " ");
					// fall thru
				case ISCEVT_InfSyncStarted:
					showStatus1(statusMsg);
					showStatus2(" ");
					appLogWrite(_pApp, event->code, event->type, statusMsg);
                    break;
                    
                default: // ignore other event code
             	    if(TSTR_LEN(statusMsg) > 0)
					{	
						showStatus2(statusMsg);
						appLogWrite(_pApp, event->code, event->type, statusMsg);
					}
                    break;
            } // switch (event->code)
            break;
            
        case ISCEVTTYPE_Conflict:
		{
			#define SQL_BLOB					-98
			ISCLISTENCONFLICT *pConflict = (ISCLISTENCONFLICT *) event->info;
			int i=0;
			int showSize = 0;
			int temp = 0;
			
			showStatus1(statusMsg);
			showStatus2(" ");

			if (!pConflict) break;
			
			buf[0] = '\0';
			StrCopy(buf, ISYNCSTR_MSG_CONFLICT);
			StrCat(buf, pConflict->table);
			appLogWrite(_pApp, event->code, event->type, buf);
			
			switch (pConflict->op)
			{
				case 1:
					StrCopy(buf, ISYNCSTR_MSG_OP_DELETE);
					StrCat(buf, FILE_NL);
					StrCat(buf, ISYNCSTR_MSG_DATA);
					break;
				case 2:
					StrCopy(buf, ISYNCSTR_MSG_OP_INSERT);
					StrCat(buf, FILE_NL);
					StrCat(buf, ISYNCSTR_MSG_DATA);
					break;
				case 3:
					StrCopy(buf, ISYNCSTR_MSG_OP_UPDATE);
					StrCat(buf, FILE_NL);
					StrCat(buf, ISYNCSTR_MSG_DATA);
					break;
				default:
					StrCopy(buf, ISYNCSTR_MSG_OP_INVALID);
					StrCat(buf, FILE_NL);
					StrCat(buf, ISYNCSTR_MSG_DATA);
					break;
			}
			logWrite(_pApp->hLog, buf);

			for (i=0; i<pConflict->colc; i++)
			{
				ISCLISTENCOLUMN *pColumn = &(pConflict->colv[i]);				
				buf[0] = '\0';

				if (pColumn->data == NULL) 
				{
					StrCat(buf, ISYNCSTR_MSG_NULL);
				}
				else if (pColumn->type == SQL_BLOB) 
				{
					StrCat(buf, " * ");
				}
				else 
				{
					if(pColumn->size > 100)
						showSize = 100;
					else
						showSize = pColumn->size;
					
					temp = StrLen(buf);										
					MemMove(buf + StrLen(buf), &(pColumn->data[0]), showSize);
					buf[temp + showSize] = '\0';
					StrCat(buf, " ");
						
					if(showSize < pColumn->size)
						StrCat(buf, "...");					
				}
				logWrite(_pApp->hLog, buf);								
			}

			buf[0] = '\0';
			break;
		}

        
        
        default:
        	if(TSTR_LEN(statusMsg) > 0)
			{	
				showStatus2(statusMsg);
				appLogWrite(_pApp, event->code, event->type, statusMsg);
			}			
            break;
    } // switch (event->type)

	
	return ISCRTNCB_Default;
}
