/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */
/************************************************************************
 *
 * File Name:  UserForm.c
 * Description: Server/Client/Network Settings panels
 *
 ************************************************************************
 */

#ifdef UNDER_PO_31
#	include <Pilot.h>
#else
#	include <PalmOS.h>
#	include <PalmCompatibility.h>
#	include <VFSMgr.h>

#endif
#include <SysEvtMgr.h>
#include "StarterRsc.h"

#include "app.h"
#include "MainForm.h"
#include "UserForm.h"

// User Settings form
static void UserFormInit(FormPtr frm);
static Boolean UserFormHandleEvent(EventPtr eventP);
static Boolean saveUserForm(FormPtr frm);

static char	*traceValues[] = {ISYNCSTR_Default,	ISYNCSTR_Detailed};

// Server Settings form
static void ServerFormInit(FormPtr frm);
static Boolean ServerURLListInit(FormPtr frm);
static Boolean ServerFormHandleEvent(EventPtr eventP);
static Boolean saveServerForm(FormPtr frm);
static Boolean selectURLList();
static Boolean updateURLList(FormPtr frm, isy_TCHAR *serverURL);
static void updatePasswordDisplay(FormPtr frm);
static void enterPassword(FormPtr frm);

// Network Settings form
static void NetworkProxyInit(FormPtr frm);
static void NetworkFormInit(FormPtr frm);
static void showTimeoutLength(FormPtr frm, int timeoutLength);
static void setTimeoutLength(FormPtr frm);
static void showMessageSize(FormPtr frm, long msgSize);
static void setMessageSize(FormPtr frm);
static void defaultNetworkForm(FormPtr frm);
static void saveNetworkForm(FormPtr frm);
static void getFieldText(FieldPtr fld, isy_TCHAR *textBuf, int bufSize);
static void updateFieldText(FieldPtr fld, isy_TCHAR *fieldText);

static char *timeoutValues[] =	{ 	(ISYNCSTR_TIMEOUTMinimum),(ISYNCSTR_TIMEOUT1min), (ISYNCSTR_TIMEOUT2min),
					  				(ISYNCSTR_TIMEOUT3min),	(ISYNCSTR_TIMEOUT4min),	(ISYNCSTR_TIMEOUT5min),
					  				(ISYNCSTR_TIMEOUT6min),	(ISYNCSTR_TIMEOUT7min),	(ISYNCSTR_TIMEOUT8min),
				  					(ISYNCSTR_TIMEOUT9min),	(ISYNCSTR_TIMEOUT10min), (ISYNCSTR_TIMEOUT15min),
				  					(ISYNCSTR_TIMEOUT20min), (ISYNCSTR_TIMEOUT25min), (ISYNCSTR_TIMEOUT30min),
				  					(ISYNCSTR_TIMEOUT40min), (ISYNCSTR_TIMEOUT50min), (ISYNCSTR_TIMEOUT60min),
				  					(ISYNCSTR_TIMEOUTNever)
								};

static char *networkValues[] =	{	
									ISYNCSTR_Default, ISYNCSTR_4KMODEM, ISYNCSTR_9KGSM, 
									ISYNCSTR_19KMODEM, ISYNCSTR_56KMODEM, ISYNCSTR_64KGPRS
								};


static ISYNCPREFS f_prefs;
#define MAX_HIST    5

static DmOpenRef serverDB;
static VoidHand hDummy = NULL;
static VoidPtr pListChoices;
static VoidHand hChoices = NULL;
static CharPtr listChoices;

#define PALM_MEM_EXPANSION
#ifdef PALM_MEM_EXPANSION
static VoidPtr pSlotListChoices;
static VoidHand hSlotListChoices = NULL;
static CharPtr slotListChoices;
static isy_TCHAR slotList[256] = {'\0'};
#endif

static void UserFormInit(FormPtr frm)
{
	ListPtr traceLst;
	
	Err err;
	UInt32 vfsMgrVersion;

#ifdef PALM_MEM_EXPANSION
	VolumeInfoType volInfo;
	UInt32 volIterator;
	UInt16 volRefNum;	

	int offset = 0, slotCount = 0, selectedSlot = 0;
	ListPtr slotLst;
	ControlPtr ctlPtr;
	int i;
	isy_TCHAR tempSlot[5] = {'\0'};
	isy_TCHAR tempBuff[5] = {'\0'};
	
#endif

	GET_APP_
	
	// Set the trace dropdown list
	int numItems = 0;
	traceLst = (ListPtr)GetObjectPtr(frm, UserTraceList);
	
	numItems = LstGetNumberOfItems(traceLst);
	LstSetListChoices(traceLst, traceValues, numItems);
	
	LstSetSelection(traceLst, appPrefOptionTrace(_pApp) ? 1 : 0); // 0 - Default, 1 - Detailed
	LstDrawList(traceLst);
	CtlSetLabel((ControlPtr)GetObjectPtr(frm, UserTracePopTrigger), LstGetSelectionText(traceLst, LstGetSelection(traceLst)));

	//clear slotList.
	for(i=0; i<256; i++)
		slotList[i] = '\0';		
		
	appGetPref(ISYNCPREF_TargetSlot, appPrefTargetSlot(_pApp), sizeof(appPrefTargetSlot(_pApp)));
	// main memory is slot 0 (default)
	StrCopy(slotList + offset, ISYNCSTR_Device);
	offset += StrLen(slotList + offset) + 1;
	slotCount++;

	
#ifdef PALM_MEM_EXPANSION		
	err = FtrGet(sysFileCVFSMgr, vfsFtrIDVersion, &vfsMgrVersion); 
			
	if (err == errNone) {					
			
		for (volIterator = vfsIteratorStart; volIterator != vfsIteratorStop; ) 
		{
			err = VFSVolumeEnumerate(&volRefNum, &volIterator);
			if (err == errNone) 
			{
				err = VFSVolumeInfo(volRefNum, &volInfo);
				if(err == errNone)
				{
				    switch (volInfo.mediaType) 
				    {
					    case expMediaType_MemoryStick:
					      StrCat(slotList + offset, "MS ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    case expMediaType_CompactFlash:
					      StrCat(slotList + offset, "CF ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    case expMediaType_SecureDigital:
					      StrCat(slotList + offset, "SD ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    case expMediaType_MultiMediaCard:
					      StrCat(slotList + offset, "MMC ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    case expMediaType_SmartMedia:
					      StrCat(slotList + offset, "SM ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    case expMediaType_RAMDisk:
					      StrCat(slotList + offset, "RAM ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    case expMediaType_PoserHost:
					    case expMediaType_MacSim:
					      StrCat(slotList + offset, "POSE ");
					      offset += StrLen(slotList + offset) + 1;
					      break;
					    default:
					      StrCat(slotList + offset, ISYNCSTR_Unknown);
					      offset += StrLen(slotList + offset) + 1;
					      break;
				    }
				} 				

				/*if(StrAToI(appPrefTargetSlot(_pApp)) == volRefNum)
					slotFound = slotCount;*/
				slotCount++;
			}
		}
	}
#endif

  	if(slotCount <= 1)
  	{   		
  		//LstSetSelection((ListPtr)GetObjectPtr(frm, UserTargetSlotList), 1);
  		
    	// Either VFS manager not installed, or no memory expansion cards
	    FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetDirField));
	    FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetDirLabel));	
#ifdef PALM_MEM_EXPANSION
		//FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetSlotLabel));
		//FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetSlotPopTrigger));
#endif
	}
	
	if(slotCount > 1)
	{
		// Set the target dir list selection
		appGetPref(ISYNCPREF_TargetDir, appPrefTargetDir(_pApp), sizeof(appPrefTargetDir(_pApp)));
		updateFieldText(GetObjectPtr(frm, UserTargetDirField), appPrefTargetDir(_pApp));
		FrmSetFocus(frm, FrmGetObjectIndex(frm, UserTargetDirField));
	}
	
	if(hSlotListChoices)
	{
		MemHandleUnlock(hSlotListChoices);
		MemHandleFree(hSlotListChoices);
		hSlotListChoices = NULL;		
	}
	
	hSlotListChoices = SysFormPointerArrayToStrings(slotList, slotCount);
	if(!hSlotListChoices)
		ErrDisplay("Form string pointers failed.");
	else
	{
		pSlotListChoices = MemHandleLock(hSlotListChoices);
		slotLst = (ListPtr)GetObjectPtr(frm, UserTargetSlotList);
		LstDrawList(slotLst);

		LstSetListChoices(slotLst, pSlotListChoices, slotCount);
		
		LstSetHeight(slotLst, slotCount);
																	
		ctlPtr = (ControlPtr)GetObjectPtr(frm, UserTargetSlotPopTrigger);
		
		if(slotCount <= 1)
			selectedSlot = 0;
		else
			selectedSlot = StrAToI(appPrefTargetSlot(_pApp));
			
		CtlSetLabel(ctlPtr, LstGetSelectionText((ListPtr)GetObjectPtr(frm, UserTargetSlotList), selectedSlot));
		LstSetSelection((ListPtr)GetObjectPtr(frm, UserTargetSlotList), selectedSlot);
		
		setLabel(frm, UserTargetDirLabel, ISYNCSTR_TargetDir);
		setLabel(frm, UserTargetSlotLabel, ISYNCSTR_Slot);
		
		if (selectedSlot <= 0) {
			FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetDirField));
			FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetDirLabel));	
		}
		else
		{
			setLabel(frm, UserTargetSlotLabel, ISYNCSTR_Slot);
			setLabel(frm, UserTargetDirLabel, ISYNCSTR_TargetDir);
		}

	}		

	
	FrmCopyTitle(frm, ISYNCSTR_Client_Settings);
	setCtrlLabel(frm, UserOKButton, ISYNCSTR_OK);
	setCtrlLabel(frm, UserCancelButton, ISYNCSTR_Cancel);    
	setLabel(frm, UserTraceLabel, ISYNCSTR_Trace);

}

static Boolean UserFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
	FormPtr frm;
	ListPtr slotLst;
	UInt16 slotSel = 0;

	switch (eventP->eType) {		

	case frmCloseEvent:
		break;

	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID) {
		case UserOKButton:
			handled = !saveUserForm(FrmGetActiveForm());
			break;
		case UserCancelButton:			
			break;
		case UserTargetSlotPopTrigger:
			break;
		default:
			break;
		}
		break;

	case popSelectEvent:
		if (eventP->data.ctlSelect.controlID == UserTargetSlotPopTrigger) {
			frm = FrmGetActiveForm();
			slotLst = GetObjectPtr(frm, UserTargetSlotList);
			slotSel = LstGetSelection(slotLst);
			CtlSetLabel(GetObjectPtr(frm, UserTargetSlotPopTrigger), LstGetSelectionText(slotLst, slotSel));
			if (eventP->data.popSelect.selection > 0) {
				FrmShowObject(frm, FrmGetObjectIndex(frm, UserTargetDirField));
				FrmShowObject(frm, FrmGetObjectIndex(frm, UserTargetDirLabel));	
			} else {
				FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetDirField));
				FrmHideObject(frm, FrmGetObjectIndex(frm, UserTargetDirLabel));	
			}
			handled = true;
		}
		break;

	default:
		break;
	
	}
	
	return handled;
}

static Boolean saveUserForm(FormPtr frm)
{
	static char targetValue[80] = {'\0'};
	isy_TCHAR traceOn[2];
	
	GET_APP_
	
#ifdef PALM_MEM_EXPANSION
	isy_TCHAR tempSlot[5] = {'\0'};

	ListPtr slotLst;
	UInt16 slotSel = 0;
#endif	
	getFieldText(GetObjectPtr(frm, UserTargetDirField), targetValue, sizeof(targetValue));
	appSetPref(ISYNCPREF_TargetDir, targetValue);
	
#ifdef PALM_MEM_EXPANSION
	//Get slot number
	slotLst = (GetObjectPtr(frm, UserTargetSlotList));
	
	slotSel = LstGetSelection(slotLst);

	if(slotSel != noListSelection)
	{	
		if(LstGetSelectionText(slotLst, slotSel) != NULL)
		{
			//TSTR_N_CPY(tempSlot, LstGetSelectionText(slotLst, slotSel), sizeof(tempSlot) - 1);	
			StrIToA(tempSlot, slotSel);	
	    	appSetPref(ISYNCPREF_TargetSlot, tempSlot);
	    }
	}
#endif
	//end slot number retrieval

	if (LstGetSelection((ListPtr) GetObjectPtr(frm, UserTraceList)) == 1) { // 0 - Default, 1 - Detailed
		TSTR_CPY(traceOn, isy_T("Y"));
		appSetPref(ISYNCPREF_Trace, traceOn); // Trace ON
	} else {
		TSTR_CPY(traceOn, isy_T("N"));
		appSetPref(ISYNCPREF_Trace, traceOn); // Trace OFF
	}		
	
	return true;
}

#ifdef PALM_MEM_EXPANSION
static void showTargetSlot(FormPtr frm, int targetDirNum)
{
	ListPtr  list = GetObjectPtr(frm, UserTargetSlotList);
	isy_TCHAR *selDir;

	LstSetSelection(list, targetDirNum);
	selDir = LstGetSelectionText(list, targetDirNum);
	CtlSetLabel(GetObjectPtr(frm, UserTargetSlotPopTrigger), selDir);
} // showTargetSlot

static void setTargetSlot(FormPtr frm)
{
	isy_TCHAR targetDirPref[8];
	ListPtr list = GetObjectPtr(frm, UserTargetSlotList);
	int sel = LstGetSelection(list);
	isy_TCHAR *selDir = LstGetSelectionText(list, sel);

	GET_APP_
	
	TSTR_ITOA(targetDirPref, sel);
	
	appSetPref(ISYNCPREF_TargetDir, targetDirPref); 

} // setTargetSlot
#endif

Boolean doUserForm()
{
	FormPtr prevForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(UserForm);
	Word hitButton;
		
	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, UserFormHandleEvent);
	FrmDrawForm(frm);

	UserFormInit(frm);
	
	hitButton = FrmDoDialog(frm);
	
	if (prevForm) 
	{
		FrmSetActiveForm(prevForm);
		FrmDrawForm(prevForm);
	}
	FrmDeleteForm(frm);
	
#ifdef PALM_MEM_EXPANSION
	//Free Network form's locks	
	if(hSlotListChoices)
	{
		MemHandleUnlock(hSlotListChoices);
		MemHandleFree(hSlotListChoices);
		hSlotListChoices = NULL;
	}	
#endif

	if(hDummy)
	{	
		MemHandleUnlock(hDummy);
		MemHandleFree(hDummy);
		hDummy = NULL;
	}
			
	if(hChoices)
	{
		MemHandleUnlock(hChoices);
		MemHandleFree(hChoices);
		hChoices = NULL;
	}
		
	return hitButton == UserOKButton;
}

static void ServerFormInit(FormPtr frm)
{
	GET_APP_
	
	appGetPref(ISYNCPREF_URL,      f_prefs.URL, sizeof(f_prefs.URL)/sizeof(isy_TCHAR));			// protocol://IP or hostname:port
	appGetPref(ISYNCPREF_User,     f_prefs.user, sizeof(f_prefs.user)/sizeof(isy_TCHAR));			// user ID
	appGetPref(ISYNCPREF_Password, f_prefs.password, sizeof(f_prefs.password)/sizeof(isy_TCHAR));	// password

	ServerURLListInit(frm);
	updateFieldText(GetObjectPtr(frm, ServerUserNameField), f_prefs.user);
	updateFieldText(GetObjectPtr(frm, ServerPasswordField), f_prefs.password);

	if (StrLen(f_prefs.URL) == 0) {
		FrmSetFocus(frm, FrmGetObjectIndex(frm, ServerURLField));
	} else if (StrLen(f_prefs.user) == 0) {
		FrmSetFocus(frm, FrmGetObjectIndex(frm, ServerUserNameField));
//	} else {
//		FrmSetFocus(frm, FrmGetObjectIndex(frm, ServerPasswordField));
	}

	updatePasswordDisplay(frm);
	
	// Set the SavePassword checkbox
	CtlSetValue(GetObjectPtr(frm, ServerSaveCheckbox), appPrefOptionSavePwd(_pApp));
	
	FrmCopyTitle(frm, ISYNCSTR_Server_Settings);
	setCtrlLabel(frm, ServerOKButton, ISYNCSTR_OK);
	setCtrlLabel(frm, ServerCancelButton, ISYNCSTR_Cancel);   
	setCtrlLabel(frm, ServerSaveCheckbox, ISYNCSTR_Save_Password);
	setLabel(frm, ServerUserNameLabel, ISYNCSTR_User);   
	setLabel(frm, ServerPasswordLabel, ISYNCSTR_Password);
	setLabel(frm, ServerURLLabel, ISYNCSTR_URL);   
		
}

static Boolean ServerURLListInit(FormPtr frm)
{
//	ControlPtr ctlPtr;
	UInt     numRec, cursor = 0;
	int      cursorFound = -1;
	VoidHand hrecord;
	VoidPtr  precord;
	Handle   hsz;
	CharPtr  psz;
	UInt     offset = 0;
	ListPtr  pURLLst;
	
	GET_APP_

	serverDB = DmOpenDatabaseByTypeCreator('sevr', appFileCreator, dmModeReadWrite);
	if (! serverDB) return false;

	numRec = DmNumRecords(serverDB);
	if(numRec == 0)
	{
		hrecord = DmNewRecord(serverDB, &cursor, ISYNCLEN_URL);
		precord = MemHandleLock(hrecord);
																																					
		DmSet(precord, 0, ISYNCLEN_URL, 0);
		DmWrite(precord, 0, f_prefs.URL, StrLen(f_prefs.URL));	
		
		MemHandleUnlock(hrecord);
		DmReleaseRecord(serverDB, cursor, true);
	}

	hsz = DmGetResource('tSTR', _enter_String);
	psz = MemHandleLock(hsz);

	// Free the memory for the choice list the pointer array
	if (hChoices) {
		MemHandleUnlock(hChoices);
		MemHandleFree(hChoices);
		hChoices = NULL;
	}
	if (hDummy) {
		MemHandleUnlock(hDummy);
		MemHandleFree(hDummy);
		hDummy = NULL;
	}
	// Prepare the choices list for all the current URL settings
	numRec = DmNumRecords(serverDB);
	hDummy = MemHandleNew(numRec * ISYNCLEN_URL + StrLen(psz) + 1);
	listChoices = MemHandleLock(hDummy);
						
	*listChoices = '\0';
	StrCopy(listChoices, psz);
	offset += StrLen(psz);
	*(listChoices + offset) = '\0';
	offset++;

	MemHandleUnlock(hsz);
	DmReleaseResource(hsz);
												
	for (cursor = numRec; cursor > 0; cursor--)
	{					
		//get text ptr from database record first
		hrecord = DmQueryRecord(serverDB, cursor-1);				
		precord = MemHandleLock(hrecord);
		if (cursorFound < 0 && TSTR_I_CMP(f_prefs.URL, precord) == 0) {
			cursorFound = numRec - cursor + 1; // in reverse order
		}
		StrCopy(listChoices + offset, precord);
		offset += StrLen(precord) + 1;
		MemHandleUnlock(hrecord);			
	}
								
	// Set the new choise list
	hChoices = SysFormPointerArrayToStrings(listChoices, numRec + 1);
	if(!hChoices) {
		ErrDisplay("Form string pointers failed.");
		return false;
	} else
	{
		pListChoices = MemHandleLock(hChoices);
		pURLLst = (ListPtr)GetObjectPtr(frm, ServerURLList);
//		LstDrawList(pURLLst);

		LstSetListChoices(pURLLst, pListChoices, numRec + 1);

		if(numRec + 1 < 5)
			LstSetHeight(pURLLst, numRec + 1);
		else
			LstSetHeight(pURLLst, 5);
																	
//		ctlPtr = (ControlPtr)GetObjectPtr(frm, ServerURLPopTrigger); // GUI redesign  JSY (3/32/2003)
//		CtlSetLabel(ctlPtr, f_prefs.URL);
		updateFieldText(GetObjectPtr(frm, ServerURLField), f_prefs.URL);
		LstSetSelection((ListPtr)GetObjectPtr(frm, ServerURLList), cursorFound);
	}				
												
	DmCloseDatabase(serverDB);
	return true;
}

static Boolean ServerFormHandleEvent(EventPtr eventP)
{
    Boolean handled = false;
	ListPtr pURLLst;
	Word    prior;
	FormPtr frm;
	Handle  hsz;
	CharPtr psz;

	switch (eventP->eType) {		

	case frmCloseEvent:
		break;

	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID) {
		case ServerOKButton:
			handled = !saveServerForm(FrmGetActiveForm());
			break;
		case ServerCancelButton:			
			break;
		case ServerURLPopTrigger:
			pURLLst = NULL;
			break;
		case ServerPasswordSelTrigger:
			handled = true;
			enterPassword(FrmGetActiveForm());
			break;
		}
		break;

	case popSelectEvent:
		pURLLst = GetObjectPtr(FrmGetFormPtr(ServerForm), ServerURLList);
		prior = eventP->data.popSelect.priorSelection;
		frm = FrmGetActiveForm();
		hsz = DmGetResource('tSTR', _enter_String);
		psz = MemHandleLock(hsz);

		if (StrCompare(psz, LstGetSelectionText(pURLLst, LstGetSelection(pURLLst))) == 0) {
			updateFieldText(GetObjectPtr(frm, ServerURLField), "");
			LstSetSelection(pURLLst, prior);
			FrmSetFocus(frm, FrmGetObjectIndex(frm, ServerURLField));
		} else {
			updateFieldText(GetObjectPtr(frm, ServerURLField), LstGetSelectionText(pURLLst, eventP->data.popSelect.selection));
		}
		LstEraseList(pURLLst);
		FrmDrawForm(frm);
		MemHandleUnlock(hsz);
		DmReleaseResource(hsz);
		handled = true;
		break;

	default:
		break;
	
	}
	
	return handled;
}

static Boolean updateURLList(FormPtr frm, isy_TCHAR *serverURL)
{
	VoidHand hrecord;
	VoidPtr  precord;
	UInt     numRec, cursor = 0;
	int      cursorFound = -1;

	serverDB = DmOpenDatabaseByTypeCreator('sevr', appFileCreator, dmModeReadWrite);
	if (! serverDB) return false;

	numRec = DmNumRecords(serverDB);
	for (cursor = numRec; cursorFound < 0 && cursor > 0; cursor--) {
		hrecord = DmQueryRecord(serverDB, cursor-1);				
		precord = MemHandleLock(hrecord);
		if (cursorFound < 0 && StrCompare(serverURL, precord) == 0) {
			cursorFound = cursor-1;
		}
		MemHandleUnlock(hrecord);			
	}
	if (cursorFound >= 0) {
		//Move the current record to the end
		if (cursorFound != numRec - 1) DmMoveRecord(serverDB, cursorFound, numRec);
	} else if (StrLen(serverURL) > 0) {
		cursor = numRec;
		//Add new record to the end
		hrecord = DmNewRecord(serverDB, &cursor, ISYNCLEN_URL);
		precord = MemHandleLock(hrecord);
																																					
		DmSet(precord, 0, ISYNCLEN_URL, 0);
		DmWrite(precord, 0, serverURL, StrLen(serverURL));	
		
		MemHandleUnlock(hrecord);
		DmReleaseRecord(serverDB, cursor, true);
	}

	for (numRec = DmNumRecords(serverDB); numRec > MAX_HIST; numRec--)
	{
		//Remove the first record, which is least recently used
		DmRemoveRecord(serverDB, 0);								
	}

	DmCloseDatabase(serverDB);

	return false;
}

static void enterPassword(FormPtr frm)
{
	FrmHideObject(frm, FrmGetObjectIndex(frm, ServerPasswordSelTrigger));
	FrmShowObject(frm, FrmGetObjectIndex(frm, ServerPasswordField));
	updateFieldText(GetObjectPtr(frm, ServerPasswordField), "");	
	FrmSetFocus(frm, FrmGetObjectIndex(frm, ServerPasswordField));
}

static void updatePasswordDisplay(FormPtr frm)
{
	Handle hsz;
	CharPtr psz;

	hsz = DmGetResource('tSTR', f_prefs.password[0] ? _assigned_String : _prompt_String);
	psz = MemHandleLock(hsz);
	FrmHideObject(frm, FrmGetObjectIndex(frm, ServerPasswordField));
	CtlSetLabel(GetObjectPtr(frm, ServerPasswordSelTrigger), psz);	
	
	MemHandleUnlock(hsz);
	DmReleaseResource(hsz);
}

static Boolean saveServerForm(FormPtr frm)
{
	static int aiSize[3] = { sizeof f_prefs.URL, sizeof f_prefs.user, sizeof f_prefs.password };
	static char* apValue[3] = { f_prefs.URL, f_prefs.user, f_prefs.password };
	static int aiFields[3] = { ServerURLField, ServerUserNameField, ServerPasswordField };
	int i;
	
	for (i = 0; i < sizeof(aiFields) / sizeof(aiFields[0]); i++)
	{
		FieldPtr fld = GetObjectPtr(frm, aiFields[i]);
		getFieldText(fld, apValue[i], aiSize[i]);	
	}
	
	updateURLList(frm, f_prefs.URL);
	appSetPref(ISYNCPREF_URL,  f_prefs.URL); // protocol://IP or hostname:port
	appSetPref(ISYNCPREF_User, f_prefs.user); // User name
	if (f_prefs.password[0] == '\0' ||
		CtlGetValue(GetObjectPtr(frm, ServerSaveCheckbox)) == 0) {
		// Don't memoize (or reuse) the password when the current session is done
		appSetPref(ISYNCPREF_UnsavedPassword, f_prefs.password);// Password good for once
	} else {
		// Do memoize (or reuse) the password
		appSetPref(ISYNCPREF_Password, f_prefs.password);		// Password saved
	}
			
	return true;
}

Boolean doServerForm()
{
	FormPtr prevForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(ServerForm);
	Word hitButton;
		
	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, ServerFormHandleEvent);
	FrmDrawForm(frm);

	ServerFormInit(frm);
	
	hitButton = FrmDoDialog(frm);
	
	if (prevForm) 
	{
		FrmSetActiveForm(prevForm);
		FrmDrawForm(prevForm);
	}
	FrmDeleteForm(frm);
	
	if(hDummy)
	{	
		MemHandleUnlock(hDummy);
		MemHandleFree(hDummy);
		hDummy = NULL;
	}
			
	if(hChoices)
	{
		MemHandleUnlock(hChoices);
		MemHandleFree(hChoices);
		hChoices = NULL;
	}
		
	return hitButton == ServerOKButton;
}

static void showTimeoutLength(FormPtr frm, int timeoutLength)
{

	ListPtr  list = GetObjectPtr(frm, NetworkTimeoutLengthList);
	int numItems = LstGetNumberOfItems(list);
	isy_TCHAR *selTime, *space;
	isy_TCHAR timeoutStr[4];
	int sel, i, len, timeoutMin;

	LstSetListChoices(list, timeoutValues, numItems);
	
	if (timeoutLength < 0) {
		sel = numItems - 1;
	} else if (timeoutLength == 0) {
		sel = 0;
	} else {
		for (i = 1; i < numItems-1; i++) {
			selTime = LstGetSelectionText(list, i);
			space = STR_CHR(selTime, ' ');
			if (space == NULL) continue;
			len = space - selTime;
			TSTR_N_CPY(timeoutStr, selTime, len);
			timeoutStr[len] = '\0';
			timeoutMin = TSTR_ATOI(timeoutStr);
			if (timeoutLength <= timeoutMin) break;
		}
		sel = i;
	}
	
	LstSetSelection(list, sel);
	selTime = LstGetSelectionText(list, sel);
	CtlSetLabel(GetObjectPtr(frm, NetworkTimeoutLengthPopTrigger), selTime);
		
} // showTimeoutLength

static void setTimeoutLength(FormPtr frm)
{
	isy_TCHAR timeoutPref[8];
	ListPtr list = GetObjectPtr(frm, NetworkTimeoutLengthList);
	int sel = LstGetSelection(list);
	isy_TCHAR *selTime = LstGetSelectionText(list, sel);
	isy_TCHAR *space = STR_CHR(selTime, ' ');

	GET_APP_
	
	if (sel == 0)
		TSTR_CPY(timeoutPref, isy_T("0"));
	else if (space == NULL)
		TSTR_CPY(timeoutPref, isy_T("-1"));
	else {
		int len = space - selTime, timeoutLength;
		TSTR_N_CPY(timeoutPref, selTime, len);
		timeoutPref[len] = '\0';
	    timeoutLength = TSTR_ATOI(timeoutPref);
		TSTR_ITOA(timeoutPref, timeoutLength * 60); // 60 seconds
	}
	
	appSetPref(ISYNCPREF_TimeoutRecv, timeoutPref); // Recv timeout
	appSetPref(ISYNCPREF_TimeoutSend, timeoutPref); // Send timeout
} // showTimeoutLength

/*
	the netType is of the form:
		none
		4 K Modem
		2 M MTS
	extract the number and the multiplier and return the value.
	Since we define the values in the gui we know it is properly formatted.
	
	Parameters:
		selNetType - string to parse
		sel - current position in NetworkNetTypeList
*/
static isy_UINT32 parseMsgSizeFromBaud(isy_TCHAR *selNetType, int sel)
{
	isy_TCHAR *space = STR_CHR(selNetType, ' ');
	isy_TCHAR baudIntStr[12];
	long multiplier = 1024;
	isy_TCHAR *m;

	GET_APP_
	
	// 1st entry is "none" or disabled.  Protect against malformed formats
	if (sel == 0 || space == NULL || (space - selNetType)-1 > sizeof(baudIntStr))
		return 0;
		
	{
		int len = space - selNetType, baudInt;
		
		// extract the baud int, e.g. 4, 56, 2 (Meg)
		TSTR_N_CPY(baudIntStr, selNetType, len);
		baudIntStr[len] = '\0';
	    baudInt = TSTR_ATOI(baudIntStr);
	    if (baudInt <=0 || baudInt > 10 * 1000 * 1024)
	    	ErrDisplay("Internal network type Baud Rate");
	    
	    // Now extract the multiplier, e.g. K, M
	    for (m = space; *m != '\0'; m++)
	    {
	    	if (*m == 'K' || *m == 'M')
	    		break;
	    }
	    if (*m == 'M')
	    	multiplier *= 1000;
	    	
		return (baudInt * multiplier) / 4;
	}
}

static void showMessageSize(FormPtr frm, long msgSize)
{
	ListPtr  list = GetObjectPtr(frm, NetworkNetTypeList);
	int numItems = LstGetNumberOfItems(list);
	isy_TCHAR *selNetType;
	int sel, i;
	long netBaudInt;
	
	GET_APP_

	LstSetListChoices(list, networkValues, numItems);
	
	if (msgSize <= 0)
		sel = 0;		// default is unset, or, let engine calculate
	else
	{
		for (i = 1; i < numItems-1; i++)
		{
			selNetType = LstGetSelectionText(list, i);
			netBaudInt = parseMsgSizeFromBaud(selNetType, i);
			if (msgSize <= netBaudInt)
				break;
		}
		sel = i;
	}
	
	LstSetSelection(list, sel);
	selNetType = LstGetSelectionText(list, sel);
	CtlSetLabel(GetObjectPtr(frm, NetworkNetTypeSelectPopTrigger), selNetType);
}

static void setMessageSize(FormPtr frm)
{
	ListPtr list = GetObjectPtr(frm, NetworkNetTypeList);
	int sel = LstGetSelection(list);
	isy_TCHAR *selNetType = LstGetSelectionText(list, sel);
	isy_UINT32 msgSize;
	isy_TCHAR msgSizePref[20];
	
	GET_APP_
	msgSize = parseMsgSizeFromBaud(selNetType, sel);
	TSTR_ITOA(msgSizePref, msgSize);
	
	appSetPref(ISYNCPREF_Message_Size, msgSizePref);
}


static Boolean NetworkFormHandleEvent(EventPtr eventP)
{
	FormPtr frm = FrmGetActiveForm();
    Boolean handled = false;

	switch (eventP->eType) {
	case ctlSelectEvent:
		switch (eventP->data.ctlSelect.controlID) {
		case NetworkTimeoutLengthPopTrigger:
			break;
		case NetworkDefaultButton:
			defaultNetworkForm(frm);
			handled = true;
			break;
		case NetworkUseProxyCheckbox:
			NetworkProxyInit(frm);
			handled = true;
			break;
		}					
	
	default:
		break;
	
	}
	
	return handled;
} // NetworkFormHandleEvent

static void NetworkProxyInit(FormPtr frm)
{
	if (CtlGetValue(GetObjectPtr(frm, NetworkUseProxyCheckbox))) {
		FrmShowObject(frm, FrmGetObjectIndex(frm, NetworkProxyIPLabel));
		FrmShowObject(frm, FrmGetObjectIndex(frm, NetworkProxyIPField));
		FrmShowObject(frm, FrmGetObjectIndex(frm, NetworkProxyPortLabel));	
		FrmShowObject(frm, FrmGetObjectIndex(frm, NetworkProxyPortField));
		
		setLabel(frm, NetworkProxyIPLabel, ISYNCSTR_Server);
		setLabel(frm, NetworkProxyPortLabel, ISYNCSTR_Port);
	    
	} else {
		FrmHideObject(frm, FrmGetObjectIndex(frm, NetworkProxyIPLabel));
		FrmHideObject(frm, FrmGetObjectIndex(frm, NetworkProxyIPField));
		FrmHideObject(frm, FrmGetObjectIndex(frm, NetworkProxyPortLabel));	
		FrmHideObject(frm, FrmGetObjectIndex(frm, NetworkProxyPortField));	
	}
}

static void NetworkFormInit(FormPtr frm)
{
	isy_TCHAR timeoutRecv[8];
	int       timeoutLength;
	isy_TCHAR msgSizeBuf[20];
	isy_TCHAR *colon;
	isy_TCHAR *proxyIP;
	isy_TCHAR *proxyPort;
	isy_BOOL  dropNetLib;
	
	GET_APP_

	// Set the DropNetLib checkbox
	dropNetLib = _pApp->prefs.flag & appDropNetLib;
	CtlSetValue(GetObjectPtr(frm, NetworkDropNetLibCheckbox), dropNetLib);

	// Set the timeout list selection
	appGetPref(ISYNCPREF_TimeoutRecv, timeoutRecv, sizeof(timeoutRecv)); // Send timeout
	timeoutLength = TSTR_ATOI(timeoutRecv);
	if (timeoutLength > 0) timeoutLength = timeoutLength / 60; // 60 second
	showTimeoutLength(frm, timeoutLength);
	
	// Set the network baud list selection
	appGetPref(ISYNCPREF_Message_Size, msgSizeBuf, sizeof(msgSizeBuf));
	showMessageSize(frm, TSTR_ATOI(msgSizeBuf));	

	// Set the usability of the Proxy Host/Port fields
    appGetPref(ISYNCPREF_Proxy,     appPrefProxy(_pApp), sizeof(appPrefProxy(_pApp)));			// protocol://IP or hostname:port
	proxyIP = appPrefProxy(_pApp);
    colon = TSTR_CHR(appPrefProxy(_pApp), (isy_TCHAR) ':');
	if (colon) {
		*colon = (isy_TCHAR) '\0';
		proxyPort = colon + 1;
	} else {
		proxyPort = isy_T("");
	}
	updateFieldText(GetObjectPtr(frm, NetworkProxyIPField), proxyIP);
	updateFieldText(GetObjectPtr(frm, NetworkProxyPortField), proxyPort);
	CtlSetValue(GetObjectPtr(frm, NetworkUseProxyCheckbox), appPrefOptionProxy(_pApp));
	NetworkProxyInit(frm);
	if (colon) *colon = (isy_TCHAR) ':';

	FrmCopyTitle(frm, ISYNCSTR_Network_Settings);
	setCtrlLabel(frm, NetworkOKButton, ISYNCSTR_OK);
	setCtrlLabel(frm, NetworkCancelButton, ISYNCSTR_Cancel);
	setCtrlLabel(frm, NetworkUseProxyCheckbox, ISYNCSTR_Proxy);
	setLabel(frm, NetworkTimeoutLabel, ISYNCSTR_Timeout);
	setLabel(frm, NetworkNetTypeLabel, ISYNCSTR_Network_Speed);                        
}

static void defaultNetworkForm(FormPtr frm)
{
	// Set the timeout list selection
	showTimeoutLength(frm, appDefaultTimeout);

	// Turn off network baud setting, disable its use in calculations
	showMessageSize(frm, appDefaultMessageSize);

	// Proxy OFF
	CtlSetValue(GetObjectPtr(frm, NetworkUseProxyCheckbox), 0);
	NetworkProxyInit(frm);
} // defaultNetworkForm

static void saveNetworkForm(FormPtr frm)
{
	isy_BOOL  dropNetLib;

	GET_APP_

	// drop the Net.lib
	dropNetLib = CtlGetValue(GetObjectPtr(frm, NetworkDropNetLibCheckbox));
	if (dropNetLib)
		_pApp->prefs.flag |= appDropNetLib;
	else
		_pApp->prefs.flag &= ~((isy_DWORD) appDropNetLib);

	// Use proxy
	getFieldText(GetObjectPtr(frm, NetworkProxyIPField), appPrefProxy(_pApp), sizeof(appPrefProxy(_pApp)));
	TSTR_CAT(appPrefProxy(_pApp), isy_T(":"));
	getFieldText(GetObjectPtr(frm, NetworkProxyPortField), appPrefProxy(_pApp)+TSTR_LEN(appPrefProxy(_pApp)), sizeof(appPrefProxy(_pApp)) - TSTR_LEN(appPrefProxy(_pApp)));
	if (CtlGetValue(GetObjectPtr(frm, NetworkUseProxyCheckbox)))
		appSetPrefOptionProxy(_pApp);
	else
		appResetPrefOptionProxy(_pApp);
	
	appSavePreferences();

	setTimeoutLength(frm);
	setMessageSize(frm);
} // saveNetworkForm

Boolean doNetworkForm()
{
	FormPtr prevForm = FrmGetActiveForm();
	FormPtr frm = FrmInitForm(NetworkForm);
	Word hitButton;

	FrmSetActiveForm(frm);
	FrmSetEventHandler(frm, NetworkFormHandleEvent);
	FrmDrawForm(frm);
	NetworkFormInit(frm);
		
	do hitButton = FrmDoDialog(frm);
	while (hitButton == NetworkDefaultButton);
	
	if (hitButton == NetworkOKButton)
		saveNetworkForm(frm);

	if (prevForm) 
	{
		FrmSetActiveForm(prevForm);
		FrmDrawForm(prevForm);
	}
	FrmDeleteForm(frm);

	return hitButton == NetworkOKButton;
}

static void getFieldText(FieldPtr fld, isy_TCHAR *textBuf, int bufSize)
{
	Handle h = FldGetTextHandle(fld);
	isy_TCHAR *text;

	if (h)
	{
		text = (isy_TCHAR *) MemHandleLock(h);
		TSTR_N_CPY(textBuf, text, bufSize);
		textBuf[bufSize - 1] = 0;
		MemHandleUnlock(h);
	}
	// let Palm do the clean-up of fields
} // getFieldText

static void updateFieldText(FieldPtr fld, isy_TCHAR *fieldText)
{
	Handle h = MemHandleNew((TSTR_LEN(fieldText) + 1) * sizeof(isy_TCHAR));
	Handle hOld = FldGetTextHandle(fld);
	isy_TCHAR *text = (isy_TCHAR *) MemHandleLock(h);

	TSTR_CPY(text, fieldText);
	MemHandleUnlock(h);
	FldSetTextHandle(fld, h);
	FldDrawField(fld);
	if (hOld) MemHandleFree(hOld);
} // updateFieldText
