/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  EpocOsDefs.cpp
 *
 * Description:
 *    EPOC-specific OS routines.
 *
 */

#include <estlib.h>
#include <ioctl.h>

#include <e32std.h>
#include <f32file.h>
#include "epocosdefs.h"
#include "App.h"

//get the TLS pointer
void* epocGetTls()
{
	return Dll::Tls();
}

//set the TLS pointer
void epocSetTls(void* ptr)
{
	Dll::SetTls(ptr);
}

//globals accessor functions
void epocCreateGlobals()
{
	EPOCGLOBALS* globals = (EPOCGLOBALS*) MEM_PTR_ALLOC(sizeof(EPOCGLOBALS));
	MEM_SET(globals, 0, sizeof(EPOCGLOBALS));

	APPDATA* appPtr = (APPDATA*) MEM_PTR_ALLOC(sizeof(APPDATA));
	MEM_SET(appPtr, 0, sizeof(APPDATA));

	globals->hApp = appPtr;
	EPUIDATA* dataPtr = (EPUIDATA*) MEM_PTR_ALLOC(sizeof(EPUIDATA));
	MEM_SET(dataPtr, 0, sizeof(EPUIDATA));
	globals->hData = dataPtr;
	Dll::SetTls(globals);
}


void epocDestroyGlobals()
{
	EPOCGLOBALS* globals = (EPOCGLOBALS*) Dll::Tls();
	APPDATA* appPtr = (APPDATA*) globals->hApp;

	MEM_PTR_FREE(appPtr);
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;
	MEM_PTR_FREE(dataPtr);

	for (int i=0; i<globals->numLibs; i++)
		MEM_PTR_FREE(globals->adapterNames[i]);

	MEM_PTR_FREE(globals);
}


EPOCGLOBALS* epocGetGlobals()
{
	return (EPOCGLOBALS*) Dll::Tls();
}

//returns the old behavior -- the client must store it
int epocDisableAutoSwitchOff()
{
#if !(defined(UNICODE) || defined(_UNICODE)) 
  TAutoSwitchOffBehavior oldBehavior;
	UserHal::AutoSwitchOffBehavior(oldBehavior);
	UserHal::SetAutoSwitchOffBehavior(ESwitchOffDisabled);

	return oldBehavior;
#endif
 
  return 0;
}

void epocEnableAutoSwitchOff(int behaviorNum)
{
#if !(defined(UNICODE) || defined(_UNICODE)) 
	TAutoSwitchOffBehavior behavior = ESwitchOffDisabled;

	switch (behaviorNum) {
	case 0:
		behavior = ESwitchOffDisabled;
		break;
		
	case 1:
	  behavior = ESwitchOffEnabledOnBatteries;
		break;
		
	case 2:
	  behavior = ESwitchOffEnabledAlways;
	}

	UserHal::SetAutoSwitchOffBehavior(behavior);
#endif  
}

void epocGetCurrentDateTime(int* py, int* pm, int* pd, int* phr, int* pmi, int* pse)
{
	TTime time;
	time.HomeTime();

	TDateTime dateTime = time.DateTime();

	*py = dateTime.Year();
	*pm = dateTime.Month()+1;
	*pd = dateTime.Day()+1;
	*phr = dateTime.Hour();
	*pmi = dateTime.Minute();
	*pse = dateTime.Second();
}
