/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  Model.h
 *
 * Description:
 *    Classes for storing user information
 *
 */

#ifndef _MODEL_H
#define _MODEL_H

#include "db2sync.h"
#include "charconv.h"
#include "f32file.h"


//======================================================================
/*
Unfortunately CUserInfo and CServerInfo need to define two stores for
their members. This is basically because of unicode. If the chars were
always one byte then the GetXxx() functions could return a TPtr which
had beed initialized to point to the c string. This does not work in
unicode because the descriptors have to be 16 bit and therefore cannot
use a regular char*.
*/
//======================================================================
void ConvertUnicodeTextL(CCnvCharacterSetConverter& aCharacterSetConverter, 
            RFs& aFileServerSession, TUint aForeignCharacterSet, 
            const TDesC16& aUnicodeText, TDes8& outputBuffer);

void ConvertForeignTextL(CCnvCharacterSetConverter& aCharacterSetConverter,
     RFs& aFileServerSession, TUint aForeignCharacterSet, 
     const TDesC8& aForeignText, TDes16& outputBuffer);
    
//======================================================================

void DesCToCharPtr(HBufC* pinDes, isy_TCHAR* inPtr);
void CharPtrToFieldDes(const isy_TCHAR* inPtr, TBuf16<ISYNCLEN_MiscInfo>& outDes);
void CharPtrToTIntArray(char * inPtr, TUint8* array, TInt len);

struct Jobs 
{
	isy_TCHAR name[ISYNCLEN_MiscInfo];
	unsigned long mode;
};

//======================================================================
// CJobs
// This stores the various jobs--items which can be synced individually.
class CJobs:public CBase
{
public:
	enum ESynchronizeAction
	{
		synchronizeAction_Skip,
		synchronizeAction_Synchronize,
		synchronizeAction_Refresh,
		synchronizeAction_Resume
	};

	CJobs();
	virtual ~CJobs();

	TInt						ItemCount();
	ESynchronizeAction			ActionForItem(TInt inIndex);
	TBufC<ISYNCLEN_MiscInfo>	ActionDesForItem(TInt inIndex);
	void						SetActionForItem(TInt inIndex, ESynchronizeAction inAction);
	HBufC* CJobs::NameDesForItem(TInt inIndex);

  void GetJobs();

private:
	TBuf<ISYNCLEN_MiscInfo>			iActionName_Skip;
	TBuf<ISYNCLEN_MiscInfo>			iActionName_Synchronize;
	TBuf<ISYNCLEN_MiscInfo>			iActionName_Refresh;
	TBuf<ISYNCLEN_MiscInfo>			iActionName_Resume;

	Jobs* jobArray;
	int numJobs;
	int size;
};

//======================================================================
// CModel.
// This is the data model of the program. It contains all of the smaller objects
// which store the area-specific settings.
class CModel
{
public:
	CModel();
	void ConstructL();
	virtual ~CModel();
	CJobs& Jobs() { return iJobs; }

private:
	CJobs iJobs;
};

#endif //_MODEL_H

