/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  WorkerThread.cpp
 *
 * Description:
 *    Sync engine thread implementation
 *
 */

#include "Workerthread.h"
#include "Dialogs.h"
#include "SyncDriver.h"

_LIT(kThreadName_Synchronization,"SyncThread");

//======================================================================
// CSynchronizationThreadHandler
CSynchronizationThreadHandler::CSynchronizationThreadHandler(CProgressDialog* inProgressDialogPtr)
	:CActive(EPriorityStandard)
{
	// set up parameters to thread
	iParams.iProgressDialogPtr=inProgressDialogPtr;

	CActiveScheduler::Add(this);
	__DECLARE_NAME(_S("CSynchronizationThreadHandler"));
};

CSynchronizationThreadHandler::~CSynchronizationThreadHandler() 
{
	// cancel any requests and tell server 
	Cancel();
	delete iParams.iEnginePtr;
}


void CSynchronizationThreadHandler::RunL() 
{
	iParams.iProgressDialogPtr->WorkerThreadFinishedL();
}

void CSynchronizationThreadHandler::DoCancel() 
{
}

// initiate a request
void CSynchronizationThreadHandler::Synchronize(void* globals)
{
	RThread thread;

	iParams.iEnginePtr=new(ELeave) CSynchronizationEngine(iParams);
	iParams.globals = globals;

	// generate thread, leave if fails
	// The new thread must use the same heap as this one. Otherwise memory that
	// is allocated by one thread cannot be deallocated by the other.
	TInt result = thread.Create(
		kThreadName_Synchronization, (TThreadFunction)SynchronizationThreadMethod,
		0x4000, 1000000, 1000000, &iParams, EOwnerThread
	);
  	User::LeaveIfError(result) ;

	// log on to thread -	sets iStatus to KRequestPending 
	//						requests notification of thread completion
	thread.Logon(iStatus) ;

	// give thread low priority 
	thread.SetPriority(EPriorityMuchLess) ;

	// resume thread (wake it up sometime after this function returns)
  	thread.Resume();

	// Close the handle, not the thread.
  	thread.Close();

	// ensure scheduler checks status 
	SetActive();
}


// Thread routine that sorts out parameters & calls engine
TInt CSynchronizationThreadHandler::SynchronizationThreadMethod(TAny* inParamsPtr)
{
	CSynchronizationParams* paramsPtr=(CSynchronizationParams*)inParamsPtr;
	CSynchronizationEngine* enginePtr=(CSynchronizationEngine*)paramsPtr->iEnginePtr;
	void* globals = paramsPtr->globals;

	Dll::SetTls(globals);

	TRAPD(error, enginePtr->Synchronize());
	if(error)
	{
	}

	return KErrNone ;
}


//======================================================================
// CSynchronizationEngine
CSynchronizationEngine::CSynchronizationEngine(const CSynchronizationParams& inParams)
	:iParams(inParams)
{
}

void CSynchronizationEngine::Synchronize()
{
	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;

	StaticSetStatusL((isy_TCHAR*)((ISYNCSTR_InfSyncStarted)), iParams.iEnginePtr);

	//callbacks for status and error messages
	dataPtr->setStatus = StaticSetStatusL;
	dataPtr->messageBox = StaticMessageBoxL;
	dataPtr->engine = iParams.iEnginePtr;

	int oldBehavior = epocDisableAutoSwitchOff();
	syncLoop();
	epocEnableAutoSwitchOff(oldBehavior);

	CloseSTDLIB();
}

void CSynchronizationEngine::SetStatusTextL(const TBuf16<ISYNCLEN_MiscInfo>& inEntry)
{
	iParams.iProgressDialogPtr->PostMessageL(inEntry);
}

void CSynchronizationEngine::SetErrorTextL(const TBuf16<ISYNCLEN_MiscInfo>& inEntry)
{
	iParams.iProgressDialogPtr->PostMessageErrorL(inEntry);
}

void CSynchronizationEngine::StaticSetStatusL(isy_TCHAR* inCString, void* inEnginePtr)
{
	TBuf16<ISYNCLEN_MiscInfo> des;
	CharPtrToFieldDes(inCString, des);
 	((CSynchronizationEngine*)inEnginePtr)->SetStatusTextL(des);
}

void CSynchronizationEngine::StaticSetErrorL(const isy_TCHAR* inCString, void* inEnginePtr)
{
	TBuf16<ISYNCLEN_MiscInfo> des;
	CharPtrToFieldDes(inCString, des);
	((CSynchronizationEngine*)inEnginePtr)->SetErrorTextL(des);
}

int CSynchronizationEngine::StaticMessageBoxL(isy_TCHAR* message, void* inEnginePtr, int buttons, void* args)
{
	return ((CSynchronizationEngine*)inEnginePtr)->MessageBoxL(message, buttons, args);
}

int CSynchronizationEngine::MessageBoxL(isy_TCHAR* message, int buttons, void* args)
{
	EPOCGLOBALS* globals = epocGetGlobals();
	EPUIDATA* dataPtr = (EPUIDATA*) globals->hData;
	
	iParams.iProgressDialogPtr->msgBoxButtonClicked = 0;
	iParams.iProgressDialogPtr->SetMessage(message);
	iParams.iProgressDialogPtr->SetMessageBoxFlag(buttons);
	iParams.iProgressDialogPtr->SetArgs(args);

	dataPtr->msgBoxButtonClicked = 0;
	while (dataPtr->msgBoxButtonClicked == 0)
		User::After(2000000);

	return dataPtr->msgBoxButtonClicked;
}

//======================================================================
