/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  WorkerThread.h
 *
 * Description:
 *    Thread which runs the sync engine code
 *
 */

#ifndef _WORKERTHREAD_H
#define _WORKERTHREAD_H

#include "db2sync.h"

class CProgressDialog;
class CSynchronizationEngine;

//======================================================================
// CSynchronizationParams
class CSynchronizationParams
{
public:
	CProgressDialog*		iProgressDialogPtr;
	CSynchronizationEngine*	iEnginePtr;
	void* globals;
};

//======================================================================
// CSynchronizationEngine
// This is the thread class.
class CSynchronizationEngine
{
public:
	CSynchronizationEngine(const CSynchronizationParams& inParams);
	void Synchronize();
	static void	StaticSetStatusL(isy_TCHAR* inCString, void* inEnginePtr);
	static void	StaticSetErrorL(const isy_TCHAR* inCString, void* inEnginePtr);
	static int StaticMessageBoxL(isy_TCHAR* message, void* inEnginePtr, int buttons, void* args);

protected:
	void SetStatusTextL(const TBuf16<ISYNCLEN_MiscInfo>& inEntry);
	void SetErrorTextL(const TBuf16<ISYNCLEN_MiscInfo>& inEntry);
	int MessageBoxL(isy_TCHAR* message, int buttons, void *args);
	const CSynchronizationParams& iParams;
};

//======================================================================
// CSynchronizationThreadHandler
//
// The active object.
class CSynchronizationThreadHandler:public CActive
{
public:
	CSynchronizationThreadHandler(CProgressDialog* inProgressDialogPtr);
	virtual ~CSynchronizationThreadHandler();

	void Synchronize(void* globals);

private:
	void RunL();
	void DoCancel();
	static TInt SynchronizationThreadMethod(TAny* inParamsPtr);

private:
	CSynchronizationParams	iParams;
};

#endif //_WORKERTHREAD_H