/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name: db2sync.h
 *
 * Description:
 *    Main classes for Isync app
 *
 */

#ifndef _ISYNCUI_H
#define _ISYNCUI_H

#include "Common.h"
#include "App.h"

// UID of app
#ifdef _UNICODE
const TUid KUidIkISync = { 0x10009586 };
#endif

class CISyncDocument;
class CModel;
class CISyncAppUi;

//======================================================================
// CISyncAppView
class CISyncAppView
	:public CCoeControl
	,public MCoeControlObserver
	,public MCoeControlBrushContext
{
public:
	CISyncAppView(CISyncAppUi* inUiPtr);
	virtual	~CISyncAppView();
	void	ConstructL(const TRect& aRect);
private:
	// From MCoeControlObserver.
	void	HandleControlEventL(CCoeControl* aControl, TCoeEvent aEventType);

	// from CCoeControl
	void	Draw(const TRect& /*aRect*/) const;
	TInt	CountComponentControls() const;
	CCoeControl* ComponentControl(TInt aIndex) const;

private:
	void	AddButtonL(const TInt aId, const TDesC& aText);

	CISyncAppUi*		iUiPtr;
	CEikControlGroup*	iButtonsPtr;
};

//======================================================================
// CISyncAppUi
class CISyncAppUi : public CEikAppUi
	{
public:
	void	ConstructL();
	virtual	~CISyncAppUi();

	void	HandleCommandL(TInt aCommand);
	void	ShowApplicationsDialog();
	void	ShowServerSettingsDialog();
	void	ShowClientSettingsDialog();
	void	ShowNetworkSettingsDialog();
	void	ShowJobsDialog();
	void	ShowProgressDialog();
	void	ShowLogDialog();
	void	ShowAboutDialog();

	static void	ShowMessageBoxDialog(isy_TCHAR* msg);

private:
	CISyncAppView*		iAppViewPtr;
	CModel*				iModelPtr;
};

//======================================================================
// CISyncDocument
class CISyncDocument : public CEikDocument
{
public:
	CISyncDocument(CEikApplication& aApp);
	~CISyncDocument();
	static CISyncDocument* NewL(CEikApplication& aApp);
	void ConstructL();
	void ResetModelL();
	CModel* Model()					{return iModelPtr;}

	void	SaveData();
	void	RestoreData();

private: // from CEikDocument

	CEikAppUi*		CreateAppUiL();
	CModel*			iModelPtr;
	RFs fs;

};

//======================================================================
// CISyncApplication
class CISyncApplication : public CEikApplication
{
private: // from CApaApplication
	CApaDocument* CreateDocumentL();
	TUid AppDllUid() const;
};

#endif //_ISYNCUI_H
