/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  osdefs_ep.c
 *
 * Description:
 *    Common source file for OS-specific definitions
 *
 */

#include "osdefs.h"

#define MAX_STRING_LEN  500

char* TcharToChar(isy_TCHAR* tchar_str, char* char_str)
{
   int i, len;
  
   len = TSTR_LEN(tchar_str);
   for (i = 0; i <= len && i<MAX_STRING_LEN; i++)
      char_str[i]=(char) tchar_str[i];  
   return char_str;
}

//EPOC has no itoa function
void reverse(char s[])
{
	int i, j;
	char c;

	for (i=0, j = strlen(s)-1; i<j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}

char* itoa(char s[], int n)
{
	int i, sign;

	if ((sign = n) < 0)
		n = -n;

	i=0;
	do {
		s[i++] = (char) (n % 10 + '0');
	} while ((n /= 10) > 0);
	if (sign < 0)
		s[i++] = '-';
	s[i] = '\0';
	reverse(s);

	return s;
}

long max(long num1, long num2)
{
	if (num1 > num2)
		return num1;

	return num2;
}

long min(long num1, long num2)
{
	if (num1 < num2)
		return num1;

	return num2;
}

#if defined(UNDER_EP) && defined(_UNICODE)

wchar_t* wcschr(const wchar_t* pStr, wchar_t chr)
{
	while (*pStr != L'\0')
	{
		if (*pStr == chr)
			return (wchar_t*) pStr;

		pStr++;
	}
	return NULL;
}


wchar_t* wcsrchr(const wchar_t* pStr, wchar_t chr)
{
	const wchar_t *s = NULL;

	while (*pStr != L'\0')
	{
		if (*pStr == chr)
			s = pStr;

		pStr++;
	}
	return (wchar_t*) s;
}

wchar_t* wcsstr(const wchar_t* pStr, const wchar_t* pStr2)
{
	wchar_t* s = NULL;
	wchar_t* t = NULL;	
	
	while(*pStr)
	{
		if (*pStr == *pStr2)
		{
			s = (wchar_t *) pStr;
			t = (wchar_t *) pStr2;
			
			while(*s == *t)
			{
				if (*t == L'\0')
					break;
		
				s++;
				t++;			
			}
			if (*t == L'\0')
				return (wchar_t*) pStr;
		}
		pStr++;
	}
	return NULL;
}

wchar_t* wcsncpy(wchar_t* pDst, const wchar_t* pSrc, long cb)
{
	wchar_t* dest = pDst;

	while (cb-- > 0)
	{
		*dest++ = *pSrc++;	
		if (*(pSrc-1) == L'\0')
		{
			//pad remainder with zeroes
			while (cb-- > 0)
				*dest++ = L'\0';
			break;
		}
	}
	return pDst;
}

wchar_t* wcsncat(wchar_t* pDst, const wchar_t* pSrc, long cb)
{
	wchar_t* dest = pDst;

	while (*dest) dest++;

	while (cb-- > 0)
	{
		if (*pSrc == L'\0')
			break;
		*dest++ = *pSrc++;
	}
	
	//always null terminate
	*dest = L'\0';
	return pDst;
}

int wcsicmp(const wchar_t* pDst, const wchar_t* pSrc)
{
	while (towlower(*pDst) == towlower(*pSrc))
	{
		if (*pDst == L'\0')
			return 0;
			
		pDst++;
		pSrc++;
	}
	return towlower(*pDst) - towlower(*pSrc);
}

int wcsnicmp (const wchar_t* pDst, const wchar_t* pSrc, long cb)
{
	while (cb-- > 0)
	{
		if (towlower(*pDst) != towlower(*pSrc))
			return towlower(*pDst) - towlower(*pSrc);
			
		if (*pDst == L'\0')
			break;
			
		pDst++;
		pSrc++;			
	}
	return 0;
}


wchar_t* IntToWstr(wchar_t* s, int n)
{
	char tp1[MAX_STRING_LEN] = {0};
	int i = 0, len = 0;

	STR_ITOA(tp1, n);
	len = STR_LEN(tp1);
	for (i=0; i<len; i++)
	{
		s[i] = (wchar_t) tp1[i];
	}
	s[i] = L'\0';
	return s;
}

int WstrToInt(const wchar_t* s)
{
	char tp1[MAX_STRING_LEN] = {0};
	return (int) STR_ATOI((const char*) isy_C((wchar_t*) s, tp1));
}

void  EPOC_ASSERT_V6(int expr, isy_TCHAR* msg)
{
	char tp1 [MAX_STRING_LEN] = {0};
	if (!expr)
	{
		//TODO: wrap the C++ panic
		printf((char*) isy_C(msg, tp1));
		printf("\n");
		exit(-1);
	}
}
#endif

void EPOC_ASSERT(int expr, char* msg)
{
	if (!expr)
	{
		//TODO: wrap the C++ panic
		printf(msg);
		printf("\n");
		exit(-1);
	}
}

