/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  applicationForm.cpp
 *
 * Description:
 *    Displays subscription set information
 *    User can enable/disable/reset subscription sets 
 */

#include "stdafx.h"
#include "applicationForm.h"
#include "db2sync.h"

#define ColSize		15
#define SEPCHARS	TEXT("\t\n")
extern HINSTANCE	hInst;       // The current instance, extern from db2sync.cpp

isy_TCHAR appDetailsName[ISCLEN_SubsSetName];
isy_TCHAR appDetailsID[ISCLEN_SubsSetID];

#define ISYNCMODE_Skip                          0x0000
#define ISYNCMODE_Resume                        0x0004

/**********************************************************************************
 Message handler for Application Details box
 *********************************************************************************/
LRESULT CALLBACK ApplicationDetails(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int NewPosX = 0, NewPosY = 0;
	isy_INT32	status;
	GET_APP_

	switch (message)
	{
		case WM_INITDIALOG:
			SetWindowText(GetDlgItem (hDlg, IDC_GROUP_DETAILS), ISYNCSTR_Details);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_SUBSSET), ISYNCSTR_SubsSet_Abv);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_STATUS), ISYNCSTR_Status);
			SetWindowText(GetDlgItem (hDlg, IDC_CHECK_RESET), ISYNCSTR_Reset);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			SetDlgItemText(hDlg, IDC_EDIT_SUBSSET, appDetailsName);
			
			status = iscConfigGetSubsSetStatus(_pApp->hSyncConf, appDetailsID);
			switch (status)
			{
				case ISCRTN_Succeeded :
					SetDlgItemText(hDlg, IDC_EDIT_STATUS, ISYNCSTR_Succeeded);
					break;
				case ISCRTN_Canceled:
					SetDlgItemText(hDlg, IDC_EDIT_STATUS, ISYNCSTR_Cancelled);
					break;
				case ISCRTN_Failed:
					SetDlgItemText(hDlg, IDC_EDIT_STATUS, ISYNCSTR_Failed);
					break;
				case ISCRTN_Ready:
					SetDlgItemText(hDlg, IDC_EDIT_STATUS, ISYNCSTR_Ready);
					break;
				default:
					break;			
			}
			return centerHandle(hDlg, NewPosX, NewPosY, 0 , 0);

		case WM_COMMAND:
			switch (LOWORD(wParam))
			{
				case IDOK:
				{
					//check reset subscription set checkbox
					long ret = SendDlgItemMessage(hDlg, IDC_CHECK_RESET, BM_GETCHECK, 0, 0);
					if (ret == BST_CHECKED) 
						iscConfigResetSubsSet(_pApp->hSyncConf, appDetailsID);
					//end reset subscription set checkbox

					EndDialog(hDlg, LOWORD(wParam));
					return TRUE;
				}
				case IDCANCEL:
					EndDialog(hDlg, LOWORD(wParam));
					return TRUE;
			}
	}
    return FALSE;
}

/**********************************************************************************
 Message handler for Application box.
 *********************************************************************************/
LRESULT CALLBACK Application(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
		int NewPosX, NewPosY;
		isy_TCHAR selectedApp[ISYNCLEN_MiscInfo];
		HWND hwndAppListCtrl;
		isy_TCHAR URL[ISYNCLEN_URL];
       
		static int selectedMode;

		int i = 0, rc = 0;
		selectedApp[0] = 0;

		switch (message)
		{
			case WM_INITDIALOG:
			{
					SetWindowText(GetDlgItem (hDlg, IDC_GROUP_SETTINGS), ISYNCSTR_SubsSet_Settings);
					SetWindowText(GetDlgItem (hDlg, IDC_STATIC_SERVER), ISYNCSTR_Server);
					SetWindowText(GetDlgItem (hDlg, IDC_STATIC_SUBSSET), ISYNCSTR_SubsSet_Abv);
					SetWindowText(GetDlgItem (hDlg, IDC_STATIC_STATUS), ISYNCSTR_Status);
					SetWindowText(GetDlgItem (hDlg, IDC_RADIO_DISABLE), ISYNCSTR_Disable);
					SetWindowText(GetDlgItem (hDlg, IDC_RADIO_ENABLE), ISYNCSTR_Enable);
					SetWindowText(GetDlgItem (hDlg, IDC_BUTTON_DETAILS), ISYNCSTR_Details);
					SetWindowText(GetDlgItem (hDlg, IDC_BUTTON_PURGE), ISYNCSTR_Purge);
					SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);

                    rc = syncStartup(NULL);
					*appDetailsName = '\0';
					*appDetailsID = '\0';

					GET_APP_

					if (rc == ISCRTN_Succeeded)
					{

						appGetPref(ISYNCPREF_URL, URL, ISYNCLEN_URL);
						SetDlgItemText(hDlg, IDC_SERVER, URL);
         
						hwndAppListCtrl = GetDlgItem (hDlg, IDC_APPLICATION_LIST);
						i = SendMessage (hwndAppListCtrl, LB_GETCURSEL, 0, 0);
						if ( i == LB_ERR ) 
						{
							AppModeRadioButtonAllEnable(hDlg, false);
						}
                     
						refreshApplicationControls(hDlg);       // get stored values

					}
					
					syncShutdown();
					// try to center the Application dialog and then return
					return centerHandle(hDlg, NewPosX, NewPosY,0,0);
			}
			case WM_COMMAND:
			{
					rc = syncStartup(NULL);
					GET_APP_
					hwndAppListCtrl = NULL;       // Window handle to the application list box

					if (rc != ISCRTN_Succeeded)
					{
						//todo - display error

						EndDialog(hDlg, LOWORD(wParam));
						syncShutdown();
						return TRUE;
					}

					if (HIWORD (wParam) == LBN_SELCHANGE) {
						displaySelectedAppMode(hDlg);
					}

					switch (LOWORD(wParam))
					{
                            case IDC_RADIO_DISABLE:
                                   setAppMode(hDlg,  ISYNCMODE_Skip);
                                   break;

                            case IDC_RADIO_ENABLE:
                                   setAppMode(hDlg,  ISYNCMODE_Synchronize);
                                   break;
                            
                            case IDC_REFRESH:       
                                   setAppMode(hDlg,  ISYNCMODE_Refresh);
                                   break;

                            case IDC_RESUME:
                                   setAppMode(hDlg,  ISYNCMODE_Resume);
                                   
                                   break;
							case IDC_BUTTON_PURGE:
									rc = MessageBox(hDlg, ISYNCSTR_ResetConfig, isy_T(""), MB_OKCANCEL|MB_ICONWARNING|MB_TOPMOST);
									if (rc == IDCANCEL) break;
									iscConfigPurge(_pApp->hSyncConf);
									syncShutdown();
									EndDialog(hDlg, LOWORD(wParam));
									return TRUE;

							case IDC_BUTTON_DETAILS:
									if (*appDetailsName != '\0' && *appDetailsID != '\0')
										DialogBox(hInst, (LPCTSTR)IDD_APPLICATION_DETAILS, hDlg, (DLGPROC)ApplicationDetails);
									break;

                            case IDOK:
                                   EndDialog(hDlg, LOWORD(wParam));
								   syncShutdown();
                                   return TRUE;
                            case IDCANCEL:
                                   EndDialog(hDlg, LOWORD(wParam));
								   syncShutdown();
                                   return TRUE;
					}
					break;
			}
		} //end switch
	
    return FALSE;
}


/**********************************************************************************
 refreshApplicationControls
 Add the contents of the sync engine into application list control.
 List box should be emptied prior to calling.
 *********************************************************************************/
isy_INT refreshApplicationControls(HWND hDlg)
{
		HWND hwndAppListCtrl;       // Window handle to the app list box

		isy_TCHAR id[ISCLEN_SubsSetID];
		isy_TCHAR appName[ISCLEN_SubsSetName];
		isy_TCHAR appDescription[2*ISCLEN_SubsSetName+MODE_NAME_MAXLEN+ColSize];

		unsigned long mode = 999;
		int nextAppRlt = 0; // no apps
		int iCurrApp = 0;
		int currMode = 0;
		int len,i;

		HISCCSR	hCursor;
		GET_APP_

		// Get the window handle to the application list box control in the dialog box.
		hwndAppListCtrl = GetDlgItem (hDlg, IDC_APPLICATION_LIST);
		SendMessage (hwndAppListCtrl, LB_SETTABSTOPS, 0, 00);

		iscConfigOpenCursor(_pApp->hSyncConf, &hCursor);
		while( iscConfigGetNextSubsSet(_pApp->hSyncConf, hCursor, id, appName) == ISCRTN_Succeeded )
		{
			if (TSTR_CMP(appName,isy_T("Configuration"))!=0)
			{
				// add the prev app into list:
				TSTR_CPY(appDescription, appName);
				
				len = TSTR_LEN(appDescription);
				if (len>ColSize)
				{    
					// shrink the long app name
				    appDescription[ColSize-2]=0;
					TSTR_CAT(appDescription,isy_T("..."));
				}
				for (i=0;i<ColSize-len;i++)
					TSTR_CAT(appDescription, isy_T(" "));

                TSTR_CAT(appDescription, isy_T("\t"));

				if (iscConfigSubsSetIsEnabled(_pApp->hSyncConf, id))
					TSTR_CAT(appDescription, ISYNCSTR_Enabled);
				else
					TSTR_CAT(appDescription, ISYNCSTR_Disabled);
				TSTR_CAT(appDescription, isy_T("\t\t"));
                TSTR_CAT(appDescription, appName);
                iCurrApp = SendMessage (hwndAppListCtrl, LB_ADDSTRING, 0, (LPARAM)(LPCTSTR) appDescription);
              }
		}
		iscConfigCloseCursor(_pApp->hSyncConf, hCursor);
		return 0;
}


/**********************************************************************************
 displaySelectedAppMode
 Updates the mode controls to reflect the current stored application mode.
 *********************************************************************************/
isy_VOID displaySelectedAppMode(HWND hDlg) 
{
		HWND hwndAppSyncOpt = NULL;       // Window handle to the radio buttons
  
		int nextAppRlt = 0;
		int currModeO = 0;
		isy_UINT32 mode = 0;
		isy_TCHAR appName[ISCLEN_SubsSetName];
		isy_TCHAR selectedApp[ISCLEN_SubsSetName];
		int modeSet = 0;

		isy_TCHAR id[ISCLEN_SubsSetID];
		HISCCSR	hCursor;
		GET_APP_
		isy_INT32 enabled = ISCRTN_False;

		if (GetSelectedAppName(hDlg,selectedApp)==0)
			return;

		iscConfigOpenCursor(_pApp->hSyncConf, &hCursor);
		while( iscConfigGetNextSubsSet(_pApp->hSyncConf, hCursor, id, appName) == ISCRTN_Succeeded )
		{
			if (TSTR_CMP(selectedApp, appName) == 0)
			{
				enabled = iscConfigSubsSetIsEnabled(_pApp->hSyncConf, id);
				TSTR_CPY(appDetailsName, selectedApp);
				TSTR_CPY(appDetailsID, id);
				break;
			}
		}
		iscConfigCloseCursor(_pApp->hSyncConf, hCursor);

		// Enable radio buttons
		AppModeRadioButtonAllEnable(hDlg, true);

		// uncheck all radio buttons:
		AppModeRadioButtonAllUncheck(hDlg);

		if (enabled == ISCRTN_True)
			hwndAppSyncOpt = GetDlgItem (hDlg, IDC_RADIO_ENABLE);
		else
			hwndAppSyncOpt = GetDlgItem (hDlg, IDC_RADIO_DISABLE);

		// change radio to the current mode:
		currModeO = SendMessage (hwndAppSyncOpt, BM_SETCHECK, BST_CHECKED, 0);
}

/**********************************************************************************
 setAppMode
 Sets the application mode to the engine
 *********************************************************************************/
isy_VOID setAppMode(HWND hDlg,  isy_INT32 ISYNCMODE) 
{
		HWND hwndAppListCtrl;
       
		isy_TCHAR selectedApp[ISCLEN_SubsSetName];
		int i = 0;
		int ret = 0;
		isy_INT32 selectedMode = ISYNCMODE;

		isy_TCHAR id[ISCLEN_SubsSetID];
		isy_TCHAR appName[ISCLEN_SubsSetName];
		HISCCSR	hCursor;
		GET_APP_

		if (GetSelectedAppName(hDlg,selectedApp)==0)
			return;

		iscConfigOpenCursor(_pApp->hSyncConf, &hCursor);
		while( iscConfigGetNextSubsSet(_pApp->hSyncConf, hCursor, id, appName) == ISCRTN_Succeeded )
		{
			if (TSTR_CMP(selectedApp, appName) == 0)
				break;
		}
		iscConfigCloseCursor(_pApp->hSyncConf, hCursor);

		if (ISYNCMODE == ISYNCMODE_Skip)
			iscConfigDisableSubsSet(_pApp->hSyncConf, id);
		else
			iscConfigEnableSubsSet(_pApp->hSyncConf, id);

		// Update the current listbox entry, losing any resume modes.
		hwndAppListCtrl = GetDlgItem (hDlg, IDC_APPLICATION_LIST);	// get handle for the control
		i = SendMessage (hwndAppListCtrl, LB_GETCURSEL, 0, 0);
		SendMessage (hwndAppListCtrl, LB_RESETCONTENT, 0, 0);		// clear current list
		refreshApplicationControls(hDlg);
		SendMessage (hwndAppListCtrl, LB_SETCURSEL, i, 0);      
		EnableWindow( GetDlgItem (hDlg, IDC_RESUME), false );
}

/********************************************************************
 AppModeRadioButtonAllEnable
 Enable or disable all Sync mode radio buttons.
 ********************************************************************/
LRESULT AppModeRadioButtonAllEnable(HWND hDlg, isy_BOOL bl) {
       
       HWND hwndAppSyncOpt = NULL;
       LRESULT       error = false;

       hwndAppSyncOpt = GetDlgItem (hDlg, IDC_RADIO_DISABLE);
       error |= EnableWindow( hwndAppSyncOpt, bl );
       
       hwndAppSyncOpt = GetDlgItem (hDlg, IDC_RADIO_ENABLE);
       error |= EnableWindow( hwndAppSyncOpt, bl );
       return error;
}

/********************************************************************
 AppModeRadioButtonAllUncheck
 Uncheck all Application mode radio buttons.
 ********************************************************************/
LRESULT AppModeRadioButtonAllUncheck(HWND hDlg) {

       HWND hwndAppSyncOpt = NULL;

       hwndAppSyncOpt = GetDlgItem (hDlg, IDC_RADIO_DISABLE);
       SendMessage (hwndAppSyncOpt, BM_SETCHECK, BST_UNCHECKED, 0);
       hwndAppSyncOpt = GetDlgItem (hDlg, IDC_RADIO_ENABLE); 
       SendMessage (hwndAppSyncOpt, BM_SETCHECK, BST_UNCHECKED, 0);
       return 0;
}

/********************************************************************
  Get selected application name from list box
 ********************************************************************/
//return 0 name is not obtained
//       1 is obtained
isy_INT GetSelectedAppName(HWND hDlg, isy_TCHAR *selectedApp)
{
		HWND hwndAppListCtrl = NULL;       // Window handle to the application list box
       
		long appIndex = 0;
		long appNameLen = 0;
		isy_TCHAR selectedAppDescription[2*ISCLEN_SubsSetName+MODE_NAME_MAXLEN+ColSize];
		isy_TCHAR *p;
		int ret = 0;

		//isy_TCHAR szStr1[ISYNCLEN_MiscInfo];
		//isy_TCHAR szStr2[ISYNCLEN_MiscInfo];

		hwndAppListCtrl = GetDlgItem (hDlg, IDC_APPLICATION_LIST); // get applist handle
		appIndex = SendMessage (hwndAppListCtrl, LB_GETCURSEL, 0, 0);
		if ( appIndex == LB_ERR ) {

			//ret = LoadString(hInst, IDS_MSGBOX_TITLE_NOTICE, szStr2, ISYNCLEN_MiscInfo);
			//ret = LoadString(hInst, IDS_UNKNOWN_APP_MODE, szStr1, ISYNCLEN_MiscInfo);                     
			//MessageBox(hDlg, szStr1, szStr2, MB_OK);

			MessageBox(hDlg, isy_T("An unknown application mode has been selected."), NULL, MB_OK|MB_ICONEXCLAMATION|MB_TOPMOST);
			return 0; // nothing selected
		}
       
		SendMessage (hwndAppListCtrl, LB_GETTEXT, appIndex,
			(LPARAM) selectedAppDescription);

		p = _tcsstr(selectedAppDescription,isy_T("\t\t"));
		if (p==0) return 0;
		TSTR_CPY(selectedApp,p+2);
		return 1;
}
