/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  db2sync.cpp
 *
 * Description:
 *    Main application window
 *
 */

#include "stdafx.h"
#include "db2sync.h"
#include "settingForm.h"
#include "applicationForm.h"
#include "syncForm.h"
#include "logDisplay.h"

#include <stdio.h>
#include <string.h>
#include <commctrl.h>

#if defined(WIN32_PLATFORM_PSPC)
#include <aygshell.h>
#endif

/**********************************************************************************
 Global Variables:
*********************************************************************************/
HWND				hWnd;           //main window
HWND				hWndText;
HINSTANCE			hInst;			// The current instance
HWND				hwndCB;			// The command bar handle
HISCENG				hISync;			// The IBM Sync handle
HWND                hwndGSyncButton;

isy_BOOL			bWEASupport = FALSE;
isy_BOOL			bSyncOnly = FALSE;
isy_TCHAR			szURL[ISYNCLEN_URL], 
					szUserID[ISYNCLEN_User], 
					szPassword[ISYNCLEN_Password],
					szDBDir[MAX_PATH]; 
isy_INT32			apprc = ISCRTN_Succeeded;
isy_INT32			SilentMode = 0;	//Yamato, silent mode

isy_TCHAR			szTitle[ISYNCLEN_MiscInfo];			// The title bar text
isy_TCHAR			szWindowClass[ISYNCLEN_MiscInfo];	// The window class name


#define IDD_WIN_TEXT 100
#define MAXENV  1024

#define BUTTONCLASS	       isy_T("button")
#define BUTTONWIDTH		   85
#define BUTTONHEIGHT       45
#define BUTTONOFFSETLEFT   1
#define BUTTONOFFSETTOP    2 * 8
#define ID_SYNC_BUTTON     0x8802

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass	(HINSTANCE, LPTSTR);
BOOL				InitInstance	(HINSTANCE, int);
LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About			(HWND, UINT, WPARAM, LPARAM);

//command line options
enum
{
	CMD_NONE,
	CMD_SETTINGS_ONLY,
	CMD_SUB_SETS_ONLY,
	CMD_SYNC_ONLY,
	CMD_SHOWLOG_ONLY
};
isy_INT32 cmdOption = CMD_NONE;

#define CMDSTR_SETTINGS_ONLY	isy_T("-settings_only")
#define CMDSTR_SUB_SETS_ONLY	isy_T("-subscription_sets_only")
#define CMDSTR_SYNC_ONLY		isy_T("-sync_only")
#define CMDSTR_SHOWLOG_ONLY		isy_T("-showlog")

extern void startSync(HWND hwnd);

#if defined(WIN32_PLATFORM_PSPC)
HWND CreateEmptyMenuBar(HWND hWnd, UINT menuID);
#endif

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;

	TCHAR szCommand[128];

	// Initialize global strings
	//LoadString(hInstance, IDS_ISYNC, szWindowClass, ISYNCLEN_MiscInfo);
	//LoadString(hInstance, IDS_APP_TITLE, szTitle, ISYNCLEN_MiscInfo);
	TSTR_CPY(szWindowClass, isy_T("DB2 Sync"));
	TSTR_CPY(szTitle, isy_T("DB2 Sync"));

	// Look for running instance of application
	if (FindWindow(szWindowClass, szTitle))
	{
		SetForegroundWindow(FindWindow(szWindowClass, szTitle));
		return ISCRTN_Failed;
	}

	if( _tcslen(lpCmdLine))
	{
		_stscanf( lpCmdLine, _T("%s %s %s %s"), szCommand, szURL, szUserID, szPassword);
		TCHAR* p=lpCmdLine;

		while( *p!=0 && *p!=_T('\"')) p++;
		if( *p == 0)
		{
			szDBDir[0] = _T('\\');
			szDBDir[1] = 0;
		}
		else
		{
			p++;
			_tcscpy( szDBDir, p);
			p=szDBDir;
			while(*p!=0 && *p!=_T('\"')) p++;
			if(*p != 0) *p=0;
		}

		if( _tcscmp( szDBDir, _T("\\")))
		{
			/*
			** Try to make directories
			*/
			p=szDBDir;

			while(*p)
			{
				while( *p!=0 && *p!=_T('\\')) p++;
				
				if(*p)
				{
					*p=0;
					FMKDIR(szDBDir);
					*p++=_T('\\');
				}
			};
		}
				
		if( (TSTR_I_CMP(szCommand, CMDSTR_SYNC_ONLY) == 0)
		 && _tcslen( szUserID))
		{
			cmdOption = CMD_SYNC_ONLY;
			bWEASupport = TRUE;
			bSyncOnly = TRUE;
		}
		else if (TSTR_I_CMP( szCommand, CMDSTR_SETTINGS_ONLY) == 0)
		{
			cmdOption = CMD_SETTINGS_ONLY;
			bWEASupport = TRUE;
		}
		else if (TSTR_I_CMP(lpCmdLine, CMDSTR_SUB_SETS_ONLY) == 0)
			cmdOption = CMD_SUB_SETS_ONLY;
		else if (TSTR_I_CMP(lpCmdLine, CMDSTR_SHOWLOG_ONLY) == 0)
			cmdOption = CMD_SHOWLOG_ONLY;
	}

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return ISCRTN_Failed;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR) IDS_ISYNC);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) 
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	return (int) msg.wParam;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    It is important to call this function so that the application 
//    will get 'well formed' small icons associated with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_ISYNC));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

//
//  FUNCTION: InitInstance(HANDLE, int)
//
//  PURPOSE: Saves instance handle and creates main window
//
//  COMMENTS:
//
//    In this function, we save the instance handle in a global variable and
//    create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
	GET_APP_
	
	FMKDIR(isy_T("\\Program Files\\"));
	FMKDIR(isy_T("\\Program Files\\ISync\\"));

	appInitialize();

	hInst = hInstance;		// Store instance handle in our global variable
	
	MyRegisterClass(hInstance, szWindowClass);
	//(cmdOption == CMD_SYNC_ONLY) ? NULL : 
	hWnd = CreateWindow(szWindowClass, szTitle, WS_VISIBLE,
		CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

	if (!hWnd)
		return FALSE;

	//ShowWindow(hWnd, nCmdShow);
	ShowWindow(hWnd, (cmdOption == CMD_SYNC_ONLY) ? SW_HIDE : nCmdShow);
	UpdateWindow(hWnd);
	if (hwndCB) {
		CommandBar_Show(hwndCB, TRUE);
	}

	if( (bSyncOnly == TRUE) && (bWEASupport == TRUE))
	{
		appSetPref(ISYNCPREF_URL,       szURL);
		appSetPref(ISYNCPREF_TargetDir, szDBDir);
		if (_tcslen(szUserID)) {
			TSTR_CPY(appPrefUser(_pApp), szUserID);
			TSTR_CPY(appPrefPassword(_pApp), szPassword);
			appSetPref(ISYNCPREF_User, szUserID);
			appSetPref(ISYNCPREF_Password, szPassword);
		}
	}

	switch (cmdOption)
	{
		case CMD_SETTINGS_ONLY:
			DialogBox(hInst, (LPCTSTR)IDD_SERVER_SETTINGS_BOX, hWnd, (DLGPROC) ServerSettings);
			DialogBox(hInst, (LPCTSTR)IDD_APPLICATION, hWnd, (DLGPROC)Application);
			DestroyWindow(hWnd);
			break;
		case CMD_SUB_SETS_ONLY:
			DialogBox(hInst, (LPCTSTR)IDD_APPLICATION, hWnd, (DLGPROC)Application);
			DestroyWindow(hWnd);
			break;
		case CMD_SYNC_ONLY:
			// Yamato, Silent mode			
			//DialogBox(hInst, (LPCTSTR)IDD_SYNC_STATUS_SCREEN, hWnd, (DLGPROC)SyncDialog);
			CreateDialog(hInst, (LPCTSTR)IDD_SYNC_STATUS_SCREEN, hWnd, (DLGPROC)SyncDialog);
			SilentMode = 1;
//			DestroyWindow(hWnd);
			break;
		case CMD_SHOWLOG_ONLY:
			DialogBox(hInst, (LPCTSTR)IDD_VIEW_LOG, hWnd, (DLGPROC)LogDisplay);
			DestroyWindow(hWnd);
			break;

		default:
			break;
	}

	return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
// My notes: All messages that are for the main window will pass through
// this code. On a message match, via case statement, the appropriate call
// back function will be called to take care of the event.
// Further input may be handled by the callback functions.

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	int wmId, wmEvent;
	PAINTSTRUCT ps;
	int ret = 0;
	int syncbuttonX = 0;
	int syncbuttonY = 0;
	TEXTMETRIC  tm;
	isy_TCHAR szSyncButton[ISYNCLEN_MiscInfo];

	switch (message) 
	{
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				// File Menu:
				case IDM_FILE_SERVER_SETTINGS:
				   DialogBox(hInst, (LPCTSTR)IDD_SERVER_SETTINGS_BOX, hWnd, (DLGPROC) ServerSettings);
				   break;
				case IDM_FILE_CLIENT_SETTINGS:
				   DialogBox(hInst, (LPCTSTR)IDD_CLIENT_SETTINGS_BOX, hWnd, (DLGPROC) ClientSettings);
				   break;
				case IDM_FILE_NETWORK_SETTINGS:
				   DialogBox(hInst, (LPCTSTR)IDD_NETWORK_SETTINGS_BOX, hWnd, (DLGPROC) NetworkSettings);
				   break;

				case IDM_FILE_APPLICATION:
					DialogBox(hInst, (LPCTSTR)IDD_APPLICATION, hWnd, (DLGPROC)Application);
				   break;
				case IDM_FILE_SYNC:
				   DialogBox(hInst, (LPCTSTR)IDD_SYNC_STATUS_SCREEN, hWnd, (DLGPROC)SyncDialog);
				   break;
				case IDM_VIEW_LOG:
				   DialogBox(hInst, (LPCTSTR)IDD_VIEW_LOG, hWnd, (DLGPROC)LogDisplay);
				   break;
				case IDM_FILE_EXIT:
				   DestroyWindow(hWnd);
				   break;

				case ID_SYNC_BUTTON: // sync button

					if (!isUserEntered())
					{
						MessageBox(hWnd, ISYNCSTR_NotSet, NULL, MB_OK|MB_ICONERROR|MB_TOPMOST);
						break;
					}
/*
					if (!isPasswordEntered())
					{
						ret = DialogBox(hInst, (LPCTSTR)IDD_PASSWORD, hWnd, (DLGPROC)EnterPassword);
						if (ret == 2) break; // 2 to indicate cancel in password dialog
					}
*/
					DialogBox(hInst, (LPCTSTR)IDD_SYNC_STATUS_SCREEN, hWnd, (DLGPROC)SyncDialog);
					break;

				// Help Menu:				   
				case IDM_HELP_ABOUT:
				   DialogBox(hInst, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)About);
				   break;

				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CREATE:
			#if defined(WIN32_PLATFORM_PSPC)
				CreateEmptyMenuBar(hWnd, 1234);
			#endif
			hwndCB = CommandBar_Create(hInst, hWnd, 1);
			//CommandBar_InsertMenubar(hwndCB, hInst, IDM_MENU, 0);
			//CommandBar_AddAdornments(hwndCB, 0, 0);

			//create menus
			{
				HMENU hMainMenu = CreateMenu();

				HMENU fileMenu = CreatePopupMenu();
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_FILE_SERVER_SETTINGS, ISYNCSTR_Server_Settings);
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_FILE_CLIENT_SETTINGS, ISYNCSTR_Client_Settings);
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_FILE_NETWORK_SETTINGS, ISYNCSTR_Network_Settings);
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_FILE_APPLICATION, ISYNCSTR_SubsSet_Settings);
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_FILE_SYNC, ISYNCSTR_Synchronize);
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_VIEW_LOG, ISYNCSTR_Log);
				AppendMenu(fileMenu, MF_SEPARATOR , NULL, NULL);
				AppendMenu(fileMenu, MF_STRING | MF_ENABLED, IDM_FILE_EXIT, ISYNCSTR_Exit);
				AppendMenu(hMainMenu, MF_STRING| MF_ENABLED | MF_POPUP, 
				 (UINT)fileMenu, ISYNCSTR_File);

				HMENU helpMenu = CreatePopupMenu();
				AppendMenu(helpMenu, MF_STRING | MF_ENABLED, IDM_HELP_ABOUT, ISYNCSTR_About);
				AppendMenu(hMainMenu, MF_STRING| MF_ENABLED | MF_POPUP, 
				 (UINT)helpMenu, ISYNCSTR_Help);

				CommandBar_InsertMenubarEx(hwndCB, 0, (LPTSTR)hMainMenu, 0);
				CommandBar_AddAdornments(hwndCB, 0, 0);
			}


			hdc = GetDC (hWnd) ;
            GetTextMetrics (hdc, &tm) ;
            ReleaseDC (hWnd, hdc) ;

			hWndText = CreateWindow (isy_T("static"), NULL,
                              WS_CHILD | WS_VISIBLE | SS_LEFT,
                              tm.tmAveCharWidth,          tm.tmHeight,
                              tm.tmAveCharWidth * MAXENV, tm.tmHeight,
                              hWnd, (HMENU) IDD_WIN_TEXT,
                              hInst, NULL) ;
	
			setHandlePos(hWndText, 0, 30);

			// CreateWindow()s Buttons runtime are their own window. Buttons are type hwnd.
			// Create a default push button.
			//ret = LoadString(hInst, IDS_SYNCHRONIZE, szSyncButton, ISYNCLEN_MiscInfo);
			TSTR_CPY(szSyncButton, ISYNCSTR_Synchronize);

			hwndGSyncButton = CreateWindow( BUTTONCLASS,  szSyncButton, 
                         WS_CHILD | WS_VISIBLE | BS_DEFPUSHBUTTON, 
                         BUTTONOFFSETLEFT, 
                         BUTTONOFFSETTOP, BUTTONWIDTH+50, // 50 quickie fix.
                         BUTTONHEIGHT, hWnd,
                         (HMENU) ID_SYNC_BUTTON,
                         hInst, NULL );
			syncbuttonY=0;
			centerHandle(hwndGSyncButton, syncbuttonX, syncbuttonY,0 ,0);

			break;
		case WM_PAINT:
			hdc = BeginPaint(hWnd, &ps);
			EndPaint(hWnd, &ps);
			break;
/*
		case WM_SIZE:
			if (wParam == SIZE_MINIMIZED) 
			{
				SetForegroundWindow(hWnd);
			}
			break;
*/
		case WM_DESTROY:
			appTerminate();
			CommandBar_Destroy(hwndCB);
			PostQuitMessage(apprc);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

/**********************************************************************************
 Message handler for About box.
 *********************************************************************************/
LRESULT CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int NewPosX = 0, NewPosY = 0;
	isy_INT32 rc = 0;
	isy_TCHAR about[ISYNCLEN_MiscInfo];
	GET_APP_

	switch (message)
	{
		case WM_INITDIALOG:

			SetWindowText(hDlg, ISYNCSTR_About);
			rc = syncStartup(NULL);
			if (rc == ISCRTN_Succeeded)
			{
				if (iscEngineGetInfo(_pApp->hSyncer, about, ISYNCLEN_MiscInfo) == ISCRTN_Succeeded) 
					SetDlgItemText(hDlg, IDC_EDIT_INFO, about);
			}
			syncShutdown();
			return centerHandle(hDlg, NewPosX, NewPosY, 0 , 0);

		case WM_COMMAND:
			if ((LOWORD(wParam) == IDOK) || (LOWORD(wParam) == IDCANCEL))
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}

#if defined(WIN32_PLATFORM_PSPC)
//*****************************************************************************
/**
 * Create a empty menu bar for the with SIP connected
 *
 * @param hWnd Handle of the main window
 * @param menuID Identity of menu
 *
 * @return Handle of the menubar
 */
HWND CreateEmptyMenuBar(HWND hWnd, UINT menuID)
{
	SHMENUBARINFO mbi;

	memset(&mbi, 0, sizeof(SHMENUBARINFO));
	mbi.cbSize     = sizeof(SHMENUBARINFO);
	mbi.hwndParent = hWnd;
	mbi.dwFlags    = SHCMBF_EMPTYBAR;
	//mbi.nToolBarId = menuID;
	mbi.hInstRes   = hInst;
	mbi.nBmpId     = 0;
	mbi.cBmpImages = 0;

	if (!SHCreateMenuBar(&mbi))
		return NULL;
		
	return mbi.hwndMB;
}
#endif
