/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  osdefs.h
 *
 * Description:
 *    Definitions for OS-specific routines
 **/

#ifndef _OSDEFS_CE_H_
#define _OSDEFS_CE_H_

#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <winsock.h>
#include <Winbase.h>
#include "osdefs.h"
#include <TCHAR.H>
#include <ctype.h>

// C++ marker
#ifdef __cplusplus
extern "C" {
#endif

// C has no built-in true or false
#define false 0
#define true 1

#define ISYNCDIR isy_T("\\Program Files\\iSync\\")
#define DATADIR isy_T("\\")

// Debug
#define OSS_ASSERT(expr, msg) WINCE_ASSERT(expr, msg)
void WINCE_ASSERT(int, TCHAR *);  //5452

void ceGetCurrentDateTime(int* py, int* pm, int* pd, int* phr, int* pmi, int *pse);
#define OSGET_CURRENT_DATE_TIME(y, m, d, hr, mi, se) ceGetCurrentDateTime(y, m, d, hr, mi, se)

// Memory management
#define MEM_HANDLE_ALLOC(size) malloc((unsigned long) (size))
#define MEM_HANDLE_RESIZE(handle, size)	realloc(handle, size)
#define MEM_HANDLE_FREE(handle) free((void*) (handle))
#define MEM_HANDLE_LOCK(handle) handle
#define MEM_HANDLE_UNLOCK(handle)
#define MEM_PTR_ALLOC(size) malloc((unsigned long) (size))
#define MEM_PTR_FREE(ptr) free((void*) (ptr))
#define MEM_SET(ptr, val, cb) memset((void*) (ptr), val, (unsigned long) (cb))
#define MEM_CPY(pTo, pFrom, cb) memcpy((void*) (pTo), (void*) (pFrom), (unsigned long) (cb))

#define MEM_HEAP_FREE_BYTES(id, free, max) MemHeapFreeBytes(id, free, max)
void MemHeapFreeBytes(int id, unsigned long* free, unsigned long* max);

// String manipulation
#define TSTR_LEN(psz) _tcslen((TCHAR*) (psz))
#define TSTR_CPY(pDst, pSrc) _tcscpy(pDst, pSrc)
#define TSTR_CAT(pDst, pSrc) _tcscat((TCHAR*) (pDst), (TCHAR*) (pSrc))
#define TSTR_N_CPY(pDst, pSrc, cb) _tcsncpy((TCHAR*) (pDst), (TCHAR*) (pSrc), cb)
#define TSTR_N_CAT(pDst, pSrc, cb) _tcsncat((TCHAR*) (pDst), (TCHAR*) (pSrc), cb)
#define TSTR_NI_CMP(pDst, pSrc, cb) _tcsnicmp((TCHAR*) (pDst), (TCHAR*) (pSrc), cb)
#define TSTR_I_CMP(pDst, pSrc) _tcsicmp((TCHAR*) (pDst), (TCHAR*) (pSrc))
#define TSTR_CMP(pDst, pSrc) _tcscmp((TCHAR*) (pDst), (TCHAR*) (pSrc)) // wide strcmp
#define TSTR_ITOA(buf, i) _itot(i, buf, 10) // _itoa for wide char
#define TSTR_ATOI(buf) _ttoi(buf) // wide char to int atoi
#define TSTR_CHR(pStr, chr) _tcschr((TCHAR*) (pStr), (int) (chr))
#define TSTR_STR(pStr, pStr2) _tcsstr((TCHAR*) pStr, (TCHAR*) pStr2)

#define STR_LEN(psz) strlen((char*) (psz))
#define STR_CPY(pDst, pSrc) strcpy(pDst, pSrc)
#define STR_CAT(pDst, pSrc) strcat((char*) (pDst), (char*) (pSrc))
#define STR_N_CPY(pDst, pSrc, cb) strncpy((char*) (pDst), (char*) (pSrc), cb)
#define STR_N_CAT(pDst, pSrc, cb) strncat((char*) (pDst), (char*) (pSrc), cb)
#define STR_NI_CMP(pDst, pSrc, cb) CE_strnicmp((char*) (pDst), (char*) (pSrc), cb) // mapping not fully supported
#define STR_I_CMP(pDst, pSrc) CE_stricmp((char*) (pDst), (char*) (pSrc)) // mapping not fully supported
#define STR_CMP(pDst, pSrc) strcmp((char*) (pDst), (char*) (pSrc)) // wide strcmp
#define STR_ITOA(buf, i) _itoa(i, buf, 10) // _itoa char
#define STR_ATOI(buf) atoi(buf) // char to int
#define STR_CHR(pStr, chr) strchr((char*) (pStr), (int) (chr))
#define STR_STR(pStr, pStr2) strstr((char*) pStr, (char*) pStr2)
#define STR_ISDIGIT(chr) '0'<=chr && chr <='9'    //isdigit((int) (chr))

#ifdef WIN32_PLATFORM_HPCPRO
#define isupper(c) ((c) >= (isy_TCHAR) 'A' && (c) <= (isy_TCHAR) 'Z')
#define islower(c) ((c) >= (isy_TCHAR) 'a' && (c) <= (isy_TCHAR) 'z')
#endif

long CE_stricmp(char* pDst, char* pSrc);
long CE_strnicmp(char* pDst, char* pSrc, long cb);

// File system
#define DECLARE_ERR_

#define FSEEK_BEGINNING SEEK_SET
#define FSEEK_CURRENT SEEK_CUR
#define FSEEK_END SEEK_END
#define FILE_NL isy_T("\r\n")
#define FPATH_SLASH		'\\'
#define FPATH_SLASHSTR	"\\"

#define HISYNCFILE  FILE*

#define FOPEN(fileName, openMode) _tfopen(fileName, openMode)  
#define FCLOSE(file)  fclose(file)  
#define FREAD(array, objSize, numObj, file) fread(array, objSize, numObj, file)
#define FWRITE(array, objSize, numObj, file) fwrite(array, objSize, numObj, file)
#define FSEEK(file, offset, origin) fseek(file, offset, origin)
#define REWIND(file) fseek(file, 0, SEEK_SET) // seek to start of file
#define REMOVE(fileName) (DeleteFile(fileName) == 0 ? 1 : 0)
#define FFLUSH(file) fflush(file)  
#define FMKDIR(dir) CreateDirectory(dir, NULL)

size_t WINCE_fread(void* lpBuffer, size_t nNumberOfBytesToRead, size_t lpNumberOfBytesRead, HANDLE hFile);
size_t WINCE_fwrite(void* lpBuffer, size_t nNumberOfBytesToWrite, size_t lpNumberOfBytesWritten, HANDLE hFile);

#ifdef __cplusplus
}
#endif

#endif // _OSDEFS_CE_H_
