/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  settingForm.cpp
 *
 * Description:
 *    Displays and retrieves user preferences
 *
 */

#include "stdafx.h"
#include "db2sync.h"
#include "settingForm.h"

#ifndef TEMP_TESTCODE
#define TEMP_TESTCODE

ISYNCPREFS prefs;
extern HWND				hWnd;		// from db2sync.cpp
extern HINSTANCE		hInst;		// The current instance, extern from db2sync.cpp
#endif

/*
** WEA Client support
*/
extern isy_BOOL			bWEASupport;
extern TCHAR			szURL[ISYNCLEN_URL],
						szUserID[ISYNCLEN_User], 
						szPassword[ISYNCLEN_Password],
						szDBDir[MAX_PATH];

static isy_UINT32 parseMsgSizeFromBaud(isy_TCHAR *selNetType, int sel);

#define MAXNUMSERVERS 8
struct ServerNode 
{
	isy_TCHAR server[ISYNCLEN_URL];
	struct ServerNode *prev, *next;
};

struct ServerList
{
	FILE* fp;
	isy_TCHAR fileName[ISYNCLEN_Target];
	int numNodes;
	ServerNode *front, *back;
};

ServerNode* createServerNode(isy_TCHAR *s);
ServerList* createServerList();
void addServerNode(ServerList* list, ServerNode* node);
void removeFrontServerNode(ServerList *list);
void destroyServerList(ServerList* list);

ServerNode* createServerNode(isy_TCHAR *s)
{
	ServerNode *node = (ServerNode*)MEM_PTR_ALLOC(sizeof(ServerNode));
	TSTR_CPY(node->server, s);
	node->prev = node->next = NULL;
	return node;
}

ServerList* createServerList()
{
	isy_TCHAR server[ISYNCLEN_URL];
	ServerNode *node = NULL;
	int len=0;

	ServerList *list = (ServerList*)MEM_PTR_ALLOC(sizeof(ServerList));
	list->front = list->back = NULL;
	list->numNodes = 0;
	TSTR_CPY(list->fileName, DATADIR);
	//TSTR_CPY(list->fileName, appPrefTargetDir(_pApp));
	TSTR_CAT(list->fileName, isy_T("server-isyn"));
	list->fp = FOPEN(list->fileName, isy_T("rb"));

	if (!list->fp) 
	{
		list->fp = FOPEN(list->fileName, isy_T("wb+"));

		if (!list->fp)
			return NULL;

		appGetPref(ISYNCPREF_URL, server, sizeof(server)/sizeof(isy_TCHAR));
		node = createServerNode(server);
		addServerNode(list, node);
		//todo: macro for fputs
		_fputts(server, list->fp);
		_fputts(isy_T("\n"), list->fp);
		FFLUSH(list->fp);
	}
	else 
	{
		//todo: macro for fgets
		while (_fgetts(server, ISYNCLEN_URL, list->fp) != NULL)
		{
			len = TSTR_LEN(server); 
			while ((len > 0) && ((server[len-1] == (isy_TCHAR) '\r') || (server[len-1] == (isy_TCHAR) '\n')))
				server[--len] = (isy_TCHAR) '\0';

			if (len > 0) {
				node = createServerNode(server);
				addServerNode(list, node);
			}
		}
	}

	FCLOSE(list->fp);
	return list;
}

void addServerNode(ServerList* list, ServerNode* node)
{
    node->prev = list->back;
    node->next = NULL;
    if (list->front) 
		list->back->next = node;
    else
		list->front = node;
        
	list->back = node;

	(list->numNodes)++;
	if (list->numNodes > MAXNUMSERVERS) removeFrontServerNode(list);
}

void removeFrontServerNode(ServerList *list)
{
	ServerNode *node = list->front;
	if (!list->front) return;
	list->front = list->front->next;
	if (list->front)
		list->front->prev = NULL;
	else 
		list->back = NULL;
		
	MEM_PTR_FREE(node);
	(list->numNodes)--;
}


void destroyServerList(ServerList* list)
{
	ServerNode *node = list->front, *next = NULL;
	while(node) 
	{
		next = node->next;
		MEM_PTR_FREE(node);
		node = next;
	}
	MEM_PTR_FREE(list);
}

static isy_UINT32 parseMsgSizeFromBaud(isy_TCHAR *selNetType, int sel)
{
	isy_TCHAR *space = TSTR_CHR(selNetType, (isy_TCHAR)' ');
	isy_TCHAR baudIntStr[12];
	long multiplier = 1024;
	isy_TCHAR *m;

	GET_APP_
	
	// 1st entry is "none" or disabled.  Protect against malformed formats
	if (sel == 0 || space == NULL || (space - selNetType)-1 > (sizeof(baudIntStr)/sizeof(isy_TCHAR)))
		return 0;
		
	{
		int len = space - selNetType, baudInt;
		
		// extract the baud int, e.g. 4, 56, 2 (Meg)
		TSTR_N_CPY(baudIntStr, selNetType, len);
		baudIntStr[len] = (isy_TCHAR)'\0';
		baudInt = TSTR_ATOI(baudIntStr);
		if (baudInt <= 0 || baudInt > 10 * 1000 * 1024)
			MessageBox(hWnd, isy_T("Internal network type Baud Rate"), NULL, MB_OK|MB_ICONERROR|MB_TOPMOST);
		
		// Now extract the multiplier, e.g. K, M
		for (m = space; *m != '\0'; m++)
		{
			if (*m == 'K' || *m == 'M')
				break;
		}
		if (*m == 'M')
			multiplier *= 1000;
		
		return (baudInt * multiplier) / 4;
	}
}

//param flag: SW_SHOW or SW_HIDE
static int ShowProxy(HWND hDlg, int flag)
{
	if (flag == SW_SHOW)
	{
		isy_TCHAR proxyDest[ISYNCLEN_Server+ISYNCLEN_Port] = {0};
		appGetPref(ISYNCPREF_Proxy, proxyDest, sizeof(proxyDest)/sizeof(isy_TCHAR));
		if (proxyDest)
		{
			isy_TCHAR *proxyPort = TSTR_CHR(proxyDest, (isy_TCHAR)':');
			if (proxyPort)
			{
				SetDlgItemText(hDlg, IDC_EDIT_PORT, proxyPort+1);
				proxyDest[proxyPort-proxyDest] = (isy_TCHAR)'\0';
				SetDlgItemText(hDlg, IDC_EDIT_HOST, proxyDest);
			}
		}
	}
	ShowWindow(GetDlgItem(hDlg, IDC_STATIC_HOST), flag);
	ShowWindow(GetDlgItem(hDlg, IDC_EDIT_HOST), flag);
	ShowWindow(GetDlgItem(hDlg, IDC_STATIC_PORT), flag);
	ShowWindow(GetDlgItem(hDlg, IDC_EDIT_PORT), flag);
	return 0;
}

/**********************************************************************************
 Message handler for Network Settings box.
 *********************************************************************************/
LRESULT CALLBACK NetworkSettings(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int	NewPosX = 0, NewPosY = 0;

	isy_TCHAR buf[ISYNCLEN_MiscInfo];
	int value;

	int index = 0;
	int numTimeoutValues = 19;
	int numNetworkValues = 6;

	isy_TCHAR *timeoutValues[] =	{ (ISYNCSTR_TIMEOUTMinimum),(ISYNCSTR_TIMEOUT1min),	(ISYNCSTR_TIMEOUT2min),
									  (ISYNCSTR_TIMEOUT3min),	(ISYNCSTR_TIMEOUT4min),	(ISYNCSTR_TIMEOUT5min),
									  (ISYNCSTR_TIMEOUT6min),	(ISYNCSTR_TIMEOUT7min),	(ISYNCSTR_TIMEOUT8min),
									  (ISYNCSTR_TIMEOUT9min),	(ISYNCSTR_TIMEOUT10min),(ISYNCSTR_TIMEOUT15min),
									  (ISYNCSTR_TIMEOUT20min),	(ISYNCSTR_TIMEOUT25min),(ISYNCSTR_TIMEOUT30min),
									  (ISYNCSTR_TIMEOUT40min),	(ISYNCSTR_TIMEOUT50min),(ISYNCSTR_TIMEOUT60min),
									  (ISYNCSTR_TIMEOUTNever)
									};

	isy_TCHAR *networkValues[] =	{	ISYNCSTR_Default, ISYNCSTR_4KMODEM, ISYNCSTR_9KGSM, 
										ISYNCSTR_19KMODEM,ISYNCSTR_56KMODEM, ISYNCSTR_64KGPRS
									};

	HWND hwndTimeoutComboCtrl, hwndNetworkComboCtrl;
	hwndTimeoutComboCtrl = GetDlgItem (hDlg, IDC_COMBO_TIMEOUT);
	hwndNetworkComboCtrl = GetDlgItem (hDlg, IDC_COMBO_NETWORK);

	isy_TCHAR selTime[20];
	isy_TCHAR *space;
	isy_TCHAR timeoutStr[4];
	int sel, i, len, timeoutMin;

	GET_APP_

	switch (message)
	{

		case WM_INITDIALOG:
			SetWindowText(GetDlgItem (hDlg, IDC_GROUP_NETWORK), ISYNCSTR_Network_Settings);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_TIMEOUT), ISYNCSTR_Timeout);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_NETWORK), ISYNCSTR_Network);
			SetWindowText(GetDlgItem (hDlg, IDC_CHECK_PROXY), ISYNCSTR_Proxy);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_HOST), ISYNCSTR_Host);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_PORT), ISYNCSTR_Port);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			centerHandle(hDlg, NewPosX, NewPosY,0,0);

			// Initialize the proxy
			if (appPrefOptionProxy(_pApp))
			{
				SendDlgItemMessage(hDlg, IDC_CHECK_PROXY, BM_SETCHECK, BST_CHECKED, 0);
				ShowProxy(hDlg, SW_SHOW);
			}
			else
			{
				SendDlgItemMessage(hDlg, IDC_CHECK_PROXY, BM_SETCHECK, BST_UNCHECKED, 0);
				ShowProxy(hDlg, SW_HIDE);
			}
			// end proxy

			// Initialize the timeout combo box
			MEM_SET(buf, 0, sizeof(buf));
			appGetPref(ISYNCPREF_TimeoutRecv, buf, sizeof(buf)/sizeof(isy_TCHAR));
			value = TSTR_ATOI(buf);
			if (value > 0) value = value / 60;

			for (index = 0; index < numTimeoutValues; index++)
			{
				SendMessage (hwndTimeoutComboCtrl, CB_ADDSTRING, 0, 
					(LPARAM)(LPCTSTR) timeoutValues[index]);
			}

			if (value < 0) sel = numTimeoutValues - 1;
			else if (value == 0) sel = 0;
			else 
			{
				for (i = 1; i < numTimeoutValues-1; i++) 
				{
					SendDlgItemMessage(hDlg, IDC_COMBO_TIMEOUT, CB_GETLBTEXT, i, (LPARAM)(LPCSTR) selTime);
					space = TSTR_CHR(selTime, ' ');
					if (space == NULL) continue;
					len = space - selTime;
					TSTR_N_CPY(timeoutStr, selTime, len);
					timeoutMin = TSTR_ATOI(timeoutStr);
					if (value <= timeoutMin) break;
				}
				sel = i;
			}
			SendMessage (hwndTimeoutComboCtrl, CB_SETCURSEL, sel, 0);
			// end timeout combo box

			// Initialize the network combo box
			MEM_SET(buf, 0, sizeof(buf));
			appGetPref(ISYNCPREF_Message_Size, buf, sizeof(buf)/sizeof(isy_TCHAR));
			value = TSTR_ATOI(buf);

			for (index = 0; index < numNetworkValues; index++)
			{
				SendMessage (hwndNetworkComboCtrl, CB_ADDSTRING, 0, 
					(LPARAM)(LPCTSTR) networkValues[index]);
			}

			if (value <= 0)
				sel = 0;		// default is unset, or, let engine calculate
			else
			{
				int netBaudInt = 0;
				for (i = 1; i < numNetworkValues-1; i++)
				{
					SendDlgItemMessage(hDlg, IDC_COMBO_NETWORK, CB_GETLBTEXT, i, (LPARAM)(LPCSTR) buf);
					netBaudInt = parseMsgSizeFromBaud(buf, i);
					if (value <= netBaudInt)
						break;
				}
				sel = i;
			}

			SendMessage (hwndNetworkComboCtrl, CB_SETCURSEL, sel, 0);
			// end network combo box

			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				
				//Save proxy
				long ret = SendDlgItemMessage(hDlg, IDC_CHECK_PROXY, BM_GETCHECK, 0, 0);
				if (ret == BST_CHECKED) 
				{
					isy_TCHAR proxyDest[ISYNCLEN_Server+ISYNCLEN_Port] = {0};

					appSetPrefOptionProxy(_pApp);

					GetDlgItemText(hDlg, IDC_EDIT_HOST, buf, ISYNCLEN_Server);
					TSTR_CPY(proxyDest, buf);
					TSTR_CAT(proxyDest, isy_T(":"));
					GetDlgItemText(hDlg, IDC_EDIT_PORT, buf, ISYNCLEN_Port);
					TSTR_CAT(proxyDest, buf);
					appSetPref(ISYNCPREF_Proxy, proxyDest);
				} 
				else 
				{
					appResetPrefOptionProxy(_pApp);
				}
				//end save proxy

				//Save timeout combo box
				sel = SendDlgItemMessage(hDlg, IDC_COMBO_TIMEOUT, CB_GETCURSEL, 0, 0);
				SendDlgItemMessage(hDlg, IDC_COMBO_TIMEOUT, CB_GETLBTEXT, sel, (LPARAM)(LPCSTR) selTime);
				space = TSTR_CHR(selTime, ' ');
				if (sel == 0) TSTR_CPY(buf, isy_T("0"));
				else if (space == NULL) TSTR_CPY(buf, isy_T("-1"));
				else 
				{
					len = space - selTime;
					TSTR_N_CPY(buf, selTime, len);
					value = TSTR_ATOI(buf);
					TSTR_ITOA(buf, value * 60);
				}
				appSetPref(ISYNCPREF_TimeoutRecv, buf); // Recv timeout
				appSetPref(ISYNCPREF_TimeoutSend, buf); // Send timeout
				//end save timeout combo box

				//Save network combo box
				sel = SendDlgItemMessage(hDlg, IDC_COMBO_NETWORK, CB_GETCURSEL, 0, 0);
				SendDlgItemMessage(hDlg, IDC_COMBO_NETWORK, CB_GETLBTEXT, sel, (LPARAM)(LPCSTR) buf);
				value = parseMsgSizeFromBaud(buf, sel);
				TSTR_ITOA(buf, value);
				appSetPref(ISYNCPREF_Message_Size, buf);
				//end save network combo box

				EndDialog(hDlg, LOWORD(wParam));
				return true;
			}
			else if ( LOWORD(wParam) == IDC_CHECK_PROXY )
			{
				long ret = SendDlgItemMessage(hDlg, IDC_CHECK_PROXY, BM_GETCHECK, 0, 0);
				if (ret == BST_CHECKED) 
					ShowProxy(hDlg, SW_SHOW);
				else
					ShowProxy(hDlg, SW_HIDE);
			}
			else if ( LOWORD(wParam) == IDCANCEL )
			{
				EndDialog(hDlg, LOWORD(wParam));
				return true;
			}
			break;
	}
	return false;
}

isy_BOOL isUserEntered()
{
	appGetPref(ISYNCPREF_User, prefs.user, sizeof(prefs.user)/sizeof(isy_TCHAR));
	if (!prefs.user[0]) 
		return false;
	else 
		return true;
}

isy_BOOL isPasswordEntered()
{
	appGetPref(ISYNCPREF_Password, prefs.password, sizeof(prefs.password)/sizeof(isy_TCHAR));	// password
	if (!prefs.password[0]) 
		return false;
	else 
		return true;
}

/**********************************************************************************
 Message handler for EnterPassword box.
 *********************************************************************************/
LRESULT CALLBACK EnterPassword(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int	NewPosX = 0, NewPosY = 0;

	switch (message)
	{
		case WM_INITDIALOG:
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_PASSWORD), ISYNCSTR_Password);
			SetWindowText(GetDlgItem (hDlg, IDC_CHECK_SAVE_PASSWORD), ISYNCSTR_Save_Password);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			centerHandle(hDlg, NewPosX, NewPosY,0,0);
			return true;
		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				GetDlgItemText(hDlg, IDC_EDIT_ENTER_PASSWORD, prefs.password, ISYNCLEN_Password);
				long ret = SendDlgItemMessage(hDlg, IDC_ENTER_SAVE_PASSWORD, BM_GETCHECK, 0, 0);
				if (prefs.password[0] == '\0' || ret == BST_UNCHECKED) 
				{
					// Don't memorize (or reuse) the password when the current session is done
					appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
				} else
				{
					// Do memorize (or reuse) the password
					appSetPref(ISYNCPREF_Password, prefs.password);
				}
				EndDialog(hDlg, 1); // 1 to indicate ok
				return true;
			}
			else if ( LOWORD(wParam) == IDCANCEL )
			{
				EndDialog(hDlg, 2); // 2 to indicate cancel
				return true;
			}
			break;
	}
	return false;
}

/**********************************************************************************
 Message handler for Server Settings box.
 *********************************************************************************/
LRESULT CALLBACK ServerSettings(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int			NewPosX = 0, NewPosY = 0;

	ServerList *list = NULL;
	ServerNode *node = NULL;
	bool hasDuplicate = false;
	int prefIndex = -1;

	HWND hwndIPComboCtrl;
	hwndIPComboCtrl = GetDlgItem (hDlg, IDC_EDIT_IPADDR);
	int iCurrItem = -1;

	switch (message)
	{
		case WM_INITDIALOG:
			SetWindowText(GetDlgItem (hDlg, IDC_GROUP_SERVER_SETTINGS), ISYNCSTR_Server_Settings);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_URL), ISYNCSTR_URL);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_USER), ISYNCSTR_User);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_PASSWORD), ISYNCSTR_Password);
			SetWindowText(GetDlgItem (hDlg, IDC_CHECK_PASSWORD), ISYNCSTR_Save_Password);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			/*
			** WEA Client support
			*/
			if( bWEASupport == TRUE)
			{
				EnableWindow( hwndIPComboCtrl, FALSE);
				EnableWindow( GetDlgItem( hDlg, IDC_EDIT_POUT_NUM), FALSE);
				EnableWindow( GetDlgItem( hDlg, IDC_EDIT_USER_ID), FALSE);
				EnableWindow( GetDlgItem( hDlg, IDC_EDIT_PASSWORD), FALSE);
				EnableWindow( GetDlgItem( hDlg, IDC_CHECK_PASSWORD), FALSE);
			}

			appGetPref(ISYNCPREF_URL,      prefs.URL, sizeof(prefs.URL)/sizeof(isy_TCHAR));			// IP or hostname
			appGetPref(ISYNCPREF_User,     prefs.user, sizeof(prefs.user)/sizeof(isy_TCHAR));			// user ID
			appGetPref(ISYNCPREF_Password, prefs.password, sizeof(prefs.password)/sizeof(isy_TCHAR));	// password

			centerHandle(hDlg, NewPosX, NewPosY,0,0);
			
			/*
			** WEA Client support
			*/
			if( bWEASupport == TRUE)
			{
				iCurrItem = SendMessage (hwndIPComboCtrl, CB_ADDSTRING, 0, 
					(LPARAM)(LPCTSTR) szURL);
				SendMessage (hwndIPComboCtrl, CB_SETCURSEL, 0, 0);

				SetDlgItemText(hDlg, IDC_EDIT_USER_ID, szUserID);
				SetDlgItemText(hDlg, IDC_EDIT_PASSWORD, szPassword);
			}
			else
			{
				// Initialize the Fields in settings edit boxes
				prefIndex = -1;
				list = createServerList();
				node = list->back; //display list backwards
				while (node != NULL)
				{
					iCurrItem = SendMessage (hwndIPComboCtrl, CB_ADDSTRING, 0, 
						(LPARAM)(LPCTSTR) node->server);

					if (!TSTR_I_CMP(prefs.URL, node->server)) prefIndex = iCurrItem;
					node = node->prev;
				}
				if (prefIndex == -1)
					SendMessage (hwndIPComboCtrl, CB_SETCURSEL, iCurrItem, 0);
				else
					SendMessage (hwndIPComboCtrl, CB_SETCURSEL, prefIndex, 0);
				destroyServerList(list);

				SetDlgItemText(hDlg, IDC_EDIT_USER_ID, prefs.user);
				SetDlgItemText(hDlg, IDC_EDIT_PASSWORD, prefs.password);
			}

			// password checkbox display			

			isy_INT status;
			status = appGetPref(ISYNCPREF_UnsavedPassword, prefs.password, ISYNCLEN_Password);
			if (status == ISCRTN_True)
			{
				SendDlgItemMessage(hDlg, IDC_CHECK_PASSWORD, BM_SETCHECK, BST_CHECKED, 0);
			}
			else
			{
				SendDlgItemMessage(hDlg, IDC_CHECK_PASSWORD, BM_SETCHECK, BST_UNCHECKED, 0);
			}
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				// update settings:

				//saving ip retrieved from combo box
				GetDlgItemText(hDlg, IDC_EDIT_IPADDR, prefs.URL, ISYNCLEN_MiscInfo);

				hasDuplicate = false;
				list = createServerList();
				list->fp = FOPEN(list->fileName, isy_T("wb+"));
				node = list->front;
				while (node != NULL)
				{
					if (!TSTR_I_CMP(prefs.URL, node->server)) 
					{
						hasDuplicate = true;
						break;
					}
					node = node->next;
				}
				if (!hasDuplicate) 
				{
					node = createServerNode(prefs.URL);
					addServerNode(list, node);
				}
				node = list->front;
				while (node != NULL)
				{
					_fputts(node->server, list->fp);
					_fputts(isy_T("\n"), list->fp);
					node = node->next;
				}
				FFLUSH(list->fp);
				FCLOSE(list->fp);
				destroyServerList(list);
				
				/*
				** WEA Client support
				*/
				if( bWEASupport == FALSE)
				{
					GetDlgItemText(hDlg, IDC_EDIT_USER_ID, prefs.user, ISYNCLEN_User);
					GetDlgItemText(hDlg, IDC_EDIT_PASSWORD, prefs.password, ISYNCLEN_Password);
				}

				appSetPref(ISYNCPREF_URL,      prefs.URL);		// URL
				appSetPref(ISYNCPREF_User,     prefs.user);		// user ID

				long ret = SendDlgItemMessage(hDlg, IDC_CHECK_PASSWORD, BM_GETCHECK, 0, 0);
				if (prefs.password[0] == '\0' || ret == BST_UNCHECKED) 
				{
					// Don't memorize (or reuse) the password when the current session is done
					appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
				} else
				{
					// Do memorize (or reuse) the password
					appSetPref(ISYNCPREF_Password, prefs.password);
				}
				EndDialog(hDlg, LOWORD(wParam));
				
				return true;
			}
			else if ( LOWORD(wParam) == IDCANCEL )
			{
				EndDialog(hDlg, LOWORD(wParam));
				return true; // keep data unchanged.
			}

			break;
	}
		
	return false;
}

/**********************************************************************************
 Message handler for Client Settings box.
 *********************************************************************************/
LRESULT CALLBACK ClientSettings(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int	NewPosX = 0, NewPosY = 0;

	isy_TCHAR traceOn[2];
	isy_TCHAR targetDir[ISYNCLEN_Target];

	int index = 0, sel = 0;
	int numTraceValues = 2;

	isy_TCHAR *traceValues[] =	{ISYNCSTR_Default,	ISYNCSTR_Detailed};

	HWND hwndTraceComboCtrl;
	hwndTraceComboCtrl = GetDlgItem (hDlg, IDC_COMBO_TRACE);

	GET_APP_

	switch (message)
	{

		case WM_INITDIALOG:
			SetWindowText(GetDlgItem (hDlg, IDC_GROUP_CLIENT_SETTINGS), ISYNCSTR_Client_Settings);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_TRACE), ISYNCSTR_Trace);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_TARGETDIR), ISYNCSTR_TargetDir);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			centerHandle(hDlg, NewPosX, NewPosY,0,0);

			// Initialize the trace combo box
			for (index = 0; index < numTraceValues; index++)
			{
				SendMessage (hwndTraceComboCtrl, CB_ADDSTRING, 0, 
					(LPARAM)(LPCTSTR) traceValues[index]);
			}

			appGetPref(ISYNCPREF_Trace, traceOn, sizeof(traceOn)/sizeof(isy_TCHAR));
			if (TSTR_CMP(traceOn, isy_T("Y")) == 0)
				sel = 1;

			SendMessage (hwndTraceComboCtrl, CB_SETCURSEL, sel, 0);
			// end trace combo box

			if( bWEASupport == TRUE)
			{
				EnableWindow( GetDlgItem(hDlg, IDC_EDIT_TARGETDIR), FALSE);
				SetDlgItemText(hDlg, IDC_EDIT_TARGETDIR, szDBDir);
			}
			else
			{
				//Initialize target directory
				appGetPref(ISYNCPREF_TargetDir, targetDir, sizeof(targetDir)/sizeof(isy_TCHAR));
				if(TSTR_LEN(targetDir) == 0)
					TSTR_CPY(targetDir, DATADIR);
				SetDlgItemText(hDlg, IDC_EDIT_TARGETDIR, targetDir);
				//end target directory
			}

			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				//Save trace combo box
				sel = SendDlgItemMessage(hDlg, IDC_COMBO_TRACE, CB_GETCURSEL, 0, 0);
				if (sel == 1) 
				{				
					TSTR_CPY(traceOn, isy_T("Y"));
					appSetPref(ISYNCPREF_Trace, traceOn); // Trace ON
				} 
				else 
				{
					TSTR_CPY(traceOn, isy_T("N"));
					appSetPref(ISYNCPREF_Trace, traceOn); // Trace OFF
				}
				//end save trace combo box

				//save target directory
				GetDlgItemText(hDlg, IDC_EDIT_TARGETDIR, targetDir, ISYNCLEN_Target);
				appSetPref(ISYNCPREF_TargetDir, targetDir);
				//end save target directory

				EndDialog(hDlg, LOWORD(wParam));
				return true;
			}
			else if ( LOWORD(wParam) == IDCANCEL )
			{
				EndDialog(hDlg, LOWORD(wParam));
				return true;
			}
			break;
	}
	return false;
}