/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client GUI
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

/**
 * File Name:  syncForm.cpp
 *
 * Description:
 *    Creates synchronization thread; implements callback listener
 *
 */

#include "stdafx.h"

#include "resource.h"
#include "syncForm.h"
#include "logfile.h"

extern HWND			hWnd;		// from db2sync.cpp
extern HWND			hWndText;
extern HINSTANCE	hInst;
extern isy_INT32	apprc;
extern isy_INT32	SilentMode;	//Yamato, Silent mode

isy_TCHAR DB2e_user[ISYNCLEN_DB2eUser]			= {0};
isy_TCHAR DB2e_password[ISYNCLEN_DB2ePassword]	= {0};
isy_TCHAR DB2e_target[ISYNCLEN_TargetDir]		= {0};

typedef struct {
	HWND hDlg;			// window handle for SyncDialog
	isy_TCHAR appName[ISCLEN_SubsSetName];
	isy_UINT32 mode;
	isy_UINT32 threadID;
} ISYNCGOPARAMSTRUCT;

ISYNCGOPARAMSTRUCT iSyncGoParamStruct;

/**********************************************************************************
 Message handler for local data encryption box.
 *********************************************************************************/
LRESULT CALLBACK LoginENC(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	int	NewPosX = 0, NewPosY = 0;
	
	switch (message)
	{

		case WM_INITDIALOG:
			SetWindowText(GetDlgItem (hDlg, IDC_GROUP_AUTHEN), ISYNCSTR_DB2e_Authen);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_TARGET), ISYNCSTR_Target);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_USER), ISYNCSTR_DB2e_User);
			SetWindowText(GetDlgItem (hDlg, IDC_STATIC_PASSWORD), ISYNCSTR_DB2e_Password);
			SetWindowText(GetDlgItem (hDlg, IDOK), ISYNCSTR_OK);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			SetDlgItemText(hDlg, IDC_TARGETENC, DB2e_target);
			centerHandle(hDlg, NewPosX, NewPosY, 0, 50);
			return true;

		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK)
			{
				GetDlgItemText(hDlg, IDC_USERNAMEENC, DB2e_user, ISYNCLEN_DB2eUser);
				GetDlgItemText(hDlg, IDC_PASSWORDENC, DB2e_password, ISYNCLEN_DB2ePassword);
				EndDialog(hDlg, 1);
				return true;
			}
			else if ( LOWORD(wParam) == IDCANCEL )
			{
				 EndDialog(hDlg, 2);
				 return true;
			}
			break;
	}
    return false;
}

/**********************************************************************************
 Message handler for synchronization box.
 *********************************************************************************/
LRESULT CALLBACK SyncDialog(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	int NewPosX = 0, NewPosY = 0;
	static HANDLE hThread = NULL;
	isy_TCHAR szStr[ISYNCLEN_MiscInfo] = {0};

	GET_APP_

	switch (message)
	{
		case WM_INITDIALOG:
			SetWindowText(hDlg, ISYNCSTR_Synchronization);
			SetWindowText(GetDlgItem (hDlg, IDCANCEL), ISYNCSTR_Cancel);

			centerHandle(hDlg, NewPosX, NewPosY,0,0); // try to center SyncDialog
			iSyncGoParamStruct.hDlg=hDlg;
			hThread = startSyncThread(_pApp->hSyncer, NULL, 0, &iSyncGoParamStruct.threadID);
			return TRUE;

		case WM_COMMAND:

			if (LOWORD(wParam) == IDCANCEL)
			{
				if (hThread!=NULL) 
				{
					if (LOWORD(lParam) == ID_SYNC_ENDED)
					{
						EndDialog(hDlg, LOWORD(wParam));

						//Yamato, Silent mode
						if( SilentMode)
						{
							PostQuitMessage(apprc);
							DestroyWindow(GetParent(hDlg));
						}
						return TRUE;
					}
					else 
					{
						// Yamato, cancel code
						HANDLE hCancelEvent = CreateEvent(NULL,TRUE,FALSE,_T("DB2eISyncCancelThread"));
						SetEvent( hCancelEvent);
						CloseHandle(hCancelEvent);

						//cancelled
						// endSyncThread(hThread); // hThread is initialized as soon as dialog pops up, cancel kills it.
						//LoadString(hInst, IDS_CANCEL_WAIT, szStr, ISYNCLEN_MiscInfo);
						TSTR_CPY(szStr, ISYNCSTR_InfCancelingSync);
						SetDlgItemText(hDlg, IDC_SYNC_STATUS_TXT, szStr); 
						
						//Yamato, Silent mode
						if( SilentMode)
						{
							PostQuitMessage(apprc);
							DestroyWindow(GetParent(hDlg));
						}
						return FALSE;
					}
				}
				
			}

			break;
	}
    return FALSE;
}

DWORD WINAPI iSyncGoWrapper(LPVOID iSyncGoParam) 
{
	DWORD dwResult = 0;
	GET_APP_
	
	dwResult = syncStartup(syncListener);

	//iSyncGoParam is struct containing data to call the listener with.
	_pApp->canceled = false;
	_pApp->errorCount = 0;

	logPurge(_pApp->hLog);
	appLogBuildVersion(_pApp);

	if (dwResult != ISCRTN_Succeeded) 
	{
		isy_TCHAR statusMsg[128];
		TSTR_CPY(statusMsg, ISYNCSTR_StartupError);
		//dwResult should be ISCRTN_NotFound (-3)
		TSTR_ITOA(statusMsg + TSTR_LEN(statusMsg), dwResult);
		logWrite(_pApp->hLog, statusMsg);

		SetWindowText(hWndText, ISYNCSTR_InfSyncFailed);
		goto exit1;
	}

	dwResult = iscEngineSync(_pApp->hSyncer);

	if (_pApp->sessionPassword[0])
	{
		ISYNCPREFS prefs;
		prefs.password[0] = '\0';
		appSetPref(ISYNCPREF_UnsavedPassword, prefs.password);
	}
exit1:
	apprc = (isy_INT32) (_pApp->canceled == TRUE) ? -1 : dwResult;	//Yamato
	syncShutdown();
	SendMessage(iSyncGoParamStruct.hDlg,WM_COMMAND,IDCANCEL,ID_SYNC_ENDED);
	return dwResult;
}

//Yamato, cancel code
DWORD WINAPI iSyncCancelThread(LPVOID iSyncGoParam) 
{
	GET_APP_
	
	HANDLE hCancelEvent = CreateEvent(NULL,TRUE,FALSE,_T("DB2eISyncCancelThread"));
	while( WAIT_OBJECT_0 != WaitForSingleObject(hCancelEvent,0)) Sleep(250);
	CloseHandle(hCancelEvent);
	
	_pApp->canceled = true;

	ExitThread(0);
	return 0;
}

HANDLE startSyncThread(HISCENG hThis, isy_TCHAR* appName, unsigned long mode, LPDWORD threadID)
{
	HANDLE	syncThread = NULL;
	GET_APP_

	_pApp->canceled = false;

	if ( appName == NULL ) 
		wcscpy(iSyncGoParamStruct.appName, isy_T("NULL app"));
	else 
		wcscpy(iSyncGoParamStruct.appName, appName);	

	iSyncGoParamStruct.mode = mode;

	//Yamato, Cancel code
	CreateThread( 
				NULL,			// ignored
				1,				// ignored, stack size
				iSyncCancelThread, // function to call returning type LPTHREAD_START_ROUTINE, which is type DWORD
				NULL, // pass a 32bit pointer to parameters of the above funciton.
				0,				// start thread immediately
				NULL		// long pointer to thread ID, if NULL, nothing is returned.
				);

	syncThread = CreateThread( 
				NULL,			// ignored
				1,				// ignored, stack size
				iSyncGoWrapper, // function to call returning type LPTHREAD_START_ROUTINE, which is type DWORD
				&iSyncGoParamStruct, // pass a 32bit pointer to parameters of the above funciton.
				0,				// start thread immediately
				threadID		// long pointer to thread ID, if NULL, nothing is returned.
				);

	return syncThread;
}

BOOL endSyncThread(HANDLE hThread)
{
	GET_APP_
	_pApp->canceled = true;
	return TRUE;
}

/**********************************************************************************
 Callback listener.
 *********************************************************************************/
static isy_INT32 syncListener(isy_UINT32 listenerData, ISCEVT* event, isy_VOID* pExtraInfo)
{
	GET_APP_

	HWND hDlg;
	isy_TCHAR buf[ISYNCLEN_MiscInfo];
	isy_TCHAR percent[4];
	const isy_TCHAR *statusMsg = appEventCodeToMessage(event->code);
	int NewPosX, NewPosY;

	hDlg = iSyncGoParamStruct.hDlg;

	switch (event->type) {
		case ISCEVTTYPE_Retry: // RetryOkCancel
		{
			long timeout = 60;
			int timeoutReturnValue = ISCRTNCB_ReplyNo;
			
			//Yamato, Silent mode
			if( !SilentMode)
			{
				int rc = MessageBox(hWnd, statusMsg, NULL, MB_OKCANCEL|MB_ICONEXCLAMATION|MB_TOPMOST);
				
				if(rc == 1) //yes pressed
					return ISCRTNCB_ReplyYes;
				else 
					return ISCRTNCB_ReplyNo;				
			}
			else
			{
				return ISCRTNCB_ReplyNo;				
			}
		} 
		break;
		
        case ISCEVTTYPE_Conflict:
		{
			//from sqlcli.h
			//#define  SQL_INTEGER             4
			//#define  SQL_SMALLINT            5
			#define SQL_BLOB					-98

			ISCLISTENCONFLICT *pConflict = (ISCLISTENCONFLICT *) event->info;
			int i=0;
			if (!pConflict) return ISCRTNCB_Default;
			TSTR_CPY(buf, ISYNCSTR_MSG_CONFLICT);
			TSTR_CAT(buf, pConflict->table);
			appLogWrite(_pApp, event->code, event->type, buf);
			switch (pConflict->op)
			{
				case 1:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_DELETE);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
				case 2:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_INSERT);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
				case 3:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_UPDATE);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
				default:
					TSTR_CPY(buf, ISYNCSTR_MSG_OP_INVALID);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_MSG_DATA);
					break;
			}
			logWrite(_pApp->hLog, buf);

			for (i=0; i<pConflict->colc; i++)
			{
				ISCLISTENCOLUMN *pColumn = &(pConflict->colv[i]);
				MEM_SET(buf, 0, sizeof(buf));

				if (pColumn->data == NULL) {
					TSTR_CAT(buf, ISYNCSTR_MSG_NULL);
				}
				else if (pColumn->type == SQL_BLOB) {
					TSTR_CAT(buf, isy_T(" [ * ] "));
				}
				else {
					isy_BYTE* pData;
					// for null terminated character
					pData = (isy_BYTE *) MEM_PTR_ALLOC(pColumn->size + 2);
					if (pData == NULL) break;
					MEM_CPY(pData, &(pColumn->data[0]), pColumn->size);
					pData[pColumn->size] = 0;
					MEM_CPY(buf, pData, pColumn->size > 100 ? 100:pColumn->size);
					MEM_PTR_FREE(pData);
				}
				logWrite(_pApp->hLog, buf);
			}
		} //end ISCEVTTYPE_Conflict
		break;
		case ISCEVTTYPE_Query:
		{
			switch (event->code) 
			{
				case ISCEVT_QueCancel:					
					if (_pApp->canceled)
						return ISCRTNCB_ReplyYes;
					else
						return ISCRTNCB_ReplyNo;
				break;
				case ISCEVT_QueCancelUponError:
				{	
					long timeout = 60;
					int timeoutReturnValue = ISCRTNCB_ReplyNo;
					
					//Yamato, Silent mode
					if( !SilentMode)
					{
						int rc = MessageBox(hWnd, statusMsg, NULL, MB_OKCANCEL|MB_ICONEXCLAMATION|MB_TOPMOST);
							
						if(rc == 1)
							return ISCRTNCB_ReplyNo;
						else 
							return ISCRTNCB_ReplyYes;
					}
					else
					{
						return ISCRTNCB_ReplyNo;
					}
				}
				break;
				case ISCEVT_QueLogin:
				{
					ISCLISTENARG *args = event->info;
					TSTR_CPY(DB2e_target, args->argv[0]);
					int ret = DialogBox(hInst, (LPCTSTR)IDD_DB2EAUTHEN, hDlg, (DLGPROC)LoginENC);
					if (ret == 1)
					{
						TSTR_CPY(args->argv[1], DB2e_user);
						TSTR_CPY(args->argv[2], DB2e_password);
						return ISCRTNCB_Done;
					}
				}
				break;
			} //end switch
		} //end ISCEVTTYPE_Query
		break;				
		
        case ISCEVTTYPE_Info:			
            switch (event->code) {				
				case ISCEVT_InfSubsSetStarted:
					TSTR_CPY(buf, ISYNCSTR_SubsSet);
					TSTR_CAT(buf, event->state.currSubsSet);
					appLogWrite(_pApp, event->code, event->type, buf);
					TSTR_CAT(buf, FILE_NL);
					TSTR_CAT(buf, ISYNCSTR_Synchronize);
					TSTR_CAT(buf, isy_T(" "));
					TSTR_CAT(buf, TSTR_ITOA(percent, event->state.syncProg));
					TSTR_CAT(buf, isy_T("%"));
					SetDlgItemText(hDlg, IDC_SYNC_SUMMARY, buf);
					centerHandle(hDlg, NewPosX, NewPosY,0,0);
					break;

				case ISCEVT_InfSendMsg:
				case ISCEVT_InfWaitMsg:
				case ISCEVT_InfSyncingSubs:
					if (*event->state.currSubs)
					{
						TSTR_CPY(buf, ISYNCSTR_Subscription);
						TSTR_CAT(buf, event->state.currSubs);
						appLogWrite(_pApp, event->code, event->type, buf);
						TSTR_CAT(buf, FILE_NL);
						TSTR_CAT(buf, ISYNCSTR_Synchronize);
						TSTR_CAT(buf, isy_T(" "));
						TSTR_CAT(buf, TSTR_ITOA(percent, event->state.syncProg));
						TSTR_CAT(buf, isy_T("%"));
						SetDlgItemText(hDlg, IDC_SYNC_SUMMARY, buf);
						centerHandle(hDlg, NewPosX, NewPosY,0,0);
					}
					break;

				case ISCEVT_InfSyncStarted:
					SetDlgItemText(hDlg, IDC_SYNC_SUMMARY, statusMsg);
					SetDlgItemText(hDlg, IDC_SYNC_STATUS_TXT, ISYNCSTR_TryConnect);
					SetWindowText(hWndText, statusMsg);
					appLogWrite(_pApp, event->code, event->type, statusMsg);
					break;
					
				case ISCEVT_InfSyncSucceeded:
				case ISCEVT_InfSyncFailed:
				case ISCEVT_InfSyncCanceled:
					SetWindowText(hWndText, statusMsg);
					appLogWrite(_pApp, event->code, event->type, statusMsg);
                    break;
                    
                default: // ignore other event code
             	    if(TSTR_LEN(statusMsg) > 0)
					{	
						SetDlgItemText(hDlg, IDC_SYNC_STATUS_TXT, statusMsg);
						appLogWrite(_pApp, event->code, event->type, statusMsg);
					}
                    break;
            } // switch (event->code)
            break;
        default:
        	if(TSTR_LEN(statusMsg) > 0)
			{	
				SetDlgItemText(hDlg, IDC_SYNC_STATUS_TXT, statusMsg);
				appLogWrite(_pApp, event->code, event->type, statusMsg);
			}			
            break;
    } // switch (event->type)

	return ISCRTNCB_Default;
}