/************************************************************************
 *
 * Licensed Materials - Property of IBM
 * Program Name: IBM DB2Everyplace Sync Client
 * (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
 *
 * US Government Users Restricted Rights 
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. Licensed Materials - Property of IBM
 *
 * IBM grants you ("Licensee") a non-exclusive, royalty free, license to
 * use Sample Code.  This software is provided as-is and without any
 * warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
 * WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
 * NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
 * DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
 * IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
 * PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
 * INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
 * THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
 * SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
 * DAMAGES.
 *
 ************************************************************************
 */

import com.ibm.mobileservices.isync.*;

import java.io.BufferedReader;
import java.io.PrintStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;

import java.util.Enumeration;
import java.util.Properties;

/**
	Main class presenting a command interface where a series of prompts
	allow control of the synchronization of various clients, as defined
	by the property file's "isyncprotocol" value.

	Implements a constant interface to hide definitions.
*/

public class DB2SyncConsole implements DB2SyncConstants
{
	ISyncProvider provider = null;
	ISyncService service = null;
	ISyncConfigStore config = null;
	ISyncDriver syncer = null;

	Properties isyncProps;
	File hLog = null;
	BufferedReader logfileReader = null;
	PrintStream logPrintStream = null;
	static String propFileName = null;

 	private String NetBaudNames[] =
		{"0: Default",
		"1: 4K Modem",
		"2: 9K GSM",
		"3: 19K Modem",
		"4: 56K Modem",
		"5: 64K GPRS",
		"6: 112K ISDN",
		"7: 256K DSL",
		"8: 2M  MTS",
		"9: 10M T1"};
	private int NetBaudValues[] = 
		{0,
		4800/4,
		9600/4,
		19200/4,
		(56  * 1024) / 4,
		(64  * 1024) / 4,
		(112 * 1024) / 4,
		(256 * 1024) / 4,
		(2   * 1000 * 1024) / 4,
		(10  * 1000 * 1024) / 4};


	/** 
		Constructor
	*/

	public DB2SyncConsole() {
	}


	/** 
		DB2SyncConsole entry point
	*/
	public static void main(String args[]) {
		if (args.length != 1) 
		{
			System.out.println("Usage: java DB2SyncConsole <property file>");
			return;
		}

		DB2SyncConsole iSync = new DB2SyncConsole();

		propFileName = args[0];

		try {
			iSync.startClient(propFileName);
		}
		catch (Exception e) 
		{
			e.printStackTrace();
		}
	}

	/** 
		 This function loops until getAction switches to DO_EXIT and the
		 program is terminated.
	*/
	public void startClient(String propFile)
		throws IOException
	{
		int rc;
		String action;

		/*
			Read properties file to act as the initial default values,
			or for those not set in the application saved preference file.
		*/
		readProperties(propFile);

		rc = initSettings();

		System.out.println("\n\n");

		while (true)
		{
			try
			{
				action = getAction();
				if (action.equals(DO_SYNC)) {
					System.out.print("\n\n");
					goSync();
				} else if (action.equals(DO_APPS)) {
					System.out.print("\n\n");
					goApps();
				} else if (action.equals(DO_PREFS)) {
					System.out.print("\n\n");
					goSettings();
				} else if (action.equals(DO_LOG)) {
					System.out.print("\n\n");
					goLog();
				} else if (action.equals(DO_ABOUT)) {
					goAbout();
				} else if (action.equals(DO_EXIT)) {
					goExit();
					break;
				} else if (action.equals(DO_NOTHING)) {
				} else {
					System.out.println(action + " " + DB2Sync_InvalidChoice);
				}
			}
			catch (ISyncException ise)
			{
				System.out.println(DB2SyncConsoleListener.eventCodetoMsg(ise.getCode()));
				ise.printStackTrace();
				if (logPrintStream != null)
					ise.printStackTrace(logPrintStream);
				try
				{
					syncShutdown();
				}
				catch (ISyncException ise2) { }
			} 
		}
	} // startClient

	/**
		 Prompts the user for an action to perform and returns an int
		 representing their choice.
		 @return the users choice as String
	*/
	public String getAction() throws IOException {
		String action = DO_NOTHING;
		BufferedReader br =
			new BufferedReader(new InputStreamReader(System.in));

		System.out.println(DB2Sync_MainMenu);
		System.out.println(DB2Sync_MainMenu1);
		System.out.println(DB2Sync_MainMenu2);
		System.out.println(DB2Sync_MainMenu3);
		System.out.println(DB2Sync_MainMenu4);
		System.out.println(DB2Sync_MainMenu5);
		System.out.println(DB2Sync_MainMenu6);
		System.out.print(DB2Sync_MakeChoice);

		action = br.readLine();
		if (action == null)
			System.exit(0);
		return action.trim();
	} // getAction

	/**
		 Performs the synchronization.  Before sync() is called, the
		 syncListener is set so that the application can receive callbacks
	*/
	public void goSync()
		throws ISyncException, FileNotFoundException, IOException
	{
		int rc;
		DB2SyncConsoleListener iSyncListener = new DB2SyncConsoleListener(isyncProps);

		if (syncer == null)
		{
			rc = syncStartup();
			if (rc != ISync.RTN_SUCCEEDED) {
				System.out.println(
					"\nSync Engine startup error. Make sure path is valid. Error code: "
						+ rc
						+ "\n\n");
				return;
			}
		}
		rewindLog();
		iSyncListener.setPrintStream(logPrintStream);

		syncer.setSyncListener(iSyncListener);

		rc = syncer.sync();

		System.out.print("iscEngineSync() = " + rc + "\n\n");

		ISyncSubscriptionSet ssArr[] = config.getSubscriptionSets();
		if (ssArr.length == 0)
		{
			System.out.println(DB2Sync_NoSubsetInfo);
			return;
		}


		/*
			Print status for each subscription set
		*/

		for (int i = 0; i < ssArr.length; i++) {
			System.out.println(DB2Sync_SyncStatus);

			if (ssArr[i].isEnabled()) {
				System.out.print("+  ");
			} else {
				System.out.print("-  ");
			}

			int subStatus = ssArr[i].getStatus();

			if (subStatus == ISync.RTN_CANCELED)
				System.out.print("Cancelled");
			else if (subStatus == ISync.RTN_FAILED)
				System.out.print("Failed");
			else if (subStatus == ISync.RTN_NOT_FOUND)
				System.out.print("Not Found");
			else if (subStatus == ISync.RTN_READY)
				System.out.print("Ready");
			else if (subStatus == ISync.RTN_SUCCEEDED)
				System.out.print("Succeeded!");
			else
				System.out.print("" + subStatus);

			System.out.println(
				"\t" + ssArr[i].getId() + "\t" + ssArr[i].getName());

		}

		System.out.println(DB2Sync_Line);

		System.out.print("\n\n");

		syncShutdown();
	} // goSync

	/**
		 Allows the user to enable, disable, or reset a subscription
		 set. If no changes are necessary to the sync application the user
		 can press enter to accept the default setting.  
	*/
	public void goApps() throws ISyncException, IOException {
		ISyncSubscriptionSet ssArr[] = null;
		int done = 0, rc;

		if (syncer == null)
		{
			rc = syncStartup();
			if (rc != ISync.RTN_SUCCEEDED)
			{
				System.out.print(
					"\nSync Engine startup error. Make sure path is valid. Error code: "
						+ rc
						+ "\n\n\n");
				return;
			}
		}

		System.out.println(DB2Sync_SyncApps);

		ssArr = config.getSubscriptionSets();
		if (ssArr.length == 0)
		{
			System.out.println(DB2Sync_NoSubsetInfo);
			System.out.print("\n\n");
			syncShutdown();
			return;
		}


		/*
			For each subscription set, show current enable/disable and
			refresh/synchronize condition; prompt for any changes
		*/

		for (int i = 0; i < ssArr.length; i++)
		{
			String input;

			System.out.print("Subscription set " + ssArr[i].getName());
			if (ssArr[i].isEnabled())
				System.out.print(" " + DB2Sync_Enabled);
			else
				System.out.print(" " + DB2Sync_Disabled);
			if (ssArr[i].isReset())
				System.out.println(" " + DB2Sync_ModeRefresh);
			else
				System.out.println(" " + DB2Sync_ModeSynchronize);

			System.out.println(DB2Sync_SetAppMode);
			System.out.println(DB2Sync_SetAppModeA);
			System.out.println(DB2Sync_SetAppModeB);
			System.out.println(DB2Sync_SetAppModeC);
			System.out.println(DB2Sync_SetAppModeD);
			System.out.print(DB2Sync_MakeChoice);

			BufferedReader br =
				new BufferedReader(new InputStreamReader(System.in));

			input = br.readLine();

			if (input.equals(MODE_ENABLE)) {
				ssArr[i].enable();
			} else if (input.equals(MODE_DISABLE)) {
				ssArr[i].disable();
			} else if (input.equals(MODE_RESET)) {
				ssArr[i].reset();
			} else if (input.equals(MODE_DONE)) {
				done = 1;
			}
			System.out.print("\n\n");
		}
		syncShutdown();
	} // goApps

	/*
		Default initial settings before user runs
	*/
	private void readProperties(String propFile)
		throws IOException
	{
		if (!propFile.endsWith(".properties"))
			propFile += ".properties";
		isyncProps = new Properties();
		isyncProps.load(new FileInputStream(propFile));
	} //readProperties

	/**
		Read previously saved settings if any, overwriting current ones
		in isyncProps.

		@return handle to ISYNC
	*/
	public int initSettings() throws FileNotFoundException, IOException {
		int rc = 0;

		getPrefs(propFileName);

		openLog(true);
		return rc;
	} // initSettings

	/**
		Allow the user to change the network and user settings which
		enable the sync connection to the server, if no changes are 
		necessary, the user can press enter to accept the current setting.
	*/
	public void goSettings() throws IOException {
		String buf;
		int i, validTimeout;
		BufferedReader in =
			new BufferedReader(new InputStreamReader(System.in));

		System.out.println(DB2Sync_Settings);
		System.out.println(DB2Sync_SetSettings);

		// URL [http://127.0.0.1:80]:

		//Get Server IP
		System.out.print("URL ["
			+ isyncProps.getProperty("server.url") + "]: ");
		buf = in.readLine();
		buf.trim();
		if (!buf.equals(""))
			isyncProps.put("server.url", buf);

		/*
			Get Username
		*/
		System.out.print(DB2Sync_User + " ["
			+ isyncProps.getProperty("isync.user") + "]: ");
		buf = in.readLine();
		buf.trim();
		if (!buf.equals(""))
			isyncProps.put("isync.user", buf);

		/*
			Get Password
		*/
		System.out.print(DB2Sync_Password + " [");
		for (i = isyncProps.getProperty("isync.password").length(); i > 0; i--)
			System.out.print("*");
		System.out.print("]: ");
		buf = in.readLine();
		buf.trim();
		if (!buf.equals(""))
			isyncProps.put("isync.password", buf);

		/*
			Get Path
		*/
		System.out.print(DB2Sync_Path + " ["
			+ isyncProps.getProperty("path") + "]: ");
		buf = in.readLine();
		buf.trim();
		if (!buf.equals(""))
			isyncProps.put("path", buf);

		/*
			Get Trace ON/OFF
			"Trace (0:default / 1:detailed)? [1]:" 
		*/
		System.out.print(
			DB2Sync_Trace
				+ " (0:"
				+ DB2Sync_Default
				+ " / 1:"
				+ DB2Sync_Detailed
				+ ")? ["
				+ ((isyncProps.getProperty("isync.trace").equals(DB2Sync_Default)) ? "0" : "1")
				+ "]: ");
		buf = in.readLine();
		buf.trim();
		if (!buf.equals(""))
		{
			if (buf.equals("0"))
				isyncProps.put("isync.trace", DB2Sync_Default);
			else
				isyncProps.put("isync.trace", DB2Sync_Detailed);
		}

		/*
			Check for valid time out settings.
		*/
		while (true)
		{
			//Get Timeout length
			String str_timeout = isyncProps.getProperty("isync.timeout");
			validTimeout = Integer.parseInt(str_timeout);
			if (validTimeout == 0) {
				System.out.print(
					DB2Sync_Timeout + " [" + DB2Sync_MinimumTimeout + "]: ");
			} else if (validTimeout == -1) {
				System.out.print(
					DB2Sync_Timeout + " [" + DB2Sync_NeverTimeout + "]: ");
			} else {
				validTimeout /= 60; // minutes
				System.out.print(
					DB2Sync_Timeout + " [" + validTimeout + " min]: ");
			}

			buf = in.readLine();
			buf.trim();
			try {
				if (buf.equals(""))
					break;

				validTimeout = Integer.parseInt(buf);
				isyncProps.put("isync.timeout", "" + validTimeout * 60);

				//valid timeout received. break.
				break;
			} catch (NumberFormatException e) {
				System.out.println(DB2Sync_InvalidTimeout);
				validTimeout = 0;
			}
		}
		String msgSize = isyncProps.getProperty("isync.messagesize");
		if (msgSize == null)
			msgSize = "0";

		for (int n = 0; n < NetBaudNames.length; n++)
			System.out.println(NetBaudNames[n] + "=" + NetBaudValues[n]);
		String curMsgSize = isyncProps.getProperty("isync.messagesize");
		if (curMsgSize == null)
			curMsgSize = "default";

		System.out.print("Choose Network Type [" + curMsgSize + "]: ");
		while (true)
		{
			try {
				buf = in.readLine();
				buf.trim();
				if (buf.equals(""))
					break;

				int size = Integer.parseInt(buf);
				if (size < 0 || size >= NetBaudNames.length)
				{
					System.out.println("Not a valid choice");
					continue;
				}
				isyncProps.put("isync.messagesize", String.valueOf(NetBaudValues[size]));

				//valid index
				break;
			}
			catch (NumberFormatException e)
			{
				System.out.println("Invalid index");
			}
		}

		System.out.print("\n\n");
		setPrefs(propFileName);
	} // goSettings

	/**
	 	 Prints the log file to the screen.
	*/
	public void goLog() throws IOException {
		try {
			openLog(true);
			String buf;
			System.out.println(DB2Sync_LogFile);
			while ((buf = logfileReader.readLine()) != null) {
				System.out.println(buf);
			}
			System.out.print("\n\n");
		} catch (FileNotFoundException fnf) {
			System.out.println(DB2Sync_LogFile);
			System.out.println("\n\n");
		}
		finally
		{
			logfileReader.close();
			// open output stream again.
			openLog(true);
		}
	} // goLog

	/**
		Close synchronization objects
	*/
	public void syncShutdown() throws ISyncException {

		if (syncer != null)
		{
			syncer.close();
			syncer = null;
		}

		if (config != null)
		{
			config.close();
			config = null;
		}

		if (service != null)
		{
			service.close();
			service = null;
		}
	} // syncShutdown

	/**
		Create synchronization objects
	*/
	public int syncStartup() throws ISyncException
	{
		int rc = 0;

		syncShutdown();

		//get an instance of the appropriate provider
		//
		try
		{
			Class.forName(isyncProps.getProperty("syncdriver"));
		}
		catch (java.lang.ClassNotFoundException cnfe)
		{
			throw new ISyncException(cnfe.toString());
		}
		provider = ISyncManager.getISyncProvider(isyncProps.getProperty("syncprotocol"));

		String uri = isyncProps.getProperty("server.url");

		/*
			get an instance of synchronization service from the provider
		*/

		service =
			provider.createSyncService(uri, isyncProps);

		/*
			get an instance of the configuration store
		*/ 
		config = service.getConfigStore(isyncProps.getProperty("path"));

		/*
			get an instance of the sync driver to perform synchronization
		*/ 
		syncer = config.getSyncDriver();

		return rc;
	} // syncStartup

	/**
		About the Sync Client
	*/
	public void goAbout() throws ISyncException
	{
		int rc = syncStartup();
		if (rc != ISync.RTN_SUCCEEDED) {
			System.out.print(
				"\nSync Engine startup error. Make sure path is valid. Error code: "
					+ rc
					+ "\n\n\n");
			return;
		}
		String about = "V" + getVersionInString() + "\nBUILD " + provider.getBuildDate();

		System.out.print("\n" + DB2Sync_About + "\n" + about + "\n\n");
		syncShutdown();
	} //goAbout

	/**
		Exits this program
	*/
	public void goExit() throws ISyncException, IOException {
		System.out.print(DB2Sync_Exitting + " ...\n");

		if (logPrintStream != null)
		{
			logPrintStream.close();
			logPrintStream = null;
		}

		if (logfileReader != null)
		{
			logfileReader.close();
			logfileReader = null;
		}

		setPrefs(propFileName);
		syncShutdown();
		System.exit(0);
	} //goExit

	/**
		Get application preferences from properties file ISync.properties if it exists.
		Otherwise, properties passed from command line will be used.
	*/
	public void getPrefs(String fileName)
		throws FileNotFoundException, IOException
	{
		File prefPropfile = new File(fileName);

		if (prefPropfile.exists()) {
			Properties prefProp = new Properties();
			prefProp.load(new FileInputStream(prefPropfile));

			/*
				Copy all user prefs to our global prefs, to create a
				super set (union) of values.
			*/
			Enumeration enum = prefProp.keys();

			while (enum.hasMoreElements())
			{
				String key = (String) enum.nextElement();
				String value = prefProp.getProperty(key);
				isyncProps.put(key, value);
			}
		}
	} //getPrefs

	/**
		Save application preferences to ISync.properties
	*/
	public void setPrefs(String fileName) throws FileNotFoundException, IOException
	{
		File prefPropfile = new File(fileName);

		if (isyncProps == null)
			return;

		isyncProps.save(new FileOutputStream(prefPropfile), null);
	} // setPrefs

	/**
		Open application log in current directory
	*/
	public void openLog(boolean read)
		throws IOException, FileNotFoundException
	{
		hLog = new File(LOGFILE);
		if (!hLog.exists() || !read)
		{
			//hLog.createNewFile();
			FileOutputStream fs = new FileOutputStream(hLog);
			fs.close();
		}

		if (read)
		{
			if (logPrintStream != null)
				logPrintStream.flush();

			logfileReader =
				new BufferedReader(
					new InputStreamReader(new FileInputStream(hLog)));
		}
		else
		{
			logPrintStream =
					new PrintStream(new FileOutputStream(hLog));

		}
	} // openLog


	/**
		Calls openLog to create a new application log
	*/
	public void rewindLog()
		throws FileNotFoundException, ISyncException, IOException
	{
		openLog(false);

		if (hLog != null) {
			String verStr = getVersionInString();

			logPrintStream.println("IBM SyncClient V" + verStr);
		}
	} // rewindLog

	/**
		Version string
	*/
	public String getVersionInString()
		throws ISyncException
	{
		int version = provider.getVersion();
		int verMajor = (int) (version >> 24);
		int verMinor = (int) ((version >> 16) & 0x000000FF);
		int verMod   = (int) ((version >> 8) & 0x000000FF);
		int verFix   = (int) ((version) & 0x000000FF);

		return verMajor + "." + verMinor + "." + verMod + "." + verFix;
	} // getVersionInString
}
