'/************************************************************************
' 
'  Licensed Materials - Property of IBM
'  Program Name: IBM DB2Everyplace Sync Client GUI
'  (c ) Copyright IBM Corp. 1999-2003. All Rights Reserved.
' 
'  US Government Users Restricted Rights 
'  Use, duplication or disclosure restricted by GSA ADP Schedule
'  Contract with IBM Corp. Licensed Materials - Property of IBM
' 
'  IBM grants you ("Licensee") a non-exclusive, royalty free, license to
'  use Sample Code.  This software is provided as-is and without any
'  warranties, whether EXPRESS OR  IMPLIED, INCLUDING ANY IMPLIED
'  WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR 
'  NON-INFRINGEMENT. IBM AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY 
'  DAMAGES SUFFERED BY LICENSEE THAT RESULT FROM YOUR USE OF THE SOFTWARE.
'  IN NO EVENT WILL IBM OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE,
'  PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, 
'  INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE 
'  THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE 
'  SOFTWARE, EVEN IF IBM HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH 
'  DAMAGES.
' 
' ************************************************************************
' */

Option Explicit On 

Imports System
Imports System.Text
Imports System.Collections

' Step 1:  Import IBM Sync Client namespace
Imports IBM.Data.Sync
#If (Not UNDER_CF) Then
Imports IBM.Data.Sync.DB2e
#Else
Imports IBM.Data.Sync.DB2e.CF
#End If

Public Module ISyncSample

    Class ISyncSample

        Sub New()
        End Sub

        Dim userProps As Hashtable = New Hashtable

        ' Step 2:  Implement an event handler for ISyncEvent
        '          if you are interested in event notification (optional)
        Function EventIssued(ByVal sender As Object, ByVal evt As ISyncEvent) As Integer

            Dim evtType As Integer = evt.GetEventType()
            Dim evtCode As Integer = evt.GetEventCode()

            Select Case (evtType)
                Case ISync.EVTTYPE_INFO, ISync.EVTTYPE_ERROR
                    Console.WriteLine("*********************")
                    Console.WriteLine("SubsSet:    " + evt.GetSubscriptionSetName())
                    Console.WriteLine("Subs:       " + evt.GetSubscriptionName())
                    Console.WriteLine("SubsType:   " + evt.GetSubscriptionType().ToString())
                    Console.WriteLine("Event Type: " + evtType.ToString())
                    Console.WriteLine("Event Code: " + evt.GetEventCode().ToString())
                    Console.WriteLine("Progress:   " + evt.GetSyncProgress().ToString())
                    Console.WriteLine("**********************")
                    Return ISync.RTNCB_DONE

                Case ISync.EVTTYPE_RETRY
                    Console.WriteLine("*********************")
                    Console.WriteLine("Retry...")
                    Console.WriteLine("*********************")
                    Return ISync.RTNCB_REPLY_YES

                Case ISync.EVTTYPE_CONFLICT
                    Console.WriteLine("*********************")
                    Console.WriteLine("Conflict detected!")
                    Console.WriteLine("*********************")
                    Return ISync.RTNCB_DONE

                Case ISync.EVTTYPE_QUERY
                    If (evtCode = ISync.EVT_QUE_LOGIN) Then
                        Dim login() As String = evt.GetEventInfo()
                        Console.WriteLine("Querying login for local encryption.")
                        Console.WriteLine("Target: " + login(0))
                        ' example
                        login(1) = "nurse1"
                        login(2) = "nurse1"
                        Return ISync.RTNCB_DONE
                    ElseIf (evtCode = ISync.EVT_QUE_SUBSCRIPTION_TARGET) Then
                        Dim subsType As Integer = evt.GetSubscriptionType()
                        Dim targetDir() As String = evt.GetEventInfo()

                        If (subsType = ISync.SUBSTYPE_FILE) Then
                            targetDir(0) = "filesubdir"
                        Else
                            targetDir(0) = "database=jdbcsubdir"
                        End If
                        Console.WriteLine("Sample set subs " + targetDir(0))
                        Return ISync.RTNCB_DONE
                    Else
                        Return ISync.RTNCB_DEFAULT
                    End If

                Case Else
                        Exit Select
            End Select

            Return ISync.RTNCB_DEFAULT
        End Function

        Sub RunSample(ByVal host As String, ByVal port As String, ByVal userID As String, ByVal passwrd As String)
            Dim provider As ISyncProvider = Nothing
            Dim service As ISyncService = Nothing
            Dim config As ISyncConfigStore = Nothing
            Dim syncer As ISyncDriver = Nothing
            Dim path As String = "data"         ' a data directory under current dir
            Dim ssArr() As ISyncSubscriptionSet = Nothing
            Dim rc As Integer = 0

            Try

                ' Step 3:  Get an instance DB2eISyncProvider 
                provider = DB2eISyncProvider.GetInstance()

                ' Step 4:  Get an instance of synchronization service from the provider

                ' deprecated
                ' service = provider.CreateSyncService(host, port, userID, passwrd)

                userProps.Add("isync.user", userID)
                userProps.Add("isync.password", passwrd)

		' Set a timeout period (in seconds) for synchronization (optional)
		userProps.Add("isync.timeout", "120")

                Dim uri As String = "http://" + host
                If (port <> Nothing) Then
                    uri += ":" + port
                End If

                service = provider.CreateSyncService(uri, userProps)

                ' Step 5:  Get an instance of the configuration store
                config = service.GetConfigStore(path)

                ' Step 6:  Get an instance of the sync driver to perform synchronization
                syncer = config.GetSyncDriver()

                Console.WriteLine("About: " + syncer.ToString())

                ' Associate the event from ISyncDriver with an user defined event handler
                ' (optional)
                AddHandler syncer.SyncEvent, AddressOf EventIssued

                ' Step 7:  Perform synchronization on all enabled subscription sets
                rc = syncer.Sync()

                Select Case (rc)
                    Case ISync.RTN_SUCCEEDED
                        Console.WriteLine("Synchronization succeeded")
                        Exit Select

                    Case ISync.RTN_CANCELED
                        Console.WriteLine("Synchronization canceled")
                        Exit Select

                    Case Else
                        Console.WriteLine("Synchronization failed")
                        Exit Select
                End Select

                ssArr = config.GetSubscriptionSets()

                Dim i As Integer = 0

                For i = 0 To ssArr.Length - 1

                    Console.Write("Subscription Set: " + ssArr(i).GetName() + " Status: ")

                    Select Case (ssArr(i).GetStatus())

                        Case ISync.STATUS_READY
                            Console.WriteLine("READY")
                            Exit Select

                        Case ISync.STATUS_COMPLETED
                            Console.WriteLine("COMPLETED")
                            Exit Select

                        Case ISync.STATUS_CANCELED
                            Console.WriteLine("CANCELED")
                            Exit Select

                        Case Else
                            Console.WriteLine("FAILED")
                            Exit Select
                    End Select
                Next i

            Catch ie As ISyncException

                Console.WriteLine("Exception code: " + ie.GetCode().ToString())
                Console.WriteLine("Exception message: " + ie.ToString())

            Catch e As Exception

                Console.WriteLine(e.ToString())

            Finally

                Try

                    ' Step 8:  Close and free allocated resources by the syncer object
                    If Not IsNothing(syncer) Then
                        syncer.Close()
                        syncer = Nothing
                    End If

                    'Step 9:  Close and free allocated resources by the config object
                    If Not IsNothing(config) Then
                        config.Close()
                        config = Nothing
                    End If

                    'Step 10:  Close and free allocated resources by the service object
                    If Not IsNothing(service) Then
                        service.Close()
                        service = Nothing
                    End If

                Catch ie2 As ISyncException
                    Console.WriteLine("Exception code: " + ie2.GetCode())
                    Console.WriteLine("Exception message: " + ie2.ToString())
                End Try
            End Try

        End Sub

    End Class

    Sub Main(ByVal CmdArgs() As String)
        Dim host As String = "localhost"
        Dim port As String = "8080"
        Dim userID As String = "nurse1"
        Dim passwrd As String = "nurse1"

        Dim test As ISyncSample = New ISyncSample()

        If (CmdArgs.Length > 0) Then
            If (CmdArgs.Length = 4) Then
                host = CmdArgs(0)
                port = CmdArgs(1)
                userID = CmdArgs(2)
                passwrd = CmdArgs(3)
            Else
                Console.WriteLine("Usage: ISyncSample [host] [port] [userid] [password]")
            End If
        End If

        test.RunSample(host, port, userID, passwrd)
        Console.ReadLine()
    End Sub

End Module