/******************************************************************************/
/*   5724-D04 (C) Copyright IBM Corp. 1999-2003.                              */
/******************************************************************************/

import java.sql.*;
import java.util.Hashtable;

import com.ibm.mobileservices.isync.*;
import com.ibm.mobileservices.isync.event.*;

public class SyncWorker extends Thread implements ISyncListener {

   private DB2Sync midlet;
   private ISyncProvider prov;
   private ISyncService serv;
   private ISyncConfigStore config;
   private ISyncDriver syncer;
   private String eventString;
   private int version;
   private Hashtable ht;
   private String uri = "http://127.0.0.1:8080";

   private String driver = "com.ibm.mobileservices.isync.db2e.sti.DB2eISyncProvider";
   private String protocol = ISyncManager.ISYNC_DB2E_PROTOCOL;
   
   private String user;
   private String password;

   
   public SyncWorker(DB2Sync midlet) {
      this.midlet = midlet;
   }
  
   public void run() {
      sync();
   }
  
   public void cancel() {
   	  try {
   	     syncer.cancelSync();	
   	  }
   	  catch (ISyncException iex) {
   	     midlet.reportSyncStatus("ISyncException Code: " + iex.getCode() );
   	  } 
   	  catch (Exception e) {
         midlet.reportSyncStatus(e.toString());
      }  
   }
  
   private void sync() {
     
      try {
        
         eventString = " ";
         midlet.updateSyncStat1("...");
         midlet.updateSyncStat2("...");
         
         Thread.yield();
         
		 Class.forName(driver);
         prov = ISyncManager.getISyncProvider(protocol);
         
      	 // Create Sync Service setup.  Have to use Hashtable in MIDP over Props
		  ht = new Hashtable();		 
		  ht.put("isync.user", getUser());
		  ht.put("isync.password", getPassword());
	  
         serv = prov.createSyncService(getUri(), ht);
         config = serv.getConfigStore("");
         syncer = config.getSyncDriver();
         
         version = prov.getVersion();
         System.out.println ("Version: " + getVersionAsString(version));
         
         syncer.setSyncListener(this);
         
         int rc = syncer.sync();
         
         switch (rc) {
               
             case ISync.RTN_SUCCEEDED:
                midlet.reportSyncStatus("Synchronization succeeded " + eventString);      
                break;
               
             case ISync.RTN_CANCELED:
                midlet.reportSyncStatus("Synchronization canceled " + eventString);      
               	break;
               
             default: 
                midlet.reportSyncStatus("Synchronization failed " + eventString);        
               	break;
         }        
      }
      catch (ISyncException iex) {
         midlet.reportSyncStatus("ISyncException Code: " + iex.getCode() + ", Event codes: " + eventString);	
      }
      catch (Exception e) {
         midlet.reportSyncStatus(e.toString());
      }
      finally {
         close();   
      }
   }
   
   private void close() {
   
      try {
      		
         if (syncer != null) {
            syncer.close();
            syncer = null;
         }
  
         if (config != null) {
            config.close();
            config = null;
         }
            
         if (serv != null) {
            serv.close();
            serv = null; 
         }
            
         prov = null;	
      }
      catch (ISyncException iex) {
         midlet.reportSyncStatus("ISyncException Code: " + iex.getCode() );	
      }	
      catch (Exception e) {
         midlet.reportSyncStatus(e.toString());
      }
   }
   
   public int eventIssued(ISyncEvent evt) {
			
       int evtType   = evt.getEventType();
       int evtCode   = evt.getEventCode();
       int evtProg   = evt.getSyncProgress();
       String ssName = evt.getSubscriptionSetName();
       
       eventString += evtCode + ":";
              
       switch(evtType) {
          
          // display event status
          case ISync.EVTTYPE_INFO: 
          
             switch (evtCode) {
             
             case ISync.EVT_INF_SYNCING_SUBS: 
             {	
                midlet.updateSyncStat1("Synchronizing " + ssName);
                midlet.updateSyncStat2(" ");
                break;
             }
             case ISync.EVT_INF_SYNC_STARTED: 
             { 
                midlet.updateSyncStat1("Synchronization started");	
                midlet.updateSyncStat2(" ");
                break;
             }
             case ISync.EVT_INF_PREP_MSG:
             {
                midlet.updateSyncStat2("Preparing message...");
                break;	
             } 
             case ISync.EVT_INF_SEND_MSG:
             {
                midlet.updateSyncStat2("Sending message...");
                break;	
             }
             case ISync.EVT_INF_WAIT_MSG: 
             {
             	midlet.updateSyncStat2("Awaiting server reply...");
             	break;
             }
             case ISync.EVT_INF_APPLY_MSG: 
             {
                midlet.updateSyncStat2("Applying server message...");	
                break;
             }     
             case ISync.EVT_INF_SYNC_CANCELED: 
             {
			    midlet.updateSyncStat1("Synchronization canceled");
			    midlet.updateSyncStat2(" ");
			    break;    
             }
			 case ISync.EVT_INF_SYNC_SUCCEEDED: 
			 {	
			    midlet.updateSyncStat1("Synchronization succeeded");
			    midlet.updateSyncStat2(" ");  
			    break;	
			 }
             case ISync.EVT_INF_SYNC_FAILED: 
             {
			    midlet.updateSyncStat1("Synchronization failed");
			    midlet.updateSyncStat2(" ");
			    break;		
             }
             default:
                break;
          }
 		     
 		  return ISync.RTNCB_DONE;
 		        
          case ISync.EVTTYPE_ERROR:
		     midlet.updateSyncStat2("Error: " + evtCode);
             return ISync.RTNCB_DONE;
          
          case ISync.EVTTYPE_RETRY:
             midlet.updateSyncStat2("Retry: " + evtCode);
             return ISync.RTNCB_REPLY_YES;
          
          case ISync.EVTTYPE_CONFLICT:
             
             if (evtCode == ISync.EVT_CFT_REJECT) {
             	 	
             	ConflictReader cr = (ConflictReader)evt.getEventInfo();
             	 
             	processRejectedData(cr);
                  
             }   
             		
             return ISync.RTNCB_DONE;

          case ISync.EVTTYPE_QUERY:
             //local data encryption case
             if (evtCode == ISync.EVT_QUE_LOGIN)
             {
                 String login[] = (String [])evt.getEventInfo();
                 midlet.updateSyncStat1("Querying login for local encryption.");
                 midlet.updateSyncStat2("Subscription: " + login[0]);
                        
                 //example
                 login[1] = new String("username");
                 login[2] = new String("passwd");
	            	
                 return ISync.RTNCB_DONE;
              }
              else
                 return ISync.RTNCB_DEFAULT;

          // ignore other event types
          default:
             break;
		}
      
        // let sync engine take default action
      	return ISync.RTNCB_DEFAULT ;
          
   	}  // end of eventIssued() 
   	
   	
   // process INSERT and UPDATE operation only	
   private void processRejectedData(ConflictReader cr) {
   	  int op;
      ResultSet pkrs = null;
      ResultSet rs = null;
      String cmsg;

      if (cr == null) return;

      try { 
          
         op = cr.getOperation();
          
         cmsg = "Conflicts detected in Table: " + cr.getTableName();
         
         System.out.println (cmsg);
         
         midlet.updateSyncStat2(cmsg);

         if (op != ISync.OP_DELETE) {
         
            rs = cr.getRejectedRows();
         
            processResultSet(rs);
         }
         else {
         	System.out.println ("A deleted row was rejected");
         }
         
      }
      catch (ISyncException iex) {
      	
      	 iex.printStackTrace();
      	 
         System.out.println (iex.getCode());
	     
      }
      catch (SQLException sqlex) {
      	
         sqlex.printStackTrace();
               
         while(sqlex != null) {
	        System.out.println("SQLERROR: \n" + sqlex.getErrorCode() + 
			   			", SQLState: " + sqlex.getSQLState() + 
			   			", Message: " + sqlex.getMessage() +
			   			", Vendor: " + sqlex.getErrorCode() );
		    sqlex = sqlex.getNextException();
         }         
      }

      System.out.println("");
   }


   private void processResultSet(ResultSet rs) throws SQLException {
    	
      ResultSetMetaData rsmd = rs.getMetaData();
      int numCols = rsmd.getColumnCount();
      String name;
      int type;
      int i = 0;
         
      while (rs.next()) {
   
         i = 1;

         while (i <= numCols) {
            	
            name = rsmd.getColumnName(i);
               
            type = rsmd.getColumnType(i);

            System.out.print (name + ":");

            if (type == Types.LONGVARBINARY) {
               
               byte[] data = rs.getBytes(i);
               
               if (data == null) {
                  System.out.print(" ( " + type + " ) ");
                  System.out.print(" [ NULL ] ");
               }
               else {
                  System.out.print(" ( " + type + " ) ");
                  System.out.println(" [ BLOB:" + data.length + " ] ");
               }
            }
            else {
               	
               String s = rs.getString(i);
                
               if (s == null) {
                  System.out.print(" ( " + type + " ) ");
                  System.out.print(" [ NULL ] ");
               }
               else {
                  System.out.print(" ( " + type + " ) ");
                  System.out.print(" [ " + s + " ] ");
               }
            }

            i++;
         }                
      }

      rs.close();
    }
    
    
   	private String getVersionAsString(int version) {
        
        int verMajor = (int) (version >> 24);
        int verMinor = (int) ((version >> 16) & 0x000000FF);
        int verMod   = (int) ((version >> 8) & 0x000000FF);
        int verFix   = (int) ((version) & 0x000000FF);	 

        return verMajor + "." + verMinor + "." + verMod + "." + verFix;
    }


/**
 * Returns the password.
 * @return String
 */
public String getPassword() {
	return password;
}

/**
 * Returns the user.
 * @return String
 */
public String getUser() {
	return user;
}

/**
 * Sets the password.
 * @param password The password to set
 */
public void setPassword(String password) {
	this.password = password;
}

/**
 * Sets the user.
 * @param user The user to set
 */
public void setUser(String user) {
	this.user = user;
}

/**
 * Returns the uri.
 * @return String
 */
public String getUri() {
	return uri;
}

/**
 * Sets the uri.
 * @param uri The uri to set
 */
public void setUri(String uri) {
	this.uri = uri;
}

}

